/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");

dojo.require("com.ibm.asq.common.web.client.internal.option");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() { // module container 

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;

com.ibm.asq.common.web.client.internal.CommonPreferenceClient = {

    USER_UI_CUSTOMISE_ACTIONBAR_CATEGORY: {
        key: "USER_UI_CUSTOMISE_ACTIONBAR_CATEGORY",
        value: "actionBarVertical" 
        },
    USER_UI_CUSTOMISE_ACTIONBAR_EXPANSION_CATEGORY: {
        key: "USER_UI_CUSTOMISE_ACTIONBAR_EXPANSION_CATEGORY",
        value: "expanded"
        },
    USER_UI_CUSTOMISE_ACTIONBAR_TOC_CATEGORY: {
        key: "USER_UI_CUSTOMISE_ACTIONBAR_TOC_CATEGORY",
        value: "TOCSingleSection"
        },
    USER_UI_CUSTOMISE_ACTIONBAR_RELATEDINFO_CATEGORY: {
        key: "USER_UI_CUSTOMISE_ACTIONBAR_RELATEDINFO_CATEGORY",
        value: true
        },
    USER_UI_CUSTOMISE_ACTIONBAR_MAXTABLESIZE_CATEGORY: {
        key: "USER_UI_CUSTOMISE_ACTIONBAR_MAXTABLESIZE_CATEGORY",
        value: 100
        },
    USER_UI_CUSTOMISE_ACTIONBAR_DATEFORMAT_CATEGORY: {
        key: "USER_UI_CUSTOMISE_ACTIONBAR_DATEFORMAT_CATEGORY",
        value: "dateRelativeStyle"
        },
	USER_UI_CUSTOMISE_ACTIONBAR_ADAPTERSELECTION_CATEGORY: {
		key: "USER_UI_CUSTOMISE_ACTIONBAR_ADAPTERSELECTION_CATEGORY",
        value: true
		},
	USER_UI_CUSTOMISE_ACTIONBAR_WORDWRAPCHOICE_CATEGORY: {
		key: "USER_UI_CUSTOMISE_ACTIONBAR_WORDWRAPCHOICE_CATEGORY",
        value: true
		},
    USER_UI_CUSTOMISE_QUERYLISTS_AUTORUN_MYQUERY_CATEGORY: {
        key: "USER_UI_CUSTOMISE_QUERYLISTS_AUTORUN_MYQUERY_CATEGORY",
        value: true
        },
    USER_UI_CUSTOMISE_QUERYLISTS_FILTER_INITOPEN_CATEGORY: {
        key: "USER_UI_CUSTOMISE_QUERYLISTS_FILTER_INITOPEN_CATEGORY",
        value: false
        },

    _getDefaultPreferences: function()
    {
        var preferences = [
            this.USER_UI_CUSTOMISE_ACTIONBAR_CATEGORY, 
            this.USER_UI_CUSTOMISE_ACTIONBAR_EXPANSION_CATEGORY, 
            this.USER_UI_CUSTOMISE_ACTIONBAR_TOC_CATEGORY, 
            this.USER_UI_CUSTOMISE_ACTIONBAR_RELATEDINFO_CATEGORY,
            this.USER_UI_CUSTOMISE_ACTIONBAR_DATEFORMAT_CATEGORY,
			this.USER_UI_CUSTOMISE_ACTIONBAR_ADAPTERSELECTION_CATEGORY,
			this.USER_UI_CUSTOMISE_ACTIONBAR_WORDWRAPCHOICE_CATEGORY,
            this.USER_UI_CUSTOMISE_ACTIONBAR_MAXTABLESIZE_CATEGORY,
            this.USER_UI_CUSTOMISE_QUERYLISTS_AUTORUN_MYQUERY_CATEGORY,
            this.USER_UI_CUSTOMISE_QUERYLISTS_FILTER_INITOPEN_CATEGORY
        ];
        return preferences;
    },
     
    fetchAndStoreUserPreferences: function(handler)
    {
        var self = this;
        var internalHandlerObj = {
            success: function(response) {
                var prefs = self._getDefaultPreferences();
                if (response && response.value && response.itemId) {
                    self.storedPreferences = dojo.fromJson(response.value);
					self.storedPreferencesItemId = response.itemId;
                    prefs = self.storedPreferences;
                }
                handler.doCallback(prefs);
            },
            failure: function(e) {
                handler.doCallback(self._getDefaultPreferences());
            }
        }
        var responseHandler = new ServiceResponseHandler(internalHandlerObj, "success", "failure");
        var option = new com.ibm.asq.common.web.client.internal.option();
        option.getAllPreferencesForUser(responseHandler, com.ibm.team.process.web.projectArea.itemId);
    },
	
	saveUserPreferences: function(userPreference, handler)
	{
		var self = this;
		var internalHandlerObj = {
            success: function(property){
                self.fetchAndStoreUserPreferences(handler);
            },
            error: function(e){
                console.debug(e);
            }
        }
		
		var responseHandler = new ServiceResponseHandler(internalHandlerObj, "success", "error");
		var option = new com.ibm.asq.common.web.client.internal.option();
		
		if(this.storedPreferencesItemId){
			option.updateAllPreferencesForUser(responseHandler, com.ibm.team.process.web.projectArea.itemId, this.storedPreferencesItemId, dojo.toJson(userPreference));
		}else{
			option.createAllPreferencesForUser(responseHandler, com.ibm.team.process.web.projectArea.itemId, dojo.toJson(userPreference));
		}
	},
    
    getStoredUserPreferences: function(/* Optional */ category)
    {
        var storedPrefs = (this.storedPreferences) ? this.storedPreferences : this._getDefaultPreferences();
        
        if(category) {
            for(var i = 0; i < storedPrefs.length; i++) {
                if(storedPrefs[i].key == category) {
                    return storedPrefs[i];
                }
            }
            
        }
        return storedPrefs;
    },
    
    // Replaces existing preference value, or add it to the pref store.
    _replaceOrAddNewPreference: function(prefs, newPref)
    {
        var exist = null;

        for(var i = 0; i < prefs.length; i++) {
            if(prefs[i].key == newPref.key) {
                exist = prefs[i];
                break;
            }
        }
        if(exist) {
            exist.value = newPref.value;
        }
        else {
            prefs.push(newPref);                    
        }
    },
    
    // Save a new set of preferences values in addition to the currently stored preferences.
    // If the preferences (id'ed by the key) already exist in our preference store, the new
    // values will replace the old values. If the preferences don't exist, they'll be appended to
    // the preference store.
    //
    mergeAndSaveUserPreferences: function(/* array or single object */newPreferences, handler)
    {
        // need to get the current stored user preferences, it could be the default,
        // which would be a new array every time you get it.
        var allPrefs = this.getStoredUserPreferences();
        
        if(dojo.isArray(newPreferences)) {
            for(var i = 0; i < newPreferences.length; i++) {
                this._replaceOrAddNewPreference(allPrefs, newPreferences[i]);
            }
        }
        else {
            this._replaceOrAddNewPreference(allPrefs, newPreferences);
        }
        this.saveUserPreferences(allPrefs, handler);
    },
    
    // Summary:
    //   Convenient method to returns whether the current user's date format preference should be 
    //   relative style. Relative style shows the time difference from the present when possible, like
    //   "1 minute ago", etc. If false, all date format should be shown as the absolute date format.
    //
    getIsDateRelativeStyle: function()
    {
        var showDatePref = this.getStoredUserPreferences(this.USER_UI_CUSTOMISE_ACTIONBAR_DATEFORMAT_CATEGORY.key);
        return showDatePref.value == "dateRelativeStyle";
    },
    
    // Summary:
    //    Returns the date formatter expected by dojo.date.locale.format method. 
    // 
    getDateFormatStyle: function()
    {
        return {selector:"date", formatLength:this.getDateFormatLength()};
    },

    getDateFormatLength: function()
    {
        return "medium";
    },
	
	//Summary:
	//Returns the current user preferences to show adapter selection dialog during execution or not
	
	getAutomaticAdapterSelectionPreference: function(){
        var autoAdapSelPref = this.getStoredUserPreferences(this.USER_UI_CUSTOMISE_ACTIONBAR_ADAPTERSELECTION_CATEGORY.key);
		return autoAdapSelPref.value ;
    },
	
	//Summary:
	//Returns if user prefers to warp the text in the TER listing or not. Hot replaceable
	
	getIsWordWrapEnabled: function(){
        var wordWrapPref = this.getStoredUserPreferences(this.USER_UI_CUSTOMISE_ACTIONBAR_WORDWRAPCHOICE_CATEGORY.key);
		return wordWrapPref.value ;
    }
    
};

})();
