/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.adapter.rtc.web.client.internal.TeamConcertConnectionClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.adapter.rtc.web.ui.internal.RTCAdapterUIUtil");

(function() {

var ServiceRequest					= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient				= com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Status							= com.ibm.team.repository.web.foundation.Status;
var getCurrentPA            		= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var RTCAdapterUIUtil 				= com.ibm.rqm.adapter.rtc.web.ui.internal.RTCAdapterUIUtil;

var SERVICE_URI						= "com.ibm.rqm.adapter.rtc.service.internal.rest.IConnectionRestService";

com.ibm.rqm.adapter.rtc.web.client.internal.TeamConcertConnectionClient= {

	// ---- api ------------------------------------------------------------------------------------------------------------

	getDefaultTeamConcertConnection: function(type, responceCallback) {
		switch(type){
		case "requirement":
			return this._invokeTeamConcertConnectionService("getDefaultTeamConcertRequirementsConnection", responceCallback);
		case "defect":
			return this._invokeTeamConcertConnectionService("getDefaultTeamConcertDefectsConnection", responceCallback);
		}
	},

	postDefaultTeamConcertConnection: function(type, connectionParams, responceCallback) {
		var callback = function(status, response){
			if(!status.isOK()){
				status = new Status(Status.ERROR, RTCAdapterUIUtil.getMessage("msgRQMConnectionProblemToRTC"), status);
			} 
			responceCallback(status, response);
		};
		switch(type){
		case "requirement":
			this._invokeTeamConcertConnectionService("postDefaultTeamConcertRequirementsConnection", callback, connectionParams);
			break;
		case "defect":
			this._invokeTeamConcertConnectionService("postDefaultTeamConcertDefectsConnection", callback, connectionParams);
			break;
		}
	},
	
	postDeleteDefaultTeamConcertConnection: function(type, connectionParams, responceCallback) {
		switch(type){
		case "defect":
			this._invokeTeamConcertConnectionService("postDeleteDefaultTeamConcertConnection", responceCallback, connectionParams);
			break;
		}
	},

	postDraftLink: function(linkParams, responceCallback) {
		this._invokeTeamConcertConnectionService("postDraftLink", responceCallback, linkParams);
	},
	
	postBlockDefect: function(postParams, responceCallback) {
		this._invokeTeamConcertConnectionService("postBlockDefect", responceCallback, postParams);
	},
	
	postAssociatedWorkItems: function(workItemIdentification, responceCallback) {
		this._invokeTeamConcertConnectionService("postAssociatedWorkItems", responceCallback, workItemIdentification);
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	_invokeTeamConcertConnectionService: function(method, serviceCallback, appArgs) {
		// scope each service call to current project area
		if (!appArgs) {
			var appArgs = {};
		}
		appArgs.rqmProjectAreaItemId = getCurrentPA().itemId;
		var serviceRequest= new ServiceRequest(SERVICE_URI, method, appArgs);
		var serviceResponseHandler= new ServiceResponseHandler({
			success: function(result) {
				serviceCallback(Status.OK_STATUS, result);
			},
			failure: function(error) {
				serviceCallback(new Status(Status.ERROR, error.message, error), null);
			}
		}, "success", "failure");

		TeamServerClient.invokeService(serviceRequest, serviceResponseHandler);
	},

	__sentinel: null // terminates this class definition
};

})();
