/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.WorkItemUtils");

(function(){

    var SECTION_ATTRIBUTE_ID = "com.ibm.team.workitem.workItemType.authoring.section";
	var SECTION_ATTRIBUTE = "section";
	
	var TOP_LEVEL_ATTRIBUTES = ["summary", "description", "internalState", "internalPriority", "owner", "creator", "dueDate", "workItemType", "category", SECTION_ATTRIBUTE_ID];
    var TOP_LEVEL_ATTRIBUTES_MAPPING = ["summary", "description", "state", "priority", "owner", "creator", "dueDate", "workItemType", "category", SECTION_ATTRIBUTE];
    
	var WORKITEM_DTO_QNAME = "com.ibm.team.workitem.rest.dto:WorkItemDTO";
	var ARTIFACT_WORKITEM_DTO_QNAME = "com.ibm.rqm.planning.rest.dto:TestArtifactWorkItemDTO";
	var WORKITEM_SUMMARY_DTO_QNAME = "com.ibm.team.workitem.rest.dto:WorkItemSummaryDTO";
	var ARTIFACT_WORKITEM_SUMMARY_DTO_QNAME = "com.ibm.rqm.planning.rest.dto:TestArtifactWorkItemSummaryDTO";
    
	
    com.ibm.rqm.planning.web.client.internal.WorkItemUtils = {
        TYPE_FULL: "full",          //$NON-NLS$
		TYPE_SUMMARY: "summary",    //$NON-NLS$
		  		  
		fixupDTO: function(dto) {
			var type = this.getType(dto);
            switch (type) {
				case this.TYPE_FULL:
				    return this.fixupWorkItemDTO(dto);

			    case this.TYPE_SUMMARY:
				    return this.fixupWorkItemSummaryDTO(dto);
			}
			return null;
		},
		
        fixupWorkItemDTO: function(/* TestArtifactWorkItemDTO */workItemDTO){
            var attributeMap = this.buildMap(workItemDTO.attributes)
            workItemDTO.attributes = attributeMap;
            workItemDTO = this.promoteAttributes(workItemDTO);
            return workItemDTO;
        },

        fixupWorkItemSummaryDTO: function(/* TestArtifactWorkItemSummaryDTO */dto){
            if (dto && dto.workItemSummary) {
                //promote attributes of the workitem summary to the top level 
				for (var prop in dto.workItemSummary) {
                    dto[prop] = dto.workItemSummary[prop];
                }		
				dto.workItemSummary = null;
			}		
			return dto;
        },
		        
        //fixup each workitem for one artifact
        fixupArtifactWorkItems: function(testArtifactDTO){
            return this.fixupArtifactsWorkItems([testArtifactDTO])[0];
        },
        
        //fixup each workitem for each artifact
        fixupArtifactsWorkItems: function(testArtifactDTOs){
			if (!dojo.isArray(testArtifactDTOs)) {
				return this.fixupArtifactWorkItems(testArtifactDTOs);
			}
            var dtos = testArtifactDTOs ? testArtifactDTOs : [];
            for (var i = 0; i < dtos.length; i++) {
                var testArtifactDTO = dtos[i];
                if (testArtifactDTO != null && testArtifactDTO.workItems != null && testArtifactDTO.workItems.length > 0) {
                    for (var j = 0; j < testArtifactDTO.workItems.length; j++) {
                        var fixedUpWI = null;
						//handle contained and standalone WorkItem dtos
						if (dojo.isObject(testArtifactDTO.workItems[j].workItem)) {
							fixedUpWI = this.fixupDTO(testArtifactDTO.workItems[j].workItem);
						} else {
							fixedUpWI = this.fixupDTO(testArtifactDTO.workItems[j]);
						}
                        testArtifactDTO.workItems[j] = fixedUpWI;
                    }
                }
                dtos[i] = testArtifactDTO;
            }
            return dtos;
        },
				
		getType: function(dto) {
			if (dto) {
				switch(dto._eQualifiedClassName) {
					case WORKITEM_DTO_QNAME:
					case ARTIFACT_WORKITEM_DTO_QNAME:
                        return this.TYPE_FULL;
					case WORKITEM_SUMMARY_DTO_QNAME:
					case ARTIFACT_WORKITEM_SUMMARY_DTO_QNAME:
                        return this.TYPE_SUMMARY;					   			     
				}
			}
			return null;	
		},
		
		
		
        buildMap: function(array, keyName, valueName){        
            if (!array) {
                return {};
            }            
            if (!keyName) {
                keyName = "key";
            }
            if (!valueName) {
                valueName = "value";
            }
            var map = new Object();
            for (var i in array) {
                var element = array[i];
                map[element[keyName]] = element[valueName];
            }
            return map;
        },
        
        promoteAttributes: function(/* WorkItemDTO */workItem){        
            if (!workItem) {
                return {};
            }
            //directly mapped top level attributes
            for (var i = 0; i < TOP_LEVEL_ATTRIBUTES.length; i++) {
                if (workItem.attributes[TOP_LEVEL_ATTRIBUTES[i]]) {
					workItem[TOP_LEVEL_ATTRIBUTES_MAPPING[i]] = workItem.attributes[TOP_LEVEL_ATTRIBUTES[i]];
				}
            }
            //converted top level attributes
            workItem.id = workItem.attributes.id ? workItem.attributes.id.id : "-1";
            if (workItem.owner) {
                workItem.owner.id = workItem.owner.itemId;
            }
            return workItem;
        },
		
        //convert a full WorkItemDTO to a WorkItemSummaryDTO
        toSummaryDTO: function(/*WorkItemDTO*/ dto) {
            if (this.getType(dto) == this.TYPE_SUMMARY) {
                return dto;
            }
            var summaryDTO = {};
			summaryDTO._eQualifiedClassName = WORKITEM_SUMMARY_DTO_QNAME;
            summaryDTO.id = dto.id ? dto.id : dto.attributes.id.id;
            summaryDTO.workItemItemId = dto.itemId;
			summaryDTO.modified = dto.attributes.modifiedDate;
            summaryDTO.summary = dto.summary ? dto.summary.content : null;
            summaryDTO.description = dto.description ? dto.description.content : null;
			if (dto.section) {
				summaryDTO.section = dto.section;
			}
            if (dto.category) {
                summaryDTO.categoryItemId = dto.category.id;
                summaryDTO.categoryName = dto.category.label;
            }   
            if (dto.creator) {
                summaryDTO.creatorItemId = dto.creator.itemId;
                summaryDTO.creatorName = dto.creator.name;
            }   
            if (dto.owner) {
                summaryDTO.ownerItemId = dto.owner.itemId;
                summaryDTO.ownerName = dto.owner.name;
            }   
            if (dto.state) {
                summaryDTO.state = dto.state.id;
                summaryDTO.stateName = dto.state.label;
				summaryDTO.stateIconURL = dto.state.iconUrl;
            }            
            if (dto.workItemType) {
                summaryDTO.type2 = dto.workItemType.id;
                summaryDTO.typeName = dto.workItemType.label;
                summaryDTO.typeIconURL = dto.workItemType.iconUrl;
            }   
            if (dto.priority) {
                summaryDTO.priorityValue = dto.priority.id;
                summaryDTO.priorityName = dto.priority.label;
                summaryDTO.priorityIconURL = dto.priority.iconUrl;
            }			         
			return summaryDTO;
        }	  
    };
    
})();
