/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemUtils");

(function() { //module container

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var WorkItemUtils          = com.ibm.rqm.planning.web.client.internal.WorkItemUtils;


com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient = {

	TEST_GROUP_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.ITestCaseGroupRestService",

	_invokeService: function(responseHandler, method, appArgs) {
		var actualArgs= {};
		if (appArgs) {
			var jsonArgs= dojo.toJson(appArgs);
			actualArgs.jsonString= jsonArgs;
			if(appArgs.newTestGroup) {
				actualArgs.newTestGroup = appArgs.newTestGroup;
			}
			if (appArgs.newItem) {
				actualArgs.newTestGroup = appArgs.newItem;
			}			
			if(appArgs.id) {
				actualArgs.id = appArgs.id;
			}
			if(appArgs.name) {
				actualArgs.name = appArgs.name;
			}
			if(appArgs.ownerId) {
				actualArgs.ownerId = appArgs.ownerId;
			}			
			if (appArgs.workItemData) {
				actualArgs.workItemJsonString = dojo.toJson(appArgs.workItemData);
			} /*else if (appArgs.ownerId || appArgs.workItemId) {
				var workItem = {
					owner: appArgs.ownerId != null ? appArgs.ownerId : null,
					workItemItemId: appArgs.workItemId != null ? appArgs.workItemId : null
				};
				actualArgs.workItemJsonString = dojo.toJson(workItem);
			}*/										
			if(appArgs.description) {
				actualArgs.description = appArgs.description;
			}
			if(appArgs.testCaseGroupsAdded) {
				actualArgs.testCaseGroupsAdded = appArgs.testCaseGroupsAdded;
			}			
			if(appArgs.testCaseGroupsRemoved) {
				actualArgs.testCaseGroupsRemoved = appArgs.testCaseGroupsRemoved;
			}			
			if(appArgs.testCasesAdded) {
				actualArgs.testCasesAdded = appArgs.testCasesAdded;
			}			
			if(appArgs.testCasesRemoved) {
				actualArgs.testCasesRemoved = appArgs.testCasesRemoved;
			}
			if(appArgs.resolveWorkItems) {
				actualArgs.resolveWorkItems = appArgs.resolveWorkItems;
			}						
			if(appArgs.uuid) {
				actualArgs.uuid = appArgs.uuid;
			}						
			if(appArgs.uid) {
				actualArgs.uid = appArgs.uid;
			}						
			if(appArgs.revision) {
				actualArgs.revision = appArgs.revision;
			}						
		}
		var serviceRequest= new ServiceRequest(this.TEST_GROUP_SERVICE_URI, method, actualArgs);
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},

	getTestGroups: function(serviceResponseHandler) {
		this._invokeService(serviceResponseHandler, "getTestCaseGroupsDTO", { });
	},
	
	getTestGroup: function(serviceResponseHandler, groupId) {
		var actualHandler = this.getInternalFixupHandler(serviceResponseHandler);
		this._invokeService(actualHandler, "getTestCaseGroupsDTO", {id: groupId, resolveWorkItems: true});
	},

	getTestGroupRevision: function(serviceResponseHandler, groupId, aRevision) {
		var actualHandler = this.getInternalFixupHandler(serviceResponseHandler);
		this._invokeService(actualHandler, "getTestCaseGroupsDTO", {uid: groupId, resolveWorkItems: true, revision: aRevision});
	},
	
	removeTestGroup: function(serviceResponseHandler, removeArgs) {
		this._invokeService(serviceResponseHandler, "postDeleteTestCaseGroup", removeArgs);
	},
	
	saveTestGroup: function(serviceResponseHandler, groupObj) {
		var actualHandler = this.getInternalFixupHandler(serviceResponseHandler);
		this._invokeService(actualHandler, "postTestCaseGroupDTO", groupObj);
	},
	
	getSnapshots: function(serviceResponseHandler, args)
	{
		if ( !args ) args = { };
		this._invokeService(serviceResponseHandler, "getSnapshots", args);
	},		
	
   getInternalFixupHandler: function(responseHandler){
      var internalHandler = {
         success: function(testGroupDTOs){
            //fixup each workitem for each plan
            WorkItemUtils.fixupArtifactWorkItems(testGroupDTOs);
            responseHandler.doCallback(testGroupDTOs);
         },
         failure: function(error){
            responseHandler.doErrorCallback(error);
         }
      };
      var handler = new ServiceResponseHandler(internalHandler, "success", "failure");
      return handler;
   }	
};

})();
