/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*
 *   This class is specific to options.
 */
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.provide("com.ibm.asq.common.web.client.internal.option");
dojo.declare("com.ibm.asq.common.web.client.internal.option", null, 
        
        
    {
        constructor: function(){
            this.setServiceInterface('com.ibm.rational.test.lm.common.IOptionRestService');
        },
        
        setServiceInterface: function(serviceInterface){
            this.serviceInterface = serviceInterface;
        },
        
        _invokeService : function (responseHandler, method, args) {
            var myServiceRequest=
                new com.ibm.team.repository.web.transport.ServiceRequest(
                    this.serviceInterface,
                    method, 
                    args);
                            
            com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
                myServiceRequest, responseHandler );
        
        },
        
		//
		// Get the preferences for a user
		// Parameters: responseHandler
		//             projectAreaId - UUID of the project
		getAllPreferencesForUser: function( responseHandler, projectAreaId ){
			var attributes = { "projectAreaItemId": projectAreaId };
		    return this._invokeService(responseHandler, "getUserOption", attributes);
		},
		
        //
        // Create the preferences for a user
        // Parameters: responseHandler
		//             optionId - id of the option to update, null if this is a new option
        //             projectAreaId - UUID of the project
		//             jsonString - formatted string containing the user's preferences
		createAllPreferencesForUser: function( responseHandler, projectAreaId, jsonString ){
            if(projectAreaId == null){
                throw new Error("Must provide a projectAreaId.");
			}
			var attributes = { "projectAreaItemId": projectAreaId,
							   "value": jsonString };
            return this._invokeService(responseHandler, "postCreateUserOption", attributes);
		},
		
        //
        // Update the preferences for a user
        // Parameters: responseHandler
        //             optionId - id of the option to update, null if this is a new option
        //             projectAreaId - UUID of the project
        //             jsonString - formatted string containing the user's preferences
        updateAllPreferencesForUser: function( responseHandler, projectAreaId, optionId, jsonString ){
            if(projectAreaId == null){
                throw new Error("Must provide a projectAreaId.");
            }
            if(optionId == null){
                throw new Error("Must provide an optionId.");
            }
            var attributes = { "projectAreaItemId": projectAreaId,
			                   "optionUuid": optionId };
            if(jsonString != null){
                attributes.value = jsonString;
            }
            return this._invokeService(responseHandler, "postUpdateUserOption", attributes);
        },
        
        //
        // Get preferences for a project
        // Parameters: responseHandler
        //             projectAreaId - UUID of the project
        //             categoryName - Name of the category of options to get
		getProjectPreferences: function( responseHandler, projectAreaId, categoryName ){
            var attributes = { "projectAreaItemId": projectAreaId,
                               "category": categoryName };
            return this._invokeService(responseHandler, "getProjectOption", attributes);
		},
		
        //
        // Create preferences for a project
        // Parameters: responseHandler
        //             projectAreaId - UUID of the project
        //             categoryName - Name of the category of options to set
        //             jsonString - formatted string containing the preferences
		createProjectPreferences: function( responseHandler, projectAreaId, categoryName, jsonString ){
            if(projectAreaId == null){
                throw new Error("Must provide a projectAreaId.");
            }
            if(categoryName == null){
                throw new Error("Must provide a categoryName.");
            }
            var attributes = { "projectAreaItemId": projectAreaId,
                               "category": categoryName,
                               "value": jsonString };
            return this._invokeService(responseHandler, "postCreateProjectOption", attributes);
		},
		
        //
        // Update the preferences for a project
        // Parameters: responseHandler
        //             optionId - id of the option to update, null if this is a new option
        //             projectAreaId - UUID of the project
        //             jsonString - formatted string containing the user's preferences
        updateProjectPreferences: function(responseHandler, projectAreaId, optionId, jsonString){
			if (projectAreaId == null) {
				throw new Error("Must provide a projectAreaId.");
			}
			if (optionId == null) {
				throw new Error("Must provide an optionId.");
			}
			var attributes = {
				"projectAreaItemId": projectAreaId,
				"optionUuid": optionId };
			if(jsonString != null){
				attributes.value = jsonString;
			}
			
			return this._invokeService(responseHandler, "postUpdateProjectOption", attributes);
		},
        
        //
        // Get global preferences
        // Parameters: responseHandler
        //             categoryName - Name of the category of options to get
        getGlobalPreferences: function( responseHandler, categoryName ){
            var attributes = { "category": categoryName };
            return this._invokeService(responseHandler, "getGlobalOption", attributes);
        },
        
        //
        // Create global preferences
        // Parameters: responseHandler
        //             categoryName - Name of the category of options to set
        //             jsonString - formatted string containing the preferences
        createGlobalPreferences: function( responseHandler, categoryName, jsonString ){
            if(categoryName == null){
                throw new Error("Must provide a categoryName.");
            }
            var attributes = { "category": categoryName,
                               "value": jsonString };
            return this._invokeService(responseHandler, "postCreateGlobalOption", attributes);
        },
        
        //
        // Update global preferences
        // Parameters: responseHandler
        //             optionId - id of the option to update
        //             jsonString - formatted string containing the preferences
        updateGlobalPreferences: function(responseHandler, optionId, jsonString){
            if (optionId == null) {
                throw new Error("Must provide an optionId.");
            }
            var attributes = { "optionUuid": optionId };
            if(jsonString != null){
                attributes.value = jsonString;
            }
            
            return this._invokeService(responseHandler, "postUpdateGlobalOption", attributes);
        }
    }
);
