/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.rqm.adapter.rtc.web.provider.internal.TeamConcertDefectsProvider");

dojo.require("com.ibm.rqm.defects.web.spi.DefectsProvider");

dojo.require("com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient");
dojo.require("com.ibm.rqm.adapter.rtc.web.client.internal.TeamConcertConnectionClient");
dojo.require("com.ibm.rqm.adapter.rtc.web.ui.internal.config.TeamConcertWizard");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.foundation.Status");

dojo.require("com.ibm.rqm.adapter.rtc.web.client.internal.ExternalProjectAreaDescription");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.rqm.adapter.rtc.web.ui.internal.RTCAdapterUIUtil");

(function() {

var RemoteTeamConcertClient			= com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient;
var TeamConcertConnectionClient		= com.ibm.rqm.adapter.rtc.web.client.internal.TeamConcertConnectionClient;
var TeamConcertWizard				= com.ibm.rqm.adapter.rtc.web.ui.internal.config.TeamConcertWizard;

var ServiceResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Status							= com.ibm.team.repository.web.foundation.Status;

var ExternalProjectAreaDescription	= com.ibm.rqm.adapter.rtc.web.client.internal.ExternalProjectAreaDescription;
var getCurrentProjectArea			= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var RTCAdapterUIUtil 				= com.ibm.rqm.adapter.rtc.web.ui.internal.RTCAdapterUIUtil;

dojo.declare("com.ibm.rqm.adapter.rtc.web.provider.internal.TeamConcertDefectsProvider", 
			 [com.ibm.rqm.defects.web.spi.DefectsProvider], {

	_teamServerConnection: null,
	_externalProjectArea: null,

	constructor: function(id, name) {
	},

	// ---- DefectsManager implemention ------------------------------------------------------------------------------------

	getConnectionName: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.name : null;
	},

	getConnectionDetails: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.repositoryUrl : null;
	},

	getRepositoryUrl: function() {
		return this.getConnectionDetails();
	},

	getRepositoryDescription: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.description : null;
	},

	getProjectAreaName: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.projectAreaName : null;
	},
	
	clearConnectionDetails: function(runAfterClear) {
		TeamConcertConnectionClient.postDeleteDefaultTeamConcertConnection("defect", {}, dojo.hitch(this, function(status, response) {
			var resultStatus= status;
			try {
				if (status.isOK()) {
					runAfterClear();
				} else {
					console.debug(status._message);
					resultStatus= new Status(Status.ERROR, RTCAdapterUIUtil.getMessage("labelConfigurationNotCleared"));
				}
			} finally {
				runAfterClear(resultStatus);
			}
		}));
	},

	checkProviderStatus: function(resultCallback, cancelIfNotNotLoggedIn) {
		if (this._externalProjectArea == null) {
			resultCallback(new Status(Status.ERROR, RTCAdapterUIUtil.getMessage("labelNotConfigured")));
		} else {
			var serviceCallback= function(status, response) {
				var resultStatus= status;
				try {
					if (status.isOK()) {
						if(!response.projectArea.initialized){
							resultStatus= new Status(Status.ERROR, RTCAdapterUIUtil.getMessage("labelProjectAreaNotInitialized"));	
						}
					}
				} finally {
					resultCallback(resultStatus);
				}
			};

			RemoteTeamConcertClient.getProjectArea(this._externalProjectArea.getProjectAreaItemId(), this._externalProjectArea, serviceCallback, { requestLabel: RTCAdapterUIUtil.getMessage("labelCheckDefectProviderConfiguration"), cancelIfNotLoggedIn: cancelIfNotNotLoggedIn });
		}
	},

	needsConfiguration: function() {
		return true;
	},

	isConfigured: function() {
		return this._externalProjectArea != null;
	},

	getConfigurationWizard: function() {
		return new TeamConcertWizard(this, "defect");
	},

	initialize: function(runAfterInitialization) {
		TeamConcertConnectionClient.getDefaultTeamConcertConnection("defect", dojo.hitch(this, function(status, result) {
			try {
				this._teamServerConnection= null;
				this._externalProjectArea= null;
				if (status.isOK()) {
					this._teamServerConnection= result;
	
					if (result != null) {
						this._externalProjectArea= new ExternalProjectAreaDescription(result.name, result.description, result.repositoryUrl, result.projectAreaItemId, result.projectAreaName);
					}
				} else {
					console.group("Failed to initialize TeamConcertDefectsProvider");
					status.log();
					console.groupEnd();
				}
			} finally {
				runAfterInitialization();
			}
		}));
	},

	setup: function(runAfterSetup, monitor) {
		monitor.beginTask("Initialize Defect Provider...");

		// initialize again, to reload the just saved configuration including the Project Area UUID
		this.initialize(dojo.hitch(this, function() {

			var setupRequestOptions= { requestLabel: RTCAdapterUIUtil.getMessage("labelSetupDefectsProvider") };

			RemoteTeamConcertClient.queryWorkItemAttributes({ identifier: "com.ibm.rqm.defect" }, this._externalProjectArea, dojo.hitch(this, function(status, result) {
				if (status.isOK()) {
					if(dojo.isArray(result.collection) && result.collection.length == 0) {
						var addAttributeParams= {
							identifier: "com.ibm.rqm.defect",
							type: "smallString"
						};

						RemoteTeamConcertClient.addWorkItemAttribute(addAttributeParams, this._externalProjectArea, dojo.hitch(this, function(status, result) {
							runAfterSetup(status);
						}), setupRequestOptions);
					} else {
						runAfterSetup(Status.OK_STATUS);
					}
				} else {
					runAfterSetup(status);
				}
				
			}), setupRequestOptions);
		}));
	},

	createDefect: function(linkType, itemIdOrUriArray, defectDescription, monitor, runAfterCreation) {
		monitor.beginTask(RTCAdapterUIUtil.getMessage("labelCreateDraftWorkItem"));
		var draftParams= {
			type: "types/" + this._externalProjectArea.getProjectAreaItemId() + "/defect",
			"com.ibm.rqm.defect": "defect@" + getCurrentProjectArea().itemId
		};
		
		if (dojo.isString(defectDescription["summary"])) {
			draftParams.summary= defectDescription["summary"];
		}
		
		if (dojo.isString(defectDescription["description"])) {
			draftParams.description= defectDescription["description"];
		}

		if (dojo.isArray(defectDescription["links"])) {
			draftParams.links= new Array();
			dojo.forEach(defectDescription["links"], dojo.hitch(this, function(link){
				draftParams.links.push(encodeURIComponent("linkType=" + link.type + "&source=false&uri=" + link.uri + "&comment=" + encodeURIComponent(link.comment)));
			}));
		}

		var doCreationCallback= function(status, draftUri, draftLinkParams) {
			if (dojo.isFunction(runAfterCreation)) {
				runAfterCreation(status, draftUri, draftLinkParams);
			}
		} 

		// post the draft work item to the RTC repository
		RemoteTeamConcertClient.postDraftWorkItem(draftParams, this._externalProjectArea, dojo.hitch(this, function(status, response) {
			if (status.isOK()) {
				var draftUri= response._meta.locationHref;
				
				// now post a draft link to the RQM repository
				monitor.beginTask(RTCAdapterUIUtil.getMessage("labelLinkDefectToExecutionRecord"));
				var counter = 0;
				var run = dojo.hitch(this, function(){
					var draftLinkParams= {
						draftUri: draftUri,
						itemIdOrUri: itemIdOrUriArray[counter],
						linkType: linkType,
						id: "---",
						summary: defectDescription["summary"],
						isExecResultRelated: defectDescription["isExecResultRelated"]
					};
					
					TeamConcertConnectionClient.postDraftLink(draftLinkParams, function(status, response){
						counter++;
						if(counter == itemIdOrUriArray.length){
							doCreationCallback(status, draftUri, draftLinkParams);
						} else {
							run();
						}
	                });
				});
				run();
			} else {
				doCreationCallback(status, null);
			}
		}), { requestLabel: RTCAdapterUIUtil.getMessage("labelCreateDefect") });
	},
	
	deleteDefect: function(draftUri, ewiItemId, monitor, runAfterCreation) {
		monitor.beginTask(RTCAdapterUIUtil.getMessage("labelUnAssociatingDefect"));
		var doCreationCallback= function(status, draftUri, draftLinkParams) {
			if (dojo.isFunction(runAfterCreation)) {
				runAfterCreation(status, draftUri, draftLinkParams);
			}
		} 
		var draftLinkParams= {
			defectURI: draftUri,
			ewiItemId: ewiItemId,
			isBlocked: false
		};
		TeamConcertConnectionClient.postBlockDefect(draftLinkParams, function(status, response){
            doCreationCallback(status, draftUri, draftLinkParams);
        });
	},
	
	changeBlockingDefect: function(draftUri, resultItemId,blocking, monitor, runAfterCreation) {
		monitor.beginTask(RTCAdapterUIUtil.getMessage("labelUnLinkDefectToExecutionRecord"));
		var doCreationCallback= function(status, draftUri, draftLinkParams) {
			if (dojo.isFunction(runAfterCreation)) {
				runAfterCreation(status, draftUri, draftLinkParams);
			}
		} 
		var draftLinkParams= {
			defectURI: draftUri,
			resultItemId: resultItemId,
			isBlocked: blocking
		};
		TeamConcertConnectionClient.postBlockDefect(draftLinkParams, function(status, response){
            doCreationCallback(status, draftUri, draftLinkParams);
        });
	},
	queryExistingDefects: function(query, monitor, resultCallback) {
		monitor.beginTask(RTCAdapterUIUtil.getMessage("labelSearchMatchingDefects"));
		var queryParams= {};
		if (!isNaN(parseInt(query["id"]))) {
			queryParams["id"]= query["id"];
		} else if (dojo.isString(query["fullText"])) {
			queryParams["all"]= query["fullText"];
		} else {
			throw new Error("Invalid query descriptor");
		}

		RemoteTeamConcertClient.queryWorkItems(queryParams, this._externalProjectArea, dojo.hitch(this, function(status, response) {
			var resultStatus= status;
			var result= null;
			try {
				if (status.isOK()) {
					result= dojo.map(response.collection, function(entry) {
						var workItem= entry.workItem;
						return {
							label: workItem.id.value + ": " + workItem.summary.value,
							reference: entry._meta.locationHref,
							id: workItem.id.value,
							summary: workItem.summary.value
						}
					});
				}
			} catch (e) {
				resultStatus= new Status(Status.ERROR, RTCAdapterUIUtil.getMessage("labelFailToQueryResults"), e);
			} finally {
				resultCallback(resultStatus, result);
			}
		}));
	},
	
	queryAssociatedDefects: function(linkType, isExecutionResultRelated, itemIdOrUriArray, monitor, resultCallback) {
		monitor.beginTask(RTCAdapterUIUtil.getMessage("labelFetchingAssociatedDefects"));
		
		TeamConcertConnectionClient.postAssociatedWorkItems({ linkType: linkType, isExecutionResultRelated: isExecutionResultRelated, itemIdOrUriArray: itemIdOrUriArray }, dojo.hitch(this, function(status, response) {
			var resultStatus= status;
			var result= null;
			try {
				if (status.isOK() && dojo.isArray(response)) {
					result= dojo.map(response, function(workItem) {
						return {
							label: workItem.id + ": " + workItem.summary,
							reference: workItem.uri,
							id: workItem.id,
							summary: workItem.summary,
							isExecutionResultRelated: isExecutionResultRelated,
							linkType: linkType,
							ewiItemId: workItem.ewiItemId,
							status: workItem.status
						}
					});
				}
			} catch (e) {
				resultStatus= new Status(Status.ERROR, RTCAdapterUIUtil.getMessage("labelFailToQueryResults"), e);
			} finally {
				resultCallback(resultStatus, result);
			}
		}));
	},
	
	associateDefect: function(linkType, linkDescription, monitor, runAfterAssociation) {
		monitor.beginTask(RTCAdapterUIUtil.getMessage("labelAssociatingDefect"));

        var modificationParams = {
            "com.ibm.rqm.defect": "defect@" + getCurrentProjectArea().itemId,
            links: [encodeURIComponent("linkType=" + linkDescription.linkType + "&source=false&uri=" + linkDescription.ewiUri + "&comment=" + encodeURIComponent(linkDescription.comment))]
        };

		RemoteTeamConcertClient.modifyWorkItem(linkDescription.workItemId, modificationParams, this._externalProjectArea, dojo.hitch(this, function(status, response) {
			if (status.isOK()) {
				// now post a link to the RQM repository
				monitor.beginTask(RTCAdapterUIUtil.getMessage("labelLinkDefectToExecutionRecord"));
				var draftLinkParams= {
					draftUri: linkDescription.workItemUri,
					itemIdOrUri: linkDescription.ewiUri,
					linkType: linkType,
					id: linkDescription.workItemId,
					summary: linkDescription.label
				};

                TeamConcertConnectionClient.postDraftLink(draftLinkParams, function(status, response){
                    runAfterAssociation(status, draftLinkParams);
                });
			} else {
				runAfterAssociation(status, draftLinkParams);
			}
		}), { requestLabel: RTCAdapterUIUtil.getMessage("labelAssociateDefect") });
	},

	// ---- implementation -------------------------------------------------------------------------------------------------	

	__sentinel: null // terminates this class definition
});

})();
 