/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.PageList");

dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.internal.PageRegistry");

dojo.require("net.jazz.ajax.ui.PageList");

var checkNotNull 		     = net.jazz.ajax.toolbox.lang.checkNotNull;
var removeChildren 		  = net.jazz.ajax.core.internal.util.removeChildren;
var PageRegistry 		     = net.jazz.ajax.ui.internal.PageRegistry;
var PAGE_SEP_IMG_SRC      = dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/page/templates/images/banner/REQM_tab.gif");
 

 
//extend the net.jazz.ajax.ui.PageList widget
dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.PageList", net.jazz.ajax.ui.PageList, {
	
	// summary:
	//		unordered list of links for the pages in the current application.
	// description:
	//		This widget contains no public methods; simply instantiate it and
	//		place it within a Jazz Framework-based application. See
	//		resources/ui/PageList.css for information on styling the PageList.
	
	//
	// Internal and lifecycle methods
	//
	create: function(args, srcNodeRef) {
		PageList.superclass.create.call(this, args, srcNodeRef);
	},

	postCreate: function() {
		PageList.superclass.postCreate.call(this);
		dojo.removeClass(this.domNode, "net-jazz-ajax-PageList");
	},	
	
	_reload: function() {
		removeChildren(this.domNode);
		
		// sort the pages, if necessary
		var pages = PageRegistry.getPages();
		if(this.sort) {
			console.warn("PageList no longer directly supports page ordering; see Workbench#orderPages");
		}
		
		if (!this.isAdmin)
		{
			//this.suppressedPages = ["com.ibm.team.repository.users", "com.ibm.team.repository.server"];
			this.suppressedPages = ["com.ibm.team.process.admin"];
		}
		
		// remove suppressed pages (if any) from the UI
		if(dojo.isArray(this.suppressedPages) && this.suppressedPages.length > 0){
			var map = {};
			dojo.forEach(this.suppressedPages, function(pageId){
				map[pageId] = true;
			});
			for(var i=pages.length-1; i>=0; i--){
				var pageId = pages[i].id;
				if(map[pageId]){
					pages.splice(i, 1);
				}
			}
		}
		
		// create the list and link elements
		var self = this;
		for (var i = 0; i < pages.length; i++) {
			this._addPage(pages[i], i)
		}
		/*
		dojo.forEach(pages, function(page) {
			self._addPage(page);
		});
		*/
	},
	
	_addPage: function(page, index) {
		checkNotNull(page, "page");
		this.domNode.appendChild(this._createPageListElement(page, index == 0));
		var separator = document.createElement("li");
		dojo.addClass(separator, "separator");
		this.domNode.appendChild(separator);
	},
	
	_createPageListElement: function(page, /* boolean */ first) {
		checkNotNull(page, "page");
		
		var liElement = document.createElement("li");
		var anchor = this._createPageLink(page);
		if (first == true) {
			dojo.addClass(anchor, "active")
		}
		else
		{
			dojo.addClass(anchor, "inactive");
		}
		liElement.appendChild(anchor);
		
		return liElement;
	},
	
	_createPageLink: function(page) {
		checkNotNull(page, "page");
		
		var linkElement = document.createElement("a");
		linkElement.href = page.getUri();
		linkElement.__jazzPageId = page.id;
		linkElement.appendChild( document.createTextNode(page.name));
		
		return linkElement;
	},
	
	_selectPage: function(page) {
		// summary:
		//		Marks the page link for 'id' as selected, and deselects other
		//      page links.
		// id:
		//		The id of the page to select.
		if (!page || !dojo.isString(page.id)) {
			throw new Error("PageList#_handleViewPage: Illegal Argument");
		}
		dojo.forEach(this.domNode.getElementsByTagName("A"), function(link) {
			if (link.__jazzPageId === page.id) {
				dojo.removeClass(link, "inactive");
				dojo.addClass(link, "active");
			} else {
				dojo.removeClass(link, "active");
				dojo.addClass(link, "inactive");
			}
		});
	}	
});

//var PageList = net.jazz.ajax.ui.PageList;
var PageList = com.ibm.rqm.planning.web.ui.internal.page.PageList;

})();
