/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.QualityManagerApplication");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.ui.internal._JazzApplication");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorRegistry");

dojo.require("com.ibm.rqm.planning.web.ui.internal.page.QMPage");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ProjectAreaDialog");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");

var _JazzApplication       = com.ibm.team.repository.web.ui.internal._JazzApplication;
var RepositoryClient       = com.ibm.team.repository.web.client.internal.RepositoryClient;
var InitializationData     = com.ibm.team.repository.web.client.init.InitializationData;
var PlatformUI			      = net.jazz.ajax.ui.PlatformUI;

var QMPage				      = com.ibm.rqm.planning.web.ui.internal.page.QMPage;
var ProjectAreaDialog         = com.ibm.rqm.planning.web.ui.internal.view.common.ProjectAreaDialog;
var DEFAULT_PROJECT_AREA      = "Quality Manager";
var Constants                 = com.ibm.rqm.planning.web.client.internal.Constants;
var ServiceResponseHandler    = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var PermissionsClient         = com.ibm.asq.common.web.client.internal.PermissionsClient;

function workbenchRootNode(){
	// summary:
	//		helper to get workbench root node
	return net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode();
}

// After the page loads, the Jazz Ajax Framework instantiates the workbench
// implementation specified in the web bundle's plugin.xml file. After creating
// the workbench implementation instance it calls the implementation's 'init'
// method. Note that the init method must call the parent _Application class's
// 'start' method in order for the workbench to begin normal event processing
// (e.g. watching for changes in the 
// 
// In this example, the extender creates a few top-level UI elements.
dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.QualityManagerApplication", _JazzApplication,
{
	init: function()
	{
		var srh = new ServiceResponseHandler(this, "success", "failure");
		RepositoryClient.getInitializationData(srh);
		
		//add the editor registry to the global dojo context 
		var editorRegistry = new com.ibm.rqm.planning.web.ui.internal.util.EditorRegistry();
		dojo.setObject(Constants.EDITOR_REGISTRY_OBJECT_ID, editorRegistry);
		
		var root = document.getElementsByTagName("html")[0];
		root.setAttribute("lang", dojo.locale.split("-")[0]);
        
        // On IE, specifically set the tundra CSS link's media type 
        // to screen. If this CSS is sent to the print media on certain
        // RQM print views, it could crash IE. 
        if (dojo.isIE) {
            var links = document.getElementsByTagName("link");
            for (var i = 0; i < links.length; i++) {
                if (links[i].href && links[i].href.indexOf("tundra") > -1) {
                    links[i].media = "screen";
                }
            }
        }
	},
	
	success: function(initData)
	{
		InitializationData._load(initData);
		dojo.setObject(
			"com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR",
			InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer").currentContributor);

		//check user license / login permissions
        var responseHandler = new ServiceResponseHandler(this, "_getProjectAreas", "_loginNotAllowed");        
        PermissionsClient.checkLogin(responseHandler);
		
	},
	
	_getProjectAreas: function() {
		// store the configuration properties in global vars with corresponding
		// names
		var configProperties = InitializationData.get("com.ibm.team.repository.service.configPropertyInitializer");
		for (var propName in configProperties) {
			var propVal = configProperties[propName];
			dojo.setObject(propName, propVal);
		}
		

		// set the global far for the current project area based on the last
		// segment of the document's URI and the set of project areas in the
		// repository
		/*
		var pathname = window.location.pathname;
		var projectName = pathname.substring(pathname.lastIndexOf('/')+1);
		*/
		var projectAreas = InitializationData.get("com.ibm.team.process.service.projectAreaWebUIInitializionData").projectAreas;
		
		/*var projectArea = {}; //dojo.setObject("com.ibm.team.process.web.projectArea", {});
		for (var i = 0; i < projectAreas.length; i++) {
			//find the default project area
			if (projectAreas[i].name == DEFAULT_PROJECT_AREA) {
				projectArea.itemId = projectAreas[i].itemId;
				projectArea.name = projectAreas[i].name;
				break;
			}
		}*/	
		
        var responseHandler = new ServiceResponseHandler(this, "_selectProjectArea", "_loginNotAllowed");
        PermissionsClient.getUserProjectAreas(responseHandler);  
	},
	
	_selectProjectArea: function(projectAreas) {
		//set user's allowed project areas		
		dojo.setObject("com.ibm.rqm.web.userProjectAreas", projectAreas);
		
		//get desired project area from URL
		var path = location.pathname;
		var projectName = path.substring(path.lastIndexOf('/')+1); //$NON-NLS-1$
		
		//if no project area name appended
		if (projectName == 'console') {
	        //show Project Area selection widget
	        if (!dojo.isArray(projectAreas) || projectAreas.length == 0) {
	            this._loginNotAllowed(Constants.getMessage('labelNoProjectAreas'));
	        } else if (projectAreas.length == 1) {
	            //this._checkPermissions(projectAreas[0], false, false);
				this._redirect(projectAreas[0], false);         
	        } else {
	            var paDlg = new ProjectAreaDialog({
	                projectAreas: projectAreas,
	                onOk: dojo.hitch(this, "_redirect")
	            });         
	        }			
		} else {
	        var decodedName = decodeURIComponent(projectName);
	        decodedName = decodedName.replace(/%3F/g,"?"); //$NON-NLS-1$
	        
	        //find the desired project
	        var project = this._findIn(projectAreas, decodedName, "name");
	        if (project != null) {
	            this._checkPermissions(project, false);
	        } else {
				var allProjectAreas = InitializationData.get("com.ibm.team.process.service.projectAreaWebUIInitializionData").projectAreas;
				project = this._findIn(allProjectAreas, decodedName, "name");
				if (project == null) {
					//msg = Constants.getMessage('labelInvalidProjectArea', decodedName)
					this._redirect(null, false);
				} else {
				    this._loginNotAllowed(Constants.getMessage('labelUnauthProjectArea', decodedName));	
				}
			}
		}
    },
	
	_redirect: function(projectArea, setDefaultPA) {
		//TODO set new preferences here
		var projectPath = "";
		if (projectArea) {
			projectPath = location.pathname + "/" + encodeURIComponent(projectArea.name);
		} else {
			projectPath = net.jazz.ajax._contextRoot + "/web/console";
		}
		if (location.search) {
			projectPath +=  location.search;
		}
		if (location.hash) {
			projectPath += location.hash;
		}
		location.href = projectPath;
	},
	    
	_checkPermissions: function(projectArea, setDefaultPA)
	{		
		//set the current PA to the PA selected by the user
		if (projectArea && projectArea.itemId) {
			dojo.setObject("com.ibm.team.process.web.projectArea", projectArea);
		} else {
			throw new Error(Constants.getMessage("msgMissingProjectArea", DEFAULT_PROJECT_AREA));
		}	
			
        //this._loginNotAllowed();  //DEBUG ONLY
        var responseHandler = new ServiceResponseHandler(this, "_getCommonUserPreferences", "_initPermissionFailed");        
        PermissionsClient.fetchAndStoreAllPermissions(responseHandler);
		
		//get isProjectAreaAdmin for the current project area
		var handler = {
			success: function(isAdmin) {
				//boolean response from rest service comes back as string
				dojo.setObject(Constants.IS_PROJECT_AREA_ADMIN, (isAdmin == "true"));
			},
			failure: function(error) {
				console.warn("Unable to determine if User is Project Area ADMIN.  Some functions may be disabled.");
				dojo.setObject(Constants.IS_PROJECT_AREA_ADMIN, false);
			}
		}
        var responseHandler = new ServiceResponseHandler(handler, "success", "failure");        
        PermissionsClient.checkIsProjectAreaAdmin(responseHandler);		
	},
    
    _getCommonUserPreferences: function()
    {
        var internalHandlerObj = {
            self: this,
            success: function(response) {
                this.self._preInitRtlm();
            },
            failure: function(e) {
                this.self._preInitRtlm();
            }
        }
        var responseHandler = new ServiceResponseHandler(internalHandlerObj, "success", "failure");
        com.ibm.asq.common.web.client.internal.CommonPreferenceClient.fetchAndStoreUserPreferences(responseHandler);
    },	
	
    _loginNotAllowed: function(e) 
    {
        console.debug("Can't log in due to license issue.");

		var anchor = document.createElement("a");
		anchor.appendChild( document.createTextNode(Constants.getMessage("labelReturnToLogin")) );
		anchor.id = "login-failure-anchor";
		anchor.onclick = function ()
		{
            var bindArgs =
			{
                url: net.jazz.ajax._contextRoot + "/auth/logout",
                handleAs: "json",
                headers: {
            		        accept: "text/json"
                		 },
                preventCache: dojo.isIE,
                handle: function(response, ioArgs)
				{
					window.location.href = document.location.protocol + "//" +
										   document.location.host +
										   document.location.pathname +
										   document.location.search;
                }
            };
            dojo.xhrPost(bindArgs);
		}

		var box = document.createElement("div");
		box.id = "box";
		var content = document.createElement("div");
		var lock = document.createElement("img");
		lock.id = "lock-image";
		lock.src = dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/page/templates/images/authorize_lock_obj32.gif");
		var textDiv = document.createElement("div");
		textDiv.id = "no-auth-text";
		var text1 = document.createElement("p");
		var msg = dojo.isString(e) ? e : Constants.getMessage("labelNoLicense");
		text1.appendChild(document.createTextNode(msg));
		var text2 = document.createElement("p");
		text2.appendChild(document.createTextNode(Constants.getMessage("labelNoLicenseContact")));
		var text3 = document.createElement("p");
		text3.appendChild(document.createTextNode(Constants.getMessage("labelNoLicense2")));
		text3.appendChild(anchor);
		text3.appendChild(document.createTextNode(Constants.getMessage("labelNoLicense3")));

		textDiv.appendChild(text1);
		textDiv.appendChild(text2);
		textDiv.appendChild(text3);
		content.appendChild(lock);
		content.appendChild(textDiv);
		box.appendChild(content);
		
        dojo.addClass(workbenchRootNode(), "no-auth-container");
		workbenchRootNode().appendChild(box);
    },

    _initPermissionFailed: function() 
    {
        console.debug("Unable to fetch permission information.");
        this._getCommonUserPreferences();
    },
     
    _preInitRtlm: function()
    {
        // initialize rtlm
        var cb = dojo.hitch(this, "_initRtlm");
        net.jazz.ajax.core.internal.loader.load_async("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer", cb);
    },
    
	_initRtlm: function() 
    {        
		var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_initRtlmSuccess", "_initRtlmFailure");
        com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.initializeData(responseHandler);
    },
	
	_initRtlmSuccess: function(response) {
        console.debug("initialized catalog successfully from QMApplication!");
        this._initAppUI();
	},
	
	_initRtlmFailure: function(response) {			
		console.error("initialization failed from QMApplication! " + response['errorMessage']);
        // rtlm init failed, save the error object and disable the lab management only
        dojo.setObject("com.ibm.rqm.planning.web.ui.initError", response);

        this._initAppUI();
	},
	
    _initAppUI: function()
    {
        var projectArea = dojo.getObject("com.ibm.team.process.web.projectArea");
        this._createUI({label: projectArea.name, showLoginPanel: true, suppressedPages: {}});
        this.start();        
    },
    
	_createUI: function(args) {
        CSHelpEngine.init(Constants.CSHELP_URL, Constants.CSHELP_INFO_CENTER_LABEL, Constants.CSHELP_INFO_CENTER_HREF);                      
		Constants.validateMessageBundle();
        
		// order pages
		var self = this;
		var orderedPageIDs = this.getOrderedPageIDs();
		if(orderedPageIDs) {
			PlatformUI.getWorkbench().orderPages(function(a, b) {
				return self.sortPages(a, b, orderedPageIDs);
			});
		} else {
			console.debug("No page order specified; pages will appear in arbitrary order.");
		}		
		
		dojo.addClass(dojo.body(), "com-ibm-rqm-web");
		dojo.addClass(dojo.body(), "rqm-print");
				
		var container = document.createElement("div");
		container.id = "outer-container";
	
		var qmPage = new QMPage();
		container.appendChild(qmPage.domNode);
		
		workbenchRootNode().appendChild(container);
		
		// set applicaiton name to "Rational Quality Manager"
		this.setName(Constants.getMessage("labelRQMTitle"));   
	},

	failure: function(errorObj)
	{
		console.error(errorObj.message);
	},
	
	// functions for sorting pages	
	getCurrentWorkbenchID: function() {
		return net.jazz.ajax.ui.PlatformUI._getApplicationId();
	},
	
	getOrderedPageIDs: function() {
		// parses out ordered page ids from the config property
		// example of valid JSON:
		//     {"com.ibm.team.repository.web.projectArea":
		//       ["com.ibm.team.workitem", "com.ibm.team.apt", "com.ibm.team.reports"]}
		var pageOrder = com.ibm.team.repository.web.pageorder;
		if(!pageOrder) {
			return null;
		}
		var pages = pageOrder.split(",");
		if (pages && pages.length > 0) {
		  	for (var i = 0; i < pages.length; i++) {
            pages[i] = dojo.trim(pages[i]);
		  	}
      }
		return pages;
	},

	sortPages: function(a, b, orderedPageIDs) {
		for (var i=0; i<orderedPageIDs.length; i++) {
			if(orderedPageIDs[i] === a.id){
				return -1;
			}
			if(orderedPageIDs[i] === b.id){
				return 1;
			}
		}
		return -1;
	}, 
	
    _findIn: function(array, value, byProperty) {
        for (var i = 0; i < (dojo.isArray(array) ? array.length : 0); i++) {
			if (dojo.isObject(array[i]) && byProperty != null && array[i][byProperty] && array[i][byProperty] == value) {
				return array[i]; 		 	
			} else if (array[i] == value) {
				return array[i];
			}			
		}
		return null;
    }
		
});
})();
