/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.construction.ConstructionPageController");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorFactory");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.mapper.TestPlanMapper");

//xojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionWorkItemClient");

(function() {

var ServiceResponseHandler 	    = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var EditorFactory 				= com.ibm.rqm.planning.web.ui.internal.util.EditorFactory;
var QueryClient            		= com.ibm.rqm.planning.web.client.internal.QueryClient;
var EditorFactory 				= com.ibm.rqm.planning.web.ui.internal.util.EditorFactory;
var TestScriptClient       		= com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
var TestCaseClient         		= com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var TestGroupClient        		= com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient;
var PlanningPageClient     		= com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
var TestPlanMapper				= com.ibm.rqm.planning.web.client.internal.mapper.TestPlanMapper;

//FIXME circular dependency HACK
//var ExecutionWorkItemClient		= com.ibm.rqm.execution.web.client.internal.execution.ExecutionWorkItemClient;

var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

/**
 * Main page controller
 */
dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.page.construction.ConstructionPageController", 
	null, 
	{
		constructor: function(constructionView) {
			//alert("ConstructionPageController: initializer");
			this._view = constructionView;
		},

	   /*
		* Provide access to an EditorFactory instance.
		*/
		getEditorFactory: function() {
			return EditorFactory;
		},
				
		/*
		 * The following methods support the URI navigation control
		 */
	
		defaultAction: function()
		{
			this.viewTestScripts({name: "My Test Scripts"});
		},
		
		/*
		 * Initial Navigation.
		 */
		init: function() {
			// Initial navigation is set in ViewTestGroupList initialization
			alert("ConstructionPageController: init");
			this.viewTestGroups({name: "My Test Case Groups"});
		},
		
			viewTestPlans: function(kwArgs) 
			{
				var self = this;
				var handler = 
				{
					success: function(plans)
					{
						self._view.showTestPlanList(plans, kwArgs.name);
					},
					error: function(e) 
					{
						self._error(e);
					}
				}
				
				var responseHandler = new TestPlanMapper(handler, "success", "error");			
				PlanningPageClient.getTestPlansSummary(responseHandler, kwArgs.name);				
			},	

			viewTestPlan: function(kwArgs) 
			{	
				console.debug("PlanningPageController: "+kwArgs.id+kwArgs.currentId);			
				var responseHandler = new ServiceResponseHandler(this, "_doViewTestPlan", "_error");
				PlanningPageClient.getTestPlansWithWorkItems(responseHandler, kwArgs);
			},
			
			_doViewTestPlan: function(plans)
			{
				if (plans.length > 1)
				{
					alert("PlanningPageController: too many Plans returned, should be 1.");
				}
				this._view.showTestPlan(plans[0]);
			},	

			copyTestPlan: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doCopyTestPlan", "_error");			
				PlanningPageClient.getTestPlansWithWorkItems(responseHandler, kwArgs);
			},
			
			_doCopyTestPlan: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}
				this._view.copyTestPlan(result);				
			},
			
			/* 
			 * Execution Work Items URI ACTIONS
			 */	
			newEwi: function(kwArgs) 
			{
				this._view.showNewEwi(kwArgs.name);
			},		
			 			
			viewEwi: function(kwArgs) 
			{				
				// TODO: make client call to retrieve EWI
				this._doViewEwi(kwArgs.name);		
			},
			
			_doViewEwi: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}			
				this._view.showEwi(result);	
			},		
			
			viewEwis: function(kwArgs) {
				var self = this;
				var callbackObj = {
					success: function(ewis) {
						self._view.showEwiList(ewis, kwArgs.name);
					},
					failure: function(e) {
						self._error(e);
					}				
				}
			   var args = {
   				name: kwArgs.name,
   				owner: getAuthenticatedContributor().name
				};
			
			var responseHandler = new ServiceResponseHandler(callbackObj, "success", "failure");
//			ExecutionWorkItemClient.getExecutionWorkItemDTOs(responseHandler, args);

			},			

			/* 
			 * TEST GROUP URI ACTIONS
			 */	
			newTestGroup: function(kwArgs) 
			{
				this._view.showNewTestGroup(kwArgs.name);
			},		
			 			
			viewTestGroup: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doViewTestGroup", "_error");
				if (kwArgs.uid!=null && kwArgs.revision!=null) {
					TestGroupClient.getTestGroupRevision(responseHandler, kwArgs.uid, kwArgs.revision);		
				} else {
					TestGroupClient.getTestGroup(responseHandler, kwArgs.id);		
				}		
			},
			
			copyTestGroup: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doCopyTestGroup", "_error");			
				TestGroupClient.getTestGroupRevision(responseHandler, kwArgs.uid, kwArgs.revision);		
			},
			
			_doViewTestGroup: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}			
				this._view.showTestGroup(result);	
			},				
			
			_doCopyTestGroup: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}			
				this._view.copyTestGroup(result);	
			},				
			
			viewTestGroups: function(kwArgs) 
			{
				var self = this;

				var handler = 
				{
					success: function(groups)
					{
						self._view.showTestGroupList(groups, kwArgs.name);
					},
					error: function(e) 
					{
						self._error(e);
					}
				}
				
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");			
				TestGroupClient.getTestGroups(responseHandler);				
			},	
			
			/* 
			 * TEST CASE URI ACTIONS
			 */				
			newTestCase: function(kwArgs) 
			{
				this._view.showNewTestCase(kwArgs.name);
			},
								
			viewTestCase: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doViewTestCase", "_error");
				if (kwArgs.uid!=null && kwArgs.revision!=null) {
					TestCaseClient.getTestCaseRevision(responseHandler, kwArgs.uid, kwArgs.revision);
				} else {
					TestCaseClient.getTestCase(responseHandler, kwArgs.id);
				}		
			},
			
			copyTestCase: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doCopyTestCase", "_error");			
				TestCaseClient.getTestCaseRevision(responseHandler, kwArgs.uid, kwArgs.revision);
			},
			
			_doViewTestCase: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}
				this._view.showTestCase(result);				
			},
			
			_doCopyTestCase: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}
				this._view.copyTestCase(result);				
			},
			
			viewTestCases: function(kwArgs) 
			{
				var self = this;

				var handler = 
				{
					success: function(cases)
					{
						self._view.showTestCaseList(cases, kwArgs.name);
					},
					error: function(e) 
					{
						self._error(e);
					}
				}
				
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");			
//				var responseHandler = new ServiceResponseHandler(this, "_doViewTestCases", "_error");			
				TestCaseClient.getTestCases(responseHandler);				
			},	
			
//			_doViewTestCases: function(cases) {
//				this._view.showTestCaseList(cases);
//			},					
				
		/* 
		 * TEST SCRIPT URI ACTIONS
		 */				
		newTestScript: function(kwArgs) {
			this._view.showNewTestScript(kwArgs.name);
		},
							
		viewTestScript: function(kwArgs) 
		{				
			var responseHandler = new ServiceResponseHandler(this, "_doViewTestScript", "_error");			
			TestScriptClient.getTestScript(responseHandler, kwArgs.id);
		},
		
		_doViewTestScript: function(result) {
			if ( dojo.isArray(result) ) {
				result = result[0];	
			}
			this._view.showTestScript(result);				
		},
		
		viewTestScripts: function(kwArgs) 
		{			
			var self = this;
			var callbackObj = {
				success: function(scripts) {
					self._view.showTestScriptList(scripts, kwArgs.name);
				},
				failure: function(e) {
					self._error(e);
				}				
			}
			
  			// fetch scripts
			var responseHandler=new ServiceResponseHandler(callbackObj, "success", "failure");
		    TestScriptClient.getTestScripts(responseHandler);						
		},
		
		importScripts: function(kwArgs)
		{
			this._view.showImportWizard(kwArgs.name);
		},
			
		_error: function(error) {
			alert("Error invoking service; message: " + error.message);
		}	
});

})();
