/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.home.DashboardHomePage");

dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardPage");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.page.home.DashboardContainer");

//workaround Jazz dashboard missing dojo.require and variable declaration problem in M5a.
dojo.require("com.ibm.team.dashboard.web.ui.IMemento");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var InitializationData     = com.ibm.team.repository.web.client.init.InitializationData;

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardContainer = com.ibm.rqm.planning.web.ui.internal.page.home.DashboardContainer;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

(function()
{
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.home.DashboardHomePage", com.ibm.team.dashboard.web.ui.internal.DashboardPage,
	{
		postCreate: function()
		{
			com.ibm.rqm.planning.web.ui.internal.page.home.DashboardHomePage.superclass.postCreate.apply(this);
			
			this._layout._leftNav.style.display = "none";
			this._layout._showHideControl.style.display = "none";
			dojo.addClass(this._layout._leftNav, "dashboard-left");
			dojo.addClass(this._layout._showHideControl, "dashboard-left");
		},
		
		// CAUTION!! INTERNAL API FROM DASHBOARD USED HERE. SUBJECT TO CHANGE ANY TIME! [BJ]
		showPrivateDashboard: function( contributorId, dashboardId, tabId ) {
			
			//check if there's already a private dashboard created for this contributor
			var serviceObj = {
				dashboardPage: this,
				success: function(/* ResponseDTO */ responseDTO) {
					switch(responseDTO.status) {
						case "OK":
							var dashboardItem = responseDTO.data;
							
							// if the default dashboard is already a private one, meaning it's already created, just show it.
							if(dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR)
								if( tabId )
									this.dashboardPage.viewDashboard({id:dashboardItem.getId(), tab: tabId});
								else
									this.dashboardPage.viewDashboard({id:dashboardItem.getId()});
							else
								this.dashboardPage.createPrivateDashboard(contributorId);
							break;
						case "ERROR":
							this.dashboardPage._createError.call(this.dashboardPage,responseDTO.msg);
							break;
					}
				},
				failure: function(errorObj) {
					console.error(errorObj.message);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.getDashboard({showDefault: true}, true, srh);  // refer to DashboardRestService.java for arguments usage and logic.
		},
					
		// CAUTION!! INTERNAL API FROM DASHBOARD USED HERE. SUBJECT TO CHANGE ANY TIME! [BJ]
		createPrivateDashboard: function(contributorId) {
			var serviceObj = {
				dashboardPage: this,
				success: function(/* ResponseDTO */ responseDTO) {
					switch(responseDTO.status) {
						case "OK":
							var dashboardItem = responseDTO.data;
							
							// CAUTION ON THE ARG!! THIS INTERNAL API FROM DASHBOARD AND VERY POSSIBLE TO CHANGE AFTER 1.0 [BJ]
							// (info. from Dashboard developer Curtis D'Entremont/Toronto/IBM@IBMCA)
							// to pass in the dashboard item id, use:
							// 'id' for retrieving private dashboard.
							// 'user' for profile dashboard
							// for team dashboards, it's a path to the team name with the "team" parameter, e.g. "team: Jazz/Dashboard"
							// for project dashboard, it's the default and thus no parameters.
							this.dashboardPage.viewDashboard({id:dashboardItem.getId()});
							break;
						case "ERROR":
							this.dashboardPage._createError.call(this.dashboardPage,responseDTO.msg);
							break;
					}
				},
				failure: function(errorObj) {
					console.error(errorObj.message);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.createDashboard(DashboardConstants.SCOPE_CONTRIBUTOR, contributorId, false, srh);
		},
		
		_createContentWidget: function(/* string */ dashboardTabId) {
			if (this._contentWidget){
				this._destroyContentWidget();
			}
			if (!dashboardTabId)
				dashboardTabId = "tab1";
			this._contentWidget = new DashboardContainer({dashboardPage: this, selectedTabId: dashboardTabId});
			this._layout.setMainContent(this._contentWidget.domNode);
			this._contentWidget.init();
		},
		
		_handleNavigate: function(urlObj){
			
			//Search through the viewlet pages for zoomed viewlets.  Unzoom those that
			//are found.  Otherwise, the zoomed viewlet will be made invisible with
			//a display = "none", and it will continually disable the dash board page.
			var pages = this._contentWidget.viewletPages;
			
			for( var iPage = 0; iPage < (pages ? pages.length : 0); iPage++ )
			{
				if (pages[iPage]) {
					var columns = pages[iPage].columns;
					for (var iCol = 0; iCol < columns.length; iCol++) {
						var widgets = columns[iCol].widgets;
						
						for (var iWidget = 0; iWidget < widgets.length; iWidget++) {
							if (widgets[iWidget]._viewlet._zoomed) {
								widgets[iWidget]._viewlet.container.toggleZoom(null, null);
							}
						}
					}
				}
			}
			
			return( this.inherited( "_handleNavigate", arguments ) );
		},
		
	_deleteDashboard: function(scope, scopeItemId, isPublic, modified, force) {
		var serviceObj = {
			self: this,
			postDelete: function (descriptor) {
				// see if a dashboard is open
				if (this.self._contentWidget && this.self._contentWidget instanceof DashboardContainer) {
					var dashboardItem = this.self._contentWidget.getDashboardItem();
					// see if the deleted dashboard is open
					if (descriptor.itemId == dashboardItem.getId()) {
						this.self.setCurrentDeleted();
						if (scope == DashboardConstants.SCOPE_CONTRIBUTOR && !isPublic) {
							// deleting a private dashboard, go to default
							var userItemId = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR").itemId;
							this.self.showPrivateDashboard( userItemId );
							//location.href = "#" + Constants.URL_PARAM_ACTION + "=" + Constants.URL_PARAM_ACTION_VIEW_PAGE + "&" + Constants.URL_PARAM_ACTION_VIEW_PAGE_ID + "=" + DashboardConstants.PAGE_ID;
						} else {
							// deleting a shared dashboard, return to it
							var url = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + (descriptor.urlParams ? ("&" + descriptor.urlParams) : "");
							if (location.href.substring(location.href.indexOf("#")) == url) {
								// already at the target href, call viewDashboard directly
								// can't be a private dashboard, so contributor scope means public
								if (scope == DashboardConstants.SCOPE_CONTRIBUTOR)
									var args = {user: descriptor.user};
								else if (scope == DashboardConstants.SCOPE_TEAM_AREA)
									var args = {team: decodeURIComponent(descriptor.team)};
								else
									var args = {};
								this.self.viewDashboard(args);
							} else {
								// not at target href, but viewing dashboard
								// must have tab param, so update url
								var userItemId = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR").itemId;
								this.self.showPrivateDashboard( userItemId );
								//location.href = url;
							}
						}
					}
				}
			},
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "OK":
						this.self._leftNav._removeDashboardEntry(responseDTO.data.itemId);
						this.postDelete(responseDTO.data);
						break;
					case "STALE":
						if (responseDTO.msg) {
							// msg present means modified by someone else
							alert("The dashboard could not be restored because it has been modified by " + responseDTO.msg + ". Please refresh to see the latest content.");
						}
						else {
							// no msg means modified by current user
							alert("The dashboard could not be saved because it independently modified in another browser. Please refresh to see the latest content.");
						}
						this.self._leftNav._removeDashboardEntry(responseDTO.data.itemId);
						this.postDelete(responseDTO.data);
						break;
					case "LOCKED":
						if (confirm(responseDTO.msg + " is currently editing this dashboard. Would you like to override and restore anyway? (they will be unable to save their changes)")) {
							this.self._deleteDashboard(scope, scopeItemId, isPublic, null, true);
						}
						break;
					case "ERROR":
						alert(responseDTO.msg);
						break;
				}
				
			},
			failure: function(errorObj) {
				console.error(Constants.LeftNav.REMOTE_ERROR + errorObj.message);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.deleteDashboard(scope, scopeItemId, isPublic, modified, force, srh);
	},
	
		updateBrowserTitle: function() {
			
			PlatformUI.getWorkbench().setTitle(this.getDashboardTitle());
			
		},
		
		getDashboardTitle: function(){
			var contentWidget = this._contentWidget;
			if (contentWidget && contentWidget.getDashboardItem && contentWidget.getDashboardItem()) {
				if (contentWidget.currentPage && contentWidget.currentPage.viewletPageHelper) {
					var tabTitle = contentWidget.currentPage.viewletPageHelper.getTitle();
					
					return this.messages.dashboardsGenericTitle + Constants.getMessage("labelColen") +" " +tabTitle;
				}
			}
 
			return this.messages.dashboardsGenericTitle;
		}			
	});
})();
