/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.pages.MachinePage");

dojo.require("dojo.date.locale");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.PrimaryActionButton");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.SecondaryActionsDropdownButton");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.PageLayout");

dojo.require("com.ibm.rational.test.lm.web.client.internal.machine");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.PageUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.GroupUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ImportMachineView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.MachineSearchView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetDetailView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ManageGroupsView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.GroupDetailView");

(function() {
	
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
var MACHINE_ASSET_TYPE = "Machine";

function getWorkbench() {
    return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.pages.MachinePage",
    com.ibm.asq.common.web.ui.internal.view.layout.PageLayout,
{
    hideSearchSection: true, // we don't have search in the left action panel yet

    // Parent main page that holds this assets sub-page.  All registered
    // action must have the parent main page bring up this sub-page before
    // performing the action.
    parentMainPage: null,
    
	_moreActionsButton: null,
	_searchButton: null,
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        var newLabel = RtlmUiUtil.getMessage("newLabel");
        // temporary button for new asset.
        var param = {
            id: "newMachine",
            iconClass: "newMachineIcon",
            actionLabel: newLabel,
            itemLabel: "Machine"
        };
        var newAction = new com.ibm.asq.common.web.ui.internal.view.layout.PrimaryActionButton(param);
        newAction.onClick = function(){
			window.location.href = AssetUriUtil.getNewAssetURI(MACHINE_ASSET_TYPE);
		};
        this.setPrimaryAction(newAction.domNode);

		this._moreActionsButton = new com.ibm.asq.common.web.ui.internal.view.layout.SecondaryActionsDropdownButton();
		this._moreActionsButton.setLabel(RtlmUiUtil.getMessage("moreActionsButtonLabel"));
        this.setSecondaryActions(this._moreActionsButton.domNode);
		this._getMoreActionsButton().addAction(dojo.hitch(this, "_invokeSearchAssetView"), "Search");
			
        var viewActions = new com.ibm.rational.test.lm.web.ui.internal.pages.PageNavigator({fixedSelection:"Assets"});
        this._leftNavigator.getViewsSection().appendChild(viewActions.domNode);

		var machine = new com.ibm.rational.test.lm.web.client.internal.machine();
		var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getPermissionsSuccess", "_getPermissionsError");		
		machine.getAllowedCatalogAssetOps(responseHandler);
		
        this._getMoreActionsButton().addAction(dojo.hitch(this, "_invokeShowManageGroupsView"), "Manage Groups");

		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(AssetUriUtil.PageRoot + AssetUriUtil.AssetImportKey, this, "assetImportView");
		ActionRegistry.registerAction(AssetUriUtil.PageRoot + AssetUriUtil.TemplateImportKey, this, "templateImportView");
		ActionRegistry.registerAction(AssetUriUtil.PageRoot + AssetUriUtil.AssetSearchKey, this, "findAssetsView");
        ActionRegistry.registerAction(AssetUriUtil.PageRoot + AssetUriUtil.AssetNewKey, this, "newAssetView");
        ActionRegistry.registerAction(AssetUriUtil.PageRoot + AssetUriUtil.AssetViewDetailKey, this, "viewAssetDetailView");
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.MachinePage.manageGroups", this, "showManageGroupsView");
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.MachinePage.newGroup", this, "newGroupView");
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.MachinePage.viewGroup", this, "groupDetailView");

        this.addSampleRelatedInfo();
    },
	
	// Only create the appropriate actions if the current user has the permission.
	_getPermissionsSuccess: function(response) {
		// add the sub-create actions before the import actions.
        for(var i in response) {        
            if(response[i] == "create") {
                this._addMoreCreateActions();
				break;
			}
        }
		for(var i in response) {		
			if(response[i] == "asset")			
				this._getMoreActionsButton().addAction(dojo.hitch(this, "handleAssetImportView"), RtlmUiUtil.getMessage("impLabel_asset"));
		}
	},
	
	_getPermissionsError: function(response) {
		console.debug(response);
	},
	
	_getMoreActionsButton: function() {
		if(!this._moreActionsButton) {
			this._moreActionsButton = new com.ibm.asq.common.web.ui.internal.view.layout.SecondaryActionsDropdownButton();
			this._moreActionsButton.setLabel(RtlmUiUtil.getMessage("moreActionsButtonLabel"));
			this._leftNavigator.getSecondaryActionsSection().appendChild(this._moreActionsButton.domNode);
		}
		
		return this._moreActionsButton;
	},

    init: function()
    {
        console.debug("init machine page");
    },
	
	_addMoreCreateActions: function()
	{
        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
	    var assetTypeNames = catalog.getAssetTemplateNames();
		var moreActionArea = this._getMoreActionsButton();
		
		for(var i in assetTypeNames) {
			// skip over the machine asset type. We have a Big button for that.
			if(assetTypeNames[i] == MACHINE_ASSET_TYPE)
			    continue;
		    

            var newLabel = RtlmUiUtil.getMessage("newLabel");
            var assetName = RtlmUiUtil.getMessage(assetTypeNames[i]);
			moreActionArea.addAction(
			     dojo.hitch(this, "_handleNewAssetAction"), 
                 newLabel+ ' ' + assetName,//TODO - fix concatenation!
				 assetTypeNames[i]
			);
		}
	},
	
	_handleNewAssetAction: function(args)
	{
        window.location.href = AssetUriUtil.getNewAssetURI(args);
	},
	
	findAssetsView: function(args)
	{
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }

		var findMachinesView = this.getCachedView("findAssetsView");
        var assetType = args.type;
		
        // the view is not in our cache, create a new one.
        if(!findMachinesView) {    
            findMachinesView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.MachineSearchView();
        }

        // carry out the directives.
        //
        if(args.removeViewId != null && this.getCachedView(args.removeViewId))
            this.removeCachedView(args.removeViewId);

        this.setPageView("findAssetsView", findMachinesView);
		
		return findMachinesView;
	},
	
    _invokeSearchAssetView: function()
    {
        window.location.href = AssetUriUtil.getSearchAssetURI("Machine");
    },
    
	assetImportView: function()
	{
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }

		var assetImportView = this.getCachedView(AssetUriUtil.AssetImportKey);
        if (!assetImportView) {
			assetImportView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.ImportMachineView("asset");
        }

        this.setPageView(AssetUriUtil.AssetImportKey, assetImportView);
	},
		    
    handleAssetImportView: function()
    {
        window.location.href = AssetUriUtil.getImportAssetURI();
    },
	
	templateImportView: function()
	{
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }

		var templateImportView = this.getCachedView(AssetUriUtil.TemplateImportKey);
        if (!templateImportView) {
			templateImportView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.ImportMachineView("schema");
        }

        this.setPageView(AssetUriUtil.TemplateImportKey, templateImportView);
	},
	
    newAssetView: function(args)
    {
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }

	   var newAssetView = this.getCachedView(args.id);	   
	   var type = args.type;
	   
	   if(!newAssetView)
	       newAssetView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetDetailView({assetId:args.id, assetType: type, isNew:true});
	   this.setPageView(args.id, newAssetView);	   
    },
	
	viewAssetDetailView: function(args)
    {
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }

       var assetView = this.getCachedView(args.id);     
       var type = args.type;
       var initMessage = null;
	   
	   if(args.initMessage)
	       initMessage = args.initMessage;

        // carry out the directives.
        //
        if(args.removeViewId != null && this.getCachedView(args.removeViewId))
            this.removeCachedView(args.removeViewId);
		   
       if(!assetView)
           assetView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetDetailView({assetId:args.id, assetType: type, isNew:false, statusMessage: initMessage});
       this.setPageView(args.id, assetView);    
    },
    
    _invokeShowManageGroupsView: function()
    {
        window.location.href = AssetUriUtil.getManageGroupsURI();
    },
    
    showManageGroupsView: function(args){
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }
        var manageGroupsView = this.getCachedView("manageGroupsView");
        
        // the view is not in our cache, create a new one.
        if(!manageGroupsView) {    
            manageGroupsView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.ManageGroupsView();
        }
		
        this.setPageView("manageGroupsView", manageGroupsView);

        // refresh the groups list
        if (args.refresh || !GroupUtil.isGroupsListValid()) {
			manageGroupsView.refresh();
		}
		
        if (args.initMessage)
            manageGroupsView.setStatus(args.initMessage, args.initStatus);
		else
            manageGroupsView.clearStatus();

    },
    
    newGroupView: function(args) {
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }

        var newGroupView = this.getCachedView(args.id);     
       
        if(!newGroupView)
            newGroupView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.GroupDetailView({groupId:args.id, isNew:true});
        this.setPageView(args.id, newGroupView);    
    },
    
    groupDetailView: function(args) {
        if (this.parentMainPage) {
            this.parentMainPage.showAssetsSubPage();
        }

        var initMessage = null;
		var initStatus = null;
       
        if (args.initMessage)
            initMessage = args.initMessage;

        if (args.initStatus)
            initStatus = args.initStatus;

        if (args.removeViewId != null && this.getCachedView(args.removeViewId))
            this.removeCachedView(args.removeViewId);
           
        var groupDetailView = this.getCachedView(args.id);
		
		if (!GroupUtil.isGroupValid(args.id)) {
			if (groupDetailView) {
				this.removeCachedView(args.id);
				groupDetailView = null;
			}
			GroupUtil.removeGroupFromInvalidate(args.id);
		}

        if (!groupDetailView)
            groupDetailView = new com.ibm.rational.test.lm.web.ui.internal.views.machines.GroupDetailView({groupId:args.id, isNew:false, statusMessage:initMessage, statusState:initStatus});

        this.setPageView(args.id, groupDetailView);    
    }
    
});

})();
