/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.FunctionalTooltip");

(function(){

    var Utility		= com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var Constants	   = com.ibm.rqm.planning.web.client.internal.Constants;
    
    com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder = {
    
        addTooltip: function(html, elementToConnect){
            var span = dojo.doc.createElement("span");
            return new com.ibm.asq.common.web.ui.internal.util.FunctionalTooltip({connectId: elementToConnect, label: html}, span);
        },
		
		addLabelTooltip: function(html, labelSpan, widget){
			var span = dojo.doc.createElement("span");
            widget.tooltip = new com.ibm.asq.common.web.ui.internal.util.FunctionalTooltip({connectId: labelSpan, label: html}, span);
		},		
        
        addArtifactTip: function(artifact, elementToConnect){
            var html = this.getArtifactTipHTML(artifact);
            return this.addTooltip(html, elementToConnect);
        },
        
        getArtifactTipHTML: function(artifact){
            var artifactType = Utility.getArtifactType(artifact);
            var html = "<table style=\"font-size: smaller; font-family:Verdana;\" border=0>";
            html += "<tr><td colspan=\"2\" align=\"center\"><b>" + artifactType + "</b></td></tr>";
            html += "<tr><td align=right><b>Id:</b></td><td>" + artifact.id + "</td></tr>";
            html += "<tr><td align=right><b>Name:</b></td><td>" + artifact.name + "</td></tr>";
            if (artifact.owner && dojo.isObject(artifact.owner)) {
                html += "<tr><td align=right><b>Owned By:</b></td><td>" + artifact.owner.name + "</td></tr>";
            }
            if (artifact.originator && dojo.isObject(artifact.originator)) {
                html += "<tr><td align=right><b>Created By:</b></td><td>" + artifact.originator.name + "</td></tr>";
            }
            if (artifact.modified) {
                var modified = new Date();
                modified.setTime(artifact.modified);
                html += "<tr><td align=right><b>Modified:</b></td><td>" + Utility.formatDate(modified) + "</td></tr>";
            }
			var snapshotInfo = artifact.snapshotInfo;
            if (snapshotInfo && dojo.isObject(snapshotInfo)) {
                var created = new Date();
                created.setTime(Utility.dateFromISOString(snapshotInfo.created));
                html += "<tr><td align=right><b>Snapshot Name:</b></td><td>" + snapshotInfo.name + "</td></tr>";
                html += "<tr><td align=right><b>Snapshot Description:</b></td><td>" + snapshotInfo.description + "</td></tr>";
                html += "<tr><td align=right><b>Snapshot Created:</b></td><td>" + Utility.formatDate(created) + "</td></tr>";
                html += "<tr><td align=right><b>Snapshot Originator:</b></td><td>" + snapshotInfo.originator.name + "</td></tr>";
				if (snapshotInfo.snapshotOrigin!=null) {
					html += "<tr><td align=right><b>Snapshot Origin:</b></td><td>"
					html += snapshotInfo.snapshotOrigin.itemType + " : " + snapshotInfo.snapshotOrigin.name + "</td></tr>";
				}
            }
            
            switch (artifactType) {
                case Constants.TYPE_PLAN:
                    //TODO modify for plan here !
                    break;
                    
                case Constants.TYPE_GROUP:
                    //TODO modify for group here !
                    break;
                    
                case Constants.TYPE_CASE:
                    //TODO modify for case here !
                    break;
                    
                case Constants.TYPE_SCRIPT:
                    //TODO modify for script here !
                    break;
                    
                default:
                //don't add anything additional
            }
            
            html += "</table>";
            return html;
        },
        
        addLinkTip: function(linkType, link, elementToConnect){
            var html = this.getLinkTipHTML(linkType, link);
            return this.addTooltip(html, elementToConnect);
        },
        
        getLinkTipHTML: function(linkType, link){
            var html = "<table style=\"font-size: smaller; font-family:Verdana;\" border=0>";
            html += "<tr><td align=right><b>Id:</b></td><td>" + link.id + "</td></tr>";
            html += "<tr><td align=right><b>Name:</b></td><td>" + link.comment + "</td></tr>";
            
            if (link.section != null) {
                html += "<tr><td align=right><b>Section:</b></td><td>" + link.section + "</td></tr>";
            }
            else 
                if (link.field != null) {
                    html += "<tr><td align=right><b>Field:</b></td><td>" + link.field + "</td></tr>";
                }
            html += "</table>";
            return html;
        },
        
        addWorkItemSummaryTip: function(wiSummary, elementToConnect){
            var html = this.getWorkItemSummaryTipHTML(wiSummary);
            return this.addTooltip(html, elementToConnect);
        },
        
        getWorkItemSummaryTipHTML: function(wiSummary){
            var html = "<table style=\"font-size: smaller; font-family:Verdana;\" border=0>";
            html += "<tr><td align=right><b>Id:</b></td><td>" + wiSummary.id + "</td></tr>";
            html += "<tr><td align=right><b>Name:</b></td><td>" + wiSummary.summary + "</td></tr>";
            html += "<tr><td align=right><b>Type:</b></td><td>" + wiSummary.typeName + "</td></tr>";
            html += "<tr><td align=right><b>Owner:</b></td><td>" + wiSummary.ownerName + "</td></tr>";
            html += "<tr><td align=right><b>State:</b></td><td>" + wiSummary.stateName + "</td></tr>";
            html += "<tr><td align=right><b>Originator:</b></td><td>" + wiSummary.creatorName + "</td></tr>";
            html += "</table>";
            return html;
        },
        
        addWorkItemTip: function(workItem, elementToConnect){
            var html = this.getWorkItemTipHTML(workItem);
            return this.addTooltip(html, elementToConnect);
        },
        
        getWorkItemTipHTML: function(wi){
            var html = "<table style=\"font-size: smaller; font-family:Verdana;\" border=0>";
            html += "<tr><td align=right><b>Name:</b></td><td>" + wi.summary + "</td></tr>";
            html += "<tr><td align=right><b>Id:</b></td><td>" + wi.id + "</td></tr>";
            html += "<tr><td align=right><b>Description:</b></td><td>" + wi.description + "</td></tr>";
            html += "<tr><td align=right><b>Type:</b></td><td>" + wi.workItemType.label + "</td></tr>";
            html += "<tr><td align=right><b>Owner:</b></td><td>" + wi.owner.label + "</td></tr>";
            html += "<tr><td align=right><b>State:</b></td><td>" + wi.state.label + "</td></tr>";
            html += "<tr><td align=right><b>Originator:</b></td><td>" + wi.creator.name + "</td></tr>";
            html += "</table>";
            return html;
        },
        
        addArtifactTipInSizing: function(artifact, elementToConnect, widget){
            var html = this.getArtifactTipHTMLInSizing(artifact);
            var span = dojo.doc.createElement("span");
            widget.tooltip = new com.ibm.asq.common.web.ui.internal.util.FunctionalTooltip({connectId: elementToConnect,
                																			  label: html
            																				 }, span);
            //		return this.addTooltip(html, elementToConnect);
        },
        
        getArtifactTipHTMLInSizing: function(artifact){
            var html = "<table style=\"font-size: smaller; font-family:Verdana;\" border=0>";
            html += "<tr><td align=right><b>Name:</b></td><td>" + artifact.name + "</td></tr>";
            html += "<tr><td align=right><b>Id:</b></td><td>" + artifact.artifactId + "</td></tr>";
            html += "<tr><td align=right><b>Description:</b></td><td>" + artifact.description + "</td></tr>";
            html += "<tr><td align=right><b>Owner:</b></td><td>" + artifact.owner.name + "</td></tr>";
            html += "</table>";
            return html;
        }        
    };
    
})();
