/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactOptionsContentArea");

dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactOptionsTableView");

(function(){
	
	var getCurrentProjectArea  = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
	var Constants                  = com.ibm.rqm.planning.web.client.internal.Constants;
	var Utility                    = com.ibm.rqm.planning.web.ui.internal.util.Utility;
	var PreferenceClient           = com.ibm.rqm.planning.web.client.internal.PreferenceClient;
	var ArtifactOptionsTableView   = com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactOptionsTableView;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactOptionsContentArea", 
        com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea, {
        
		constructor: function(){
			//this.csHelpId = "rqm_admin_artifact_options.html";
			
			//internal
			this._options = [];           //local objects organized for use table
			this._preferences = [];       //server objects
			this._types = [
                Constants.MODEL_TYPE_PLAN, 
				Constants.MODEL_TYPE_CASE, 
				Constants.MODEL_TYPE_SCRIPT, 
				Constants.MODEL_TYPE_EWI,
				Constants.MODEL_TYPE_RESULT
			];
			
			this._lockableTypes = [
                Constants.MODEL_TYPE_PLAN, 
                Constants.MODEL_TYPE_CASE, 
                Constants.MODEL_TYPE_SCRIPT			
			];
			
		},
		
		postCreate: function(){
            this.inherited("postCreate", arguments);
			this.csHelp.setButtonLabel(Constants.getMessage("labelArtifactOptionsHelp"));
        },
        
        setName: function(){
            this.name = Constants.getMessage("labelArtifactOptions");
            dojox.data.dom.removeChildren(this._name);
            var name = document.createTextNode(this.name);
            this._name.appendChild(name);
        },
        
        setText: function(){
            this.text = Constants.getMessage("labelArtifactOptions");
            dojox.data.dom.removeChildren(this._text);
            var text = document.createTextNode(this.text);
            this._text.appendChild(text);
        },
        
        setContent: function(){
			this._showOptionsTable();
			this._loadOptions();
        },
		
		_showOptionsTable: function(){
			var args = {
				artifactOptions: [],
				onChangeListener: dojo.hitch(this, "_onOptionChanged"),
                editable: this.isEditable(),
                enableRowSelection: false		
			}
			this._optionsTable = new ArtifactOptionsTableView(args);			
			this._content.appendChild(this._optionsTable.domNode);
		},
		
		_loadOptions: function() {
			var args = {
				projectAreaItemId: getCurrentProjectArea().itemId,
				keyPrefix: Constants.ARTIFACT_OPTIONS_PREFERENCE_KEY_PREFIX
			}
			var handler = new ServiceResponseHandler(this, "_setPreferences", "_error");
			PreferenceClient.getPreferenceEntries(handler, args.projectAreaItemId, args.keyPrefix);
		},
		
		_setPreferences: function(preferences) {
            this._preferences = preferences || [];
			this._options = this._buildOptions();
            this.setDirty(false);
            this._refreshTable();			
		},
		
		_buildOptions: function() {
			var options = [];
			for (var i = 0; i < this._types.length; i++) {
				var typeKey = this._types[i].substr(this._types[i].lastIndexOf(".") + 1);
				var lockingPref = this._getPreference(typeKey, Constants.ARTIFACT_OPTIONS_PREFERENCE_LOCKING_KEY);
				var searchPref = this._getPreference(typeKey, Constants.ARTIFACT_OPTIONS_PREFERENCE_SEARCH_LIMIT_KEY);
				var option = {
					id: 1,
					type: this._types[i],
					typeKey: typeKey,
					typeName: Utility.getDisplayNameForType(this._types[i]),
					lockingAvailable: true,
					lockingEnabled:  lockingPref ? (lockingPref == "true") : Constants.ARTIFACT_OPTIONS_PREFERENCE_LOCKING_DEFAULT,
					maxResults: searchPref ? new Number(searchPref) : Constants.ARTIFACT_OPTIONS_PREFERENCE_SEARCH_LIMIT_DEFAULT					
				}
				//handle types which don't support locking
				if (!this._isLockableType(this._types[i])) {
                    option.lockingAvailable = false;
                    option.lockingEnabled = false;
				}
				options.push(option);
			}
			return options;
		},
		
		_getPreference: function(typeName, key) {
			for (var i = 0; i < this._preferences.length; i++) {
				var fullKey = Constants.ARTIFACT_OPTIONS_PREFERENCE_KEY_PREFIX + "." + typeName + "." + key;
				if (this._preferences[i].key == fullKey) {
					return this._preferences[i].value;
				}
			}
			return null;
		},
		
        //@Abstract: Override to refresh the table when we're in display.
        refresh: function(){
			if(this._optionsTable)
                this._refreshTable();
        },

		_refreshTable: function() {
			this._optionsTable.refresh(this._options);
		},
		
        _onOptionChanged: function(optionObject) {
            this.setDirty(true);
        },		
		
        _sortOptions: function(opt1, opt2) {
            if (opt1.id < opt2.id) {
                return -1;
            } else if (opt1.id > opt2.id) {
                return 1;
            }
            return 0;
        },
		
		_isLockableType: function(type) {
			var lockableTypes = this._lockableTypes || [];
			for (var i = 0; i < lockableTypes.length; i++) {
				if (lockableTypes[i] == type) {
					return true;
				}
			}
			return false;
		},
        
        getValues: function(){
            return [];
        },
			
		onValidate: function() {
            var result = {
				message: null,
				pass: false
			}
			var newOptions = this._optionsTable.getOptions() || [];
            for (var i = 0; i < newOptions.length; i++) {
                var typeOpts = newOptions[i];
				var maxResults = new Number(typeOpts.maxResults);
                if (isNaN(maxResults)) {
                    result.message = Constants.getMessage("msgInvalidField", Constants.getMessage("labelMaxResults"), "non-negative number", typeOpts.typeName);
                    break;					
				} else if (maxResults < 100) {
                    result.message = Constants.getMessage("msgInvalidField", Constants.getMessage("labelMaxResults"), "non-negative number of at least 100", typeOpts.typeName);
                    break;					
				}
            }
            if (result.message != null) {                
                result.pass = false;
            } else {
                result.pass = true;
				this._options = newOptions;
            }            	
			return result;		
		},
		
		onSave: function(){
            if (this.isDirty()) {				
				this._doSave();          
            }
		},
		
        _doSave: function(){
			this.parent.addSaveParticipantNum();
            //var key = Constants.ARTIFACT_OPTIONS_PREFERENCE_KEY;
            //var value = dojo.toJson(this.artifactOptions, true, "   ");		
			
			var keys = [];
			var values = [];
			for (var i = 0; i < this._options.length; i++) {
				var opt = this._options[i];
				//locking preference
				if (this._isLockableType(opt.type)) {
					keys.push(Constants.ARTIFACT_OPTIONS_PREFERENCE_KEY_PREFIX + "." + opt.typeKey + "." + Constants.ARTIFACT_OPTIONS_PREFERENCE_LOCKING_KEY);
					values.push(new String(opt.lockingEnabled));
				}
                //max results preference
                keys.push(Constants.ARTIFACT_OPTIONS_PREFERENCE_KEY_PREFIX + "." + opt.typeKey + "." + Constants.ARTIFACT_OPTIONS_PREFERENCE_SEARCH_LIMIT_KEY);
                values.push(new String(opt.maxResults));				
			}	

			var self = this;
			var handler = {
				success: function(results){
					self.parent.saveFinished();
					self.setDirty(false);
					//self._setPreferences(results);
				},
				error: function(e){
					self.parent.saveFinished();
					self._error(e);
				}
			}
		
            var responsehandler = new ServiceResponseHandler(handler, "success", "error");
            PreferenceClient.savePreferenceEntries(responsehandler, getCurrentProjectArea().itemId, keys, values);         
        },
		
		_error: function(error) {
			var message = error && error.message ? error.message : Constants.getMessage("msgSysPropErrorNoDetails");
			this.setErrorMessage(Constants.getMessage("msgSysPropError"), message);
		}
    });
    
})();
