/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.BlockingDefectsPopUp");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.table.BlockingPopUpDefectTableView");
dojo.require("com.ibm.rqm.defects.web.client.DefectsManager");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.team.repository.web.foundation.Status");

var Constants						= com.ibm.rqm.planning.web.client.internal.Constants;
var BlockingPopUpDefectTableView	= com.ibm.rqm.execution.web.ui.internal.view.table.BlockingPopUpDefectTableView;
var AlertDialog 					= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
var ProgressDialog              	= com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
var Status                      	= com.ibm.team.repository.web.foundation.Status;
var getDefectsManager 				= com.ibm.rqm.defects.web.client.DefectsManager.getInstance;

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.common.BlockingDefectsPopUp",[dijit._Widget, dijit._Templated], 
{
	templateString: '<div class= "com-ibm-asq-common-tableViewer" dojoAttachPoint="_fileContainer">' +
					'    <div class="" dojoAttachPoint="_messageNode"></div>' +
					'    <div class="com-ibm-asq-common-tableViewer table-content-node" dojoAttachPoint="_table">' +
					'        <table><thead><th width="10%"></th><th width="10%">${labelid}</th><th width="40%">${labelsummary}</th><th width="30%">${labelstate}</th></thead>' +
					'            <tbody dojoAttachPoint="_tableBody"></tbody>' +
					'        </table>'+
					'    </div>'+
					'</div>',
	
	labelok: Constants.getMessage("labelOk"),
	labelcancel: Constants.getMessage("labelCancel"),
	labeltitle: Constants.getMessage("editBlockingDefects"),
	labelsummary: Constants.getMessage("labelSummary"),
	labelid: Constants.getMessage("labelId"),
	labelstate: Constants.getMessage("labelState"),
	
	postCreate: function(){
		this._messageNode.innerHTML = "Select the non-blocking defects";
	},

	initialize: function(args){
        this.ewis = args.ewis;
		this.fireToRTC = args.fireToRTC;
		this.defects = args.defects;
		this.okCallback = args.okCallback;
		this.cancelCallback = args.cancelCallback;
		this.addedDefectIds = [];
		this.removedDefectIds = [];
		this._showAsDialog();
		this.showDefects();
	},
	
	_showAsDialog: function() {
        var dialogArgs = {
            okCallback: dojo.hitch(this, "_okClick"),
            cancelCallback: dojo.hitch(this, "_cancelClick"),
			style: "z-index: 990;",
			zIndex: 990
        };
        
        this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
        this._dialog.setWidth("800px");
		this._dialog.setHeight("400px");
        this._dialog.setSecondaryTitle(this.labeltitle);
        this._dialog.getOkButton().disabled = false;
		this._dialog.open(true);		
	},
	
	showDefects: function() {
		if (this.fireToRTC) {
			ProgressDialog.doRunWithProgress(Constants.getMessage("labelQueryAssociatedBlockingDefects"), dojo.hitch(this, function(monitor) {
				this.ewiItemIdArray = [];
				for (var ewi in this.ewis){
					this.ewiItemIdArray.push(this.ewis[ewi].itemId);
				}
				getDefectsManager().queryAssociatedDefects("com.ibm.rqm.execution.linktype.ewiblockingdefect", false, this.ewiItemIdArray, monitor, dojo.hitch(this, function(status, response) {
					monitor.done();
					if	(status.isOK()){
						var ewiItemIdArr = this.ewiItemIdArray;
						var temp = [];
						if (dojo.isArray(response)){
							for(var i = 0; i < ewiItemIdArr.length; i++){
								for(var j = 0; j < response.length; j++){
									if(temp[response[j].reference] && response[j].ewiItemId == ewiItemIdArr[i] )
										temp[response[j].reference].count++;
									else if (response[j].ewiItemId == ewiItemIdArr[i]){
										temp[response[j].reference] = response[j];
										temp[response[j].reference].count = 1;
										temp[response[j].reference].isBlocking = true;
									}
								}
							}
						}
						var defects = [];
						for (var defect in temp) {
							if(temp[defect].count == ewiItemIdArr.length)
								defects.push(temp[defect]);
						}
						var args= {
		                    ewis: this.ewis,
							defects: defects,
							parentEditor: this,
							okCallback: dojo.hitch(this, this._onOk),
							cancelCallback: dojo.hitch(this, this._onCancel)
						}
						this.defectTableView = new BlockingPopUpDefectTableView(args);
						dojox.data.dom.removeChildren(this._dialog.contentContainer);
						this._dialog.contentContainer.appendChild(this.defectTableView.domNode);
					} else if(!status.isOK() && status.getSeverity() != Status.CANCEL) {
						status.log();
						var alertDialog = new AlertDialog({
			                'message': Constants.getMessage("labelUnableCreateWorkItem", status.getMessage())
			            	});
		            	alertDialog.open();
					}
				}));
			}));
		} else {
			var args = {
				defects: this.defects, 
				parentEditor: this,
				ewis: this.ewis
			};
			
			this.defectTableView = new BlockingPopUpDefectTableView(args);
			this._dialog.contentContainer.appendChild(this.defectTableView.domNode);
		}
	},
	_appendInDefectList: function(data,type){
		for(var i=0;i<data.length;i++){
			if(data[i].statusId == 5){//dont bother with closed defects
				var alertDialog = new AlertDialog({
                'message': Constants.getMessage("msgClosedDefect")
            	});
            	alertDialog.open();
				continue;
			}
			var found = false;
			for(var j=0;j<this.defects.length;j++){
				if((!data[i].isRTCDefect && this.defects[j].id == data[i].id) || (data[i].isRTCDefect && this.defects[j].draftUri == data[i].draftUri)){
					found = true;
					break;
				}
			}
			if(!found){
				var defect = {id:data[i].id,status:data[i].status,summary:data[i].summary,isBlocking:true};
				if (data[i].isRTCDefect) {
					defect.draftUri = data[i].draftUri;
				}
				this.defects.push(defect);
				if(!data[i].isRTCDefect) 
					this.addedDefectIds.push(data[i].id);
				//if removed and then added again
				for(var j=0;j<this.removedDefectIds.length;j++){
					if(this.removedDefectIds[j] == data[i].id)
						this.removedDefectIds.splice(j,1);
				}
			}
		}
		this.defectTableView.setContent(this.defects);
	},
	_updateDefectStatus:function(defectId,blocking){
		if(!blocking){
			for(var i=0;i<this.defects.length;i++){
				if(this.defects[i].id == defectId)
					this.defects.splice(i,1);
			}
			//if added and then removed
			for(var j=0;j<this.addedDefectIds.length;j++){
				if(this.addedDefectIds[j] == defectId)
					this.addedDefectIds.splice(j,1);
			}
			this.removedDefectIds.push(defectId);
			this.defectTableView.setContent(this.defects);
		}
	},
	_okClick: function(){
		if(this.okCallback)
			this.okCallback(this.ewis,this.addedDefectIds, this.removedDefectIds );
	},
	
	_cancelClick: function(){
		if(this.cancelCallback)
			this.cancelCallback();
	},
	
	_error: function(errorObj){
		console.error("Error in BlockingDefectsPopUp "+errorObj.message);
	},
	
	_closePopupDialog: function(){
		if(this._dialog)
			this._dialog.close();
	}
});
	
