/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.common.CollapseExpand");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.Section");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function() { // module container

var	ImageCache			= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
var	TooltipBuilder		= com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder;
var CSHelp				= com.ibm.asq.common.web.ui.internal.util.CSHelp;

dojo.declare(
	"com.ibm.rqm.process.web.ui.internal.view.common.CollapseExpand",
	[dijit._Widget, dijit._Templated], 
	{
		
	templatePath: dojo.moduleUrl("com.ibm.rqm.process.web", "ui/internal/view/templates/common/CollapseExpand.html"),
	
	constructor: function() {
		///////////////////////////////////////////////////////////////////////////////////
		// Initialization parameters	
		//////////////////////////////////////////////////////////////////////////////////	
		this.label = null;
		this.labelTooltip = null;
		//this.id = null;
		this.sectionWidgets = null;
		this.open = false;		
		this.showBorder = false;
		this.flagStatus = 0;
		
		this.editorOpen = false;
		this.editorAvailable = true;
		this.showActionsForEditor = true;
		
		this.cshelpID = null;
		
		//////////////////////////////////////////////////////////////////////////////////
		// Actions
		//////////////////////////////////////////////////////////////////////////////////
		this.actions = [];	//action objects should include: 'text' && ('href' || ('obj' && 'function'))
		
		//////////////////////////////////////////////////////////////////////////////////
		// States
		//////////////////////////////////////////////////////////////////////////////////
		this._isExpanded = false;
		this._isEditorOpen = false;
				
		//////////////////////////////////////////////////////////////////////////////////
		// Expand/Edit options
		//////////////////////////////////////////////////////////////////////////////////
		this.isExpandable = true;
		this.isEditable = true;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Called on creation
	///////////////////////////////////////////////////////////////////////////////////
	postCreate: function() { },
	
	initialize: function(args) {
		//allow mixin of args on initialize()
		for (var element in args) {
			this[element] = args[element];
		}

		//if editable (editor available) connect proper events
		if (this.editorAvailable && this.editorAvailable == true) {
			//this._editImg.src = ImageCache.EDIT;
			dojo.connect(this.labelSpan, "onclick", this, this._toggleEditor);
//			dojo.addClass(this.sectionGroup, "cursor-pointer");

			if (this.sectionWidgets) {
	            if ( this.sectionWidgets.length == 1 ) {
	                var editor = this.sectionWidgets[0];
	                if (editor && editor["setNameListenerNode"]) {
	                    editor.setNameListenerNode(this.labelSpan);
	                }
	            }
			}
			
		} else {
			this.editorOpen = false;
		}
		this._initializeWidget();
	},

	_initializeWidget: function() {
		//set initial collapsed/expanded state
		if (this.open && this.open == true) {
			var children = this.sectionWidgets[0].domNode.childNodes;
			for (var i = 0; i < children.length; i++) {
				var child = children[i];
				if ((child.className != null) && (child.className != "")) {
					if (dojo.hasClass("hierarchy") && dojo.hasClass("hidden") ) {
						dojo.removeClass("hierarchy");
					}
				}
			}
			this.expand();
		}
		else
		{
			this.collapse();
		}
		//set initial editor state
		if (this.editorOpen) {
			this._cshelp.className = "cshelp";
			if (this.showBorder) {
				this._titleRow.className = "title-row-expanded";
				this._sectionGroupBorderTop.className = "border-top";
				this._sectionGroupBorderBottom.className = "border-bottom hidden";		
				this._sectionGroupContainer.className = "section-group-collapsed";
			} else {
				var children = this.sectionWidgets[0].domNode.childNodes;
				for (var i = 0; i < children.length; i++) {
					var child = children[i];
					if ((child.className != null) && (child.className != "")) {
						if (dojo.hasClass("editor") &&  dojo.hasClass("shaded") && dojo.hasClass("hidden") ) {
							child.className = "editor shaded bottom-border";
							this.labelSpan.className = "label cursor-pointer shaded";
						}
					}
				}
			}
		}
		
		if (this.icon) {
			this.iconImg.src = this.icon;
		}
		
		if (this.cshelpId != null) {
			var cshelp = new CSHelp();
			cshelp.contextID = this.cshelpId;
			this._cshelp.appendChild(cshelp.domNode);
		}		

		this.setLabel(this.label);
		if ( this.labelTooltip != null ) {
			this.setLabelTooltip(this.labelTooltip);	
		}
		
		if (this.sectionWidgets) {
			for (var i= 0; i < this.sectionWidgets.length; i++) {
				this.sectionsDiv.appendChild(this.sectionWidgets[i].domNode);
			}
		}
		this._showActions();	
	},
			
	///////////////////////////////////////////////////////////////////////////////////
	// API
	///////////////////////////////////////////////////////////////////////////////////
	collapse: function() {
		this._titleRow.className = "title-row-collapsed";
		if (!this.editorOpen || !this.showActionsForEditor) {
			this._actionsBar.className = "hidden";	
		}		
		if (!this.editorOpen) {
			this._sectionGroupContainer.className = this.showBorder ? "section-group-collapsed top-border-line" : "section-group-collapsed" ;
		}
		this._sectionGroupBorderTop.className = "";
		this._sectionGroupBorderBottom.className = "";		
	},
	
	expand: function() {
		if (this.showBorder) {
			this._titleRow.className = "title-row-expanded";
			this._sectionGroupBorderTop.className = "border-top";
			this._sectionGroupBorderBottom.className = "border-bottom hidden";		
		}
		this._actionsBar.className = "actions-bar";
		this._sectionGroupContainer.className = "section-group-expanded";
	},
	
	isExpanded: function() {
		return this._isExpanded;
	},
	
	setExpression: function(/* ExpressionDTO[] */ expressions) {
		var hasExpression= false;
		for (var i= 0; i < this.sectionWidgets.length; i++) {
			hasExpression|= this.sectionWidgets[i].setExpression(expressions);
		}
		this._setDisplay(hasExpression);
	},
	
	addAction: function(action) {
		this.actions.push(action);
		this._showActions();
	},
	
	registerAction: function(text, href, obj, method) {
		var action = {
			text: text, href: href, obj: obj, method: method
		}
		this.addAction(action);
	},
	
	setLabel: function(labelText)
	{
		this.label = labelText;
		this.labelSpan.innerHTML = this.label;		
	},

	setLabelTooltip: function(tooltip)
	{
		TooltipBuilder.addLabelTooltip(tooltip, this.labelSpan, this);
	},
	
	setWorkItemUrl: function(wi, name, url)
	{
		dojox.data.dom.removeChildren(this._workItemUrl);
		
		var anchor = document.createElement("A");
		anchor.id = this.label;
		anchor.className = "";
		anchor.href = url;
		var tn = document.createTextNode(name);
		anchor.appendChild(tn);
		this._workItemUrl.appendChild(anchor);
		
		TooltipBuilder.addWorkItemTip(wi, anchor);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Expand/Edit option capability setters
	///////////////////////////////////////////////////////////////////////////////////
	
	//canEditor: function(value) {
	setEditable: function(value) {
		//can be edited
		if(value == true) {
			this.isEditable = true;
			this._editImg.src = ImageCache.EDIT;
		}
		//can't be edited
		if(!value) {
			this.isEditable = false;
			this._editImg.src = "";			
		}
	},
	
	//canExpand: function(value) {
	setExpandable: function(value) {
		//can be expanded
		if(value && value == true) {
			this.isExpandable = true;
			this.twistieImg.src = this._isVisible() ? ImageCache.TWISTIE_OPEN : ImageCache.TWISTIE_CLOSED;
		} else {
			this.isExpandable = false;
			this.twistieImg.src = "";
		}
	},
	
	closeEditor: function() {
		var section = this.labelSpan.parentNode.parentNode.parentNode.childNodes[3].childNodes[0];
		for (var i = 0; i < section.childNodes.length; i++)
		{
			var child = section.childNodes[i];
			if ((child.className != null) &&(child.className != "")){
				if (child.className == "editor shaded bottom-border")
				{
					child.className = "editor shaded hidden";
					this.labelSpan.className = "label";
				}
			}
		}
	},
	
	needTitleRow: function(show) {
		if(!show) {
			this._titleRow.className = "hidden";
		}
		else
			this._titleRow.className = "title-row-collapsed";
	},	
	
	_showActions: function() {
		dojox.data.dom.removeChildren(this._actionsBar);
		if ( this.actions != null && this.actions.length > 0 ) {
			for ( var i = 0; i < this.actions.length; i++ ) {
				var action = this.actions[i];
				var actionLink = document.createElement("a");
				actionLink.innerHTML = action.text;
				if ( action.href ) {
					actionLink.href = action.href;
				} else if ( action.obj && action.method ) {
					dojo.connect(actionLink, "onclick", action.obj, action.method);	
				}
				this._actionsBar.appendChild(actionLink);
			}
		}
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Event handlers
	///////////////////////////////////////////////////////////////////////////////////
	_toggleEditor: function(e)
	{
		if(this.isEditable) {
			e.preventDefault();
			var widget =  this.sectionWidgets[0]; 
			if (widget.maximizedForm.className == "editor") {
				widget.maximizedForm.className += " hidden";
				widget.minimizedForm.className = "editor";
				this.sectionGroup.className = "";
			}
			else {
				widget.minimizedForm.className += " hidden";
				widget.maximizedForm.className = "editor";
				this.sectionGroup.className = "expanded";
			}
			if (this._cshelp.className == "hidden") {
				this._cshelp.className = "cshelp";
			}
			else {
				this._cshelp.className = "hidden";
			}
		}
	}
});

})();
