/******************ExecutionConsole******************************************/
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.ExecutionConsole");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionTableView");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionWorkItemClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.execution.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container 	

var ServiceResponseHandler			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var ExecutionTableView 				= com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionTableView;
var ExecutionClient 				= com.ibm.rqm.execution.web.client.internal.execution.ExecutionClient;
var ExecutionWorkItemClient			= com.ibm.rqm.execution.web.client.internal.execution.ExecutionWorkItemClient;
var ERGClient 						= com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient;
var GlobalEvents 					= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var ImageCache  	   				= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility     		   			= com.ibm.rqm.execution.web.ui.internal.util.Utility;
var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
var ERGMapper						= com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper;

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.common.ExecutionConsole",
			 [dijit._Widget, dijit._Templated], 
{
		
	templateString: "<div class=\"\">"+
					"<div dojoAttachPoint=\"_table\"></div>"+
					"</div>",
	
	consoleNode: true,
	
	postCreate: function(){
		this.initialize();
	},
	
	getTableActions: function(){
	 var actions = [];       
	    actions.push({
	        actionId: "refresh",
	        label: Constants.getMessage("labelRefresh"),
	        actionHandler: dojo.hitch(this, "onRefresh"),
	        iconClass: "refresh-icon-image",
			disabled: false
	    });
		
		return actions;
	},
	
	initialize: function(){
		this._createExecutionTable();
		var responseHandler = new ServiceResponseHandler(this, "_createTable", "_error");
		ExecutionWorkItemClient.getRequestDTO(responseHandler, {});
	},
	
	_createExecutionTable: function () {
		if(!this._executionTable)
		{	
			var args = {
				requests: [],				
				actions: this.getTableActions(),
				queryName: this.name
        	}
			if(this.ergId){
				args.ergId = this.ergId;
			}
			this._executionTable = new ExecutionTableView(args);
            this._executionTable.viewHeaderNode.setCSHelpId("rqm_exec_console.html");
			this._executionTable.viewHeaderNode.setCSHelpButtonLabel(com.ibm.rqm.planning.web.client.internal.Constants.getMessage("labelExecutionTableViewHelp"));

			this._executionTable.setInProgress(Constants.getMessage("msgQueryTableList", this.name));
			//display the table
			dojox.data.dom.removeChildren(this._table);
			this._table.appendChild(this._executionTable.domNode);
		}
	},
	
	_createTable: function(requests){
		this.requests = requests;
		this._executionTable.setConsoleNode(this.consoleNode);
		this._executionTable.setInProgress(Constants.getMessage("msgQueryTableList", this.name));
		if(this.consoleNode){
			this.requests = [];
			if(requests){//Since IMPORT requests are also getting returned
				for(var i=0;i<requests.length;i++){
					if(requests[i].requestType == "EXECUTE"){//non-nls
						this.requests.push(requests[i]);
					}
				}	
			}
			requests = this.requests;
		}
		this._executionTable.setContent(requests ? requests : []);
		if(this.consoleNode)
			this._executionTable.showTable();
		this._executionTable.stopInProgress();
		if(this.consoleNode){
			var responseHandler = new ERGMapper(this, "_setAllERGs", "_error");
			ERGClient.getExecutionResultGroupDTOs(responseHandler, {});
		}	
		
	},
	
	_setAllERGs: function(ergs){
		this.suiteRequests = [];
		if(ergs){
			for(var i=0;i<ergs.length;i++){
				var suiteRequest ={};
				suiteRequest.scriptTypeName = ergs[i].name;
				suiteRequest.id = ergs[i].id;
				suiteRequest.scriptName = Constants.getMessage("labelMultiple");
				suiteRequest.configuration = Constants.getMessage("labelMultiple");
				suiteRequest.machine = Constants.getMessage("labelMultiple");
				suiteRequest.testCaseName = ergs[i].name;
				suiteRequest.icon = ImageCache.TEST_SUITE;
				suiteRequest.modified = ergs[i].modified;
				suiteRequest.link = Utility.runSuite({groupId: ergs[i].id});
				suiteRequest.progress = 0;
				var suiteProgress = 0;
				var suiteState = null;
				for(var j=0;j<(ergs[i].requests ? ergs[i].requests.length: 0);j++){
					suiteProgress += parseInt(ergs[i].requests[j].progress);
					if(	ergs[i].requests[j].state && 
						ergs[i].requests[j].state.id == "com.ibm.rqm.executionframework.common.requeststate.cancelled" && 
						ergs[i].sequentialExecution && 
						suiteState == null){
						//In case of sequential Exec if one req is cancelled, suite is cancelled
						suiteState = {
							id: "com.ibm.rqm.executionframework.common.requeststate.cancelled",
							name: Constants.getMessage("msgSuiteExecCancelled") 
						}
					}
				}
				if(suiteProgress != 0){
					suiteRequest.progress = parseInt(suiteProgress /  ergs[i].elements.length) ; 
				}
				//Build Suite state here
				if(suiteState == null){
					suiteRequest.state = {
						id: "com.ibm.rqm.executionframework.common.requeststate.taken",
						name: Constants.getMessage("msgSuiteExecRunning")
					}
					if(!suiteRequest.progress || suiteRequest.progress == 0){
						suiteRequest.state.id = "com.ibm.rqm.executionframework.common.requeststate.nottaken";
						suiteRequest.state.name = Constants.getMessage("msgSuiteExecQueued"); 
					}else if(suiteRequest.progress == 100){
						suiteRequest.state.id = "com.ibm.rqm.executionframework.common.requeststate.complete";
						suiteRequest.state.name = Constants.getMessage("msgSuiteExecCompleted");
					}
				}else{
					suiteRequest.state = suiteState;
				}	
				this.suiteRequests.push(suiteRequest);
			}
		}
		for(var z=0;z<this.suiteRequests.length;z++){
			this._executionTable.appendItem(this.suiteRequests[z]);
		}
	},
	
	onRefresh: function(){
		this.initialize();
	},

//TODO: Add updates to the console.
//	 // handler on artifacts changed
//    onArtifactChanged: function(changeType, artifactType, artifactArray) {
//        // only care about requests
//        if (artifactType == Constants.TYPE_REQUEST) {
//            if (changeType == GlobalEvents.ArtifactsAdded) {
//                // add requests to the table
//                if (!this.requests) {
//                    this.requests = artifactArray;
//                } else {
//                    this.requests = this.requests.concat(artifactArray);
//                }
//                if(this._executionTable)
//					this._executionTable.setContent(this.requests);
//            } else {
//                // iterate through the array of artifacts
//                for (var i = 0; i < artifactArray.length; i++) {
//                    // iterate through the array of requests in the table
//                    for (var j = 0; j < this.requests.length; j++) {
//                        // handle the matching requests
//                        if (artifactArray[i].id == this.requests[j].id) {
//                            if (changeType == GlobalEvents.ArtifactsUpdated) {
//                                // swap requests on update
//                                this.requests.splice(j, 1, artifactArray[i]);
//                                if(this._executionTable)
//									this._executionTable.setContent(this.requests);
//                            } else if (changeType == GlobalEvents.ArtifactsDeleted) {
//                                // remove requests from table on delete
//                                this.requests.splice(j, 1);
//                                if(this._executionTable)
//									this._executionTable.setContent(this.requests);
//                            }
//                            break;
//                        }
//                    }
//                }
//            }
//        }
//    },
	
	_error: function(errorObj){
		console.error("ERROR in Execution console "+errorObj.message);
	}
	
});
	
})();