/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.TestCaseConfigSelector");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

(function() {
var Constants           = com.ibm.rqm.planning.web.client.internal.Constants;
var CategoryClient      = com.ibm.rqm.planning.web.client.internal.CategoryClient;
var SelectUtility       = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
var FormNumericField    = com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField;
var CategorySelection   = com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection;
var EditorUtil          = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.TestCaseConfigSelector",
             [dijit._Widget, dijit._Templated], 
{
    templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/TestCaseConfigSelector.html"),                                  
				    

    height: "450px", 
    width: (dojo.isFF == 3) ? "430px" : "500px",
	
	labelWeight: Constants.getMessage("labelWeight"),
	labelPoints: Constants.getMessage("labelPoints"),
	labelowner: Constants.getMessage("labelowner"),
	
    postCreate: function(){
		this._categorySelect = new com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection({
                 artifactType: Constants.MODEL_TYPE_CASE           
        });             
        this._categoriesNode.appendChild(this._categorySelect.domNode);   
		EditorUtil.initContributors(this._owner, null);
		
	    dojox.data.dom.removeChildren(this._weight);
        this.weightWidget = new FormNumericField({
            fieldLabel: this.labelWeight,
            required: false
        });
		
        this.weightWidget.onValueChanged = dojo.hitch(this, "validate");
        this.weightWidget.setErrorMessage(Constants.getMessage("msgNumericWeightRequired"));
        this.weightWidget.setPromptMessage(Constants.getMessage("msgWeightRequired"));
        this._weight.appendChild(this.weightWidget.domNode);
    },
	
    initialize: function(args){
        this.selectedItems = args.selectedItems;
		var responseHandler = new ServiceResponseHandler(this, "_showAsDialog", "_error");
        CategoryClient.getCategoryTypeSummaries(responseHandler, Constants.MODEL_TYPE_CASE);
    },
    
	_showAsDialog: function(categoryTypeSummaries)
    {
		this._categoryTypeSummaries = categoryTypeSummaries;
		
        if (!this._dialog) {
            var dialogArgs = {
                okCallback: dojo.hitch(this, "_okAction"),
                cancelCallback: dojo.hitch(this, "_cancelAction")
            };
            
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
			this._dialog.setTitle(Constants.getMessage("labelChangeTestCaseProperties"));
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
 
            this._dialog.getOkButton().disabled = false;                                
        }
        this._dialog.open(false);
    },  
	
	_okAction: function(){
		var args = {};
		
        args.uid = [];
		args.categories = [];
        for(var i=0; i<this.selectedItems.length; i++){
            args.uid.push(this.selectedItems[i].itemId);
			
			var categories = this._reCaculateCategories(this.selectedItems[i].categories);
			args.categories.push(categories.join());
			
        }
 
        var option = this._owner.options[this._owner.selectedIndex];
        if (option.text != Constants.getMessage("labelUnassigned")) {
            args.ownerId = option.value;
        }
		
		if (dojo.trim(this.weightWidget.getValue()).length != 0) {
			args.weight = Number(dojo.trim(this.weightWidget.getValue()));
		}
		
        if (this.okCallback) {
            this.okCallback(args);
        }
    },
	
	_cancelAction: function(){
        if (this.cancelCallback) {
            this.cancelCallback();
        }
    },
	
	_reCaculateCategories: function(categories) {
        var nowCategories = this._categorySelect.getSelectedItemIds();
        var oldCategories = categories;
        var newCategories = [];
        for(var i = 0; i < (this._categoryTypeSummaries ? this._categoryTypeSummaries.length:0); i++) {
            var type = this._categoryTypeSummaries[i];
            var newCategory = this._getCategoryByType(type, nowCategories);
            if(!newCategory)
                newCategory = this._getCategoryByType(type, oldCategories);
            if(newCategory)
                newCategories.push(newCategory);
        }
        return newCategories;
    },
  
    _getCategoryByType: function(type, categories) {
        var categoriesItemIds = type.categoryItemIds;
        for(var i = 0; i < (categoriesItemIds ? categoriesItemIds.length:0); i++) {
            for(var j = 0; j < (categories?categories.length:0); j++) {
                if(categoriesItemIds[i] == categories[j])
                    return categories[j];
            }
        }
        return null;
    },
	
	validate: function() {
		if (dojo.trim(this.weightWidget.getValue()).length != 0 && this._dialog) {
			var weight = Number(dojo.trim(this.weightWidget.getValue()));
			if (weight != NaN && weight >= 0 ) {
				this._dialog._yesButton.disabled = false;
			}
			else { 
				this._dialog._yesButton.disabled = true;
			}
		}
	},
	
    _error: function(errorObj){
        console.error("Error in TestCaseConfigSelector " + errorObj.message);
    }
    
});
})();
    
