/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIConfigurationListPanel");
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIConfigurationListPanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationListPanel");
dojo.require("com.ibm.rqm.configuration.web.util.TestConfigurationUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){ // module container

	var TestConfigurationUtil = com.ibm.rqm.configuration.web.util.TestConfigurationUtil;
	var ImageCache = com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
	var Constants  = com.ibm.rqm.planning.web.client.internal.Constants;
	
    dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIConfigurationListPanel", [dijit._Widget, dijit._Templated, com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationListPanel], {
        
        constructor: function(args){
            this.testCases = args.testCases;
			this.testPhase = (args.testPhase != null) ? args.testPhase : null;
            this.parentTestPlanUUID = (args.parentTestPlanUUID != null) ? args.parentTestPlanUUID : null;
			this.isRandomSelectCase = (args.isRandomSelectCase != null) ? args.isRandomSelectCase : true;
			this.existingConfigurations = (args.existingConfigurations != null) ? args.existingConfigurations : null;
			//the test plan UUID for applying added Configurations
			this.configurationAppliedTestPlanUUID = this.parentTestPlanUUID;
        },
        
		postCreate: function(){
            this._initTitle();
			this._initTable();
			this._displayExistingConfigurations();
            this._loadConfigurations();
        },
		
		_displayExistingConfigurations: function(){
			this._testConfigListWidget.appendConfigs(this.existingConfigurations);
		},
		
		//Overriden from ConfigurationListPanel
		_addTestConfigToTable: function(configGroup){
			var newConfigs = this._demarshal(configGroup);
			var addedconfigs = [];
			var exist = null;
			
			for(var i = 0; i < newConfigs.length; i++){
				exist = false;
				for(var j = 0; j < this.existingConfigurations.length; j++){
					if(this._compareConfig(this.existingConfigurations[j], newConfigs[i])){
						exist = true;
						break;
					}
				}
				if(exist == false){
					addedconfigs.push(newConfigs[i]);
				}
			}
            this._testConfigListWidget.appendConfigs(addedconfigs);
			this._displayLoadStatus(false);
        },
		
		_compareConfig: function(config1, config2){
			var counter = 0;
			for(var i = 0; i < config1.length; i++){
				for(var j = 0; j < config2.length; j++){
					if(config1[i].key == config2[j].key && config1[i].value == config2[j].value){
						counter ++
						break;
					}
				}
			}
			if(counter == config1.length){
				return true;
			}
			return false
		},
		
		//Overriden from ConfigurationListPanel
        _initTitle: function(){
			this._title.appendChild(document.createTextNode(Constants.getMessage("labelGeneratedConfigurations")));
			if(this.testCases.length == 1){
				this._titleName.appendChild(document.createTextNode(this.testCases[0].name));
			}else{
				this._titleName.appendChild(document.createTextNode(Constants.getMessage("labelTestCases")));
			}
            dojo.connect(this._nextButton, "onclick", dojo.hitch(this, "onNext"));
            dojo.connect(this._previousButton, "onclick", dojo.hitch(this, "onPrevious"));
            this._executionwork_objImg1.src = ImageCache.EWI;
        },
		
		//Overriden from ConfigurationListPanel
		onNext: function(){
			this._displayLoadStatus(true);
			this.doNext();
			this._displayLoadStatus(false);
		},
		
		//Overriden from ConfigurationListPanel
        doNext: function(){
            var configs = this._testConfigListWidget.getTableViewer().getContent();
			
            var params = null;
            params = {
                testCases: this.testCases,
                owner: this.owner,
                configs: configs,
                testPhase: this.testPhase,
                parentTestPlanUUID: this.parentTestPlanUUID,
				isRandomSelectCase: this.isRandomSelectCase
            }
            this.controller.doNext(params);
        },
        
        _error: function(errorObj){
            confirm(errorObj.message);
        }
    });
    
})();
