/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.RecentItemsNavigator");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.RecentItemsNavigator");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container

var Constants    			= com.ibm.rqm.planning.web.client.internal.Constants;
var Utility					= com.ibm.rqm.planning.web.ui.internal.util.Utility;

/**
 * Widget to display list of link elements in the LeftNavigation.
 */
dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.construction.RecentItemsNavigator",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/construction/Navigator.html"),
//		templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/construction/Navigator.css"),

		constructor: function() {
			this.controller = null;
			this._items = [];
		},

		postCreate: function(args) {
			this.parentController = args.parentController;
		},

		_items: [],
		_currentQuery: null,
		
		/**
		 * Adds an element to the widget.
		 */
		setCurrentQuery: function(query) {
					
			this._currentQuery = query;						
											             
            var anchor= document.createElement("a");
			anchor.title = query.name;
			anchor.innerHTML = query.name;
			if ( query.type == Constants.TYPE_GROUP ) {
				anchor.href = Utility.getTestGroupsURI(query.name);
			} else if ( query.type == Constants.TYPE_CASE ) {
				anchor.href = Utility.getTestCasesURI(query.name);
			} else if ( query.type == Constants.TYPE_SCRIPT ) {
				anchor.href = Utility.getTestScriptsURI(query.name, Utility.CONSTRUCTION_PERSPECTIVE);
			}
			 		
			//add the link to the list
			dojox.data.dom.removeChildren(this.currentQueryDiv);
			this.currentQueryDiv.className = "element-div-selected";
			this.currentQueryDiv.appendChild(anchor);
			//deselect any items
			this._selectItem("");
			//this.elementListDiv.firstChild.insertBefore(elementDiv);		            
		},		
		
		/**
		 * Adds an element to the widget.
		 */
		addItem: function(item) {

			if ( this._items == null || this._items.length == 0 ) {
				dojox.data.dom.removeChildren(this.elementListDiv);	
			}	
						
			//if the item does NOT already exist in the list, add it						
			if ( this._exists(item) == false ) {			
				this._items.push(item);
	             
	            var elementDiv = document.createElement("div");
				elementDiv.className= "element-div-unselected";
	            elementDiv.appendChild( this._getItemLink(item) );
				
				//add the link to the list
				this.elementListDiv.appendChild(elementDiv);
			}
			//always select the item
			this._selectItem(item.name);
			//this.elementListDiv.firstChild.insertBefore(elementDiv);		            
		},		

		/**
		 * Adds a link for the element to the list.
		 */
		_getItemLink: function (item) {
			var linkText = item.name;

			var anchor= document.createElement("a");
			// Construct the link here
			//anchor.href = this._linkPrefix + "&name=" + escape(element.name);
			if ( item.type == Constants.TYPE_GROUP ) {
				//xojo.event.connect(anchor, "onclick", this, this._handleGroupClick);
				anchor.href = Utility.getTestGroupURI(item.id);
			} else if ( item.type == Constants.TYPE_CASE ) {
				anchor.href = Utility.getTestCaseURI(item.id);
			} else if ( item.type == Constants.TYPE_SCRIPT ) {
				anchor.href = Utility.getTestScriptURI(item.id, Utility.CONSTRUCTION_PERSPECTIVE);
			}
			anchor.title = linkText;
			var text = document.createTextNode(linkText);
			anchor.appendChild(text);

			return anchor;
		},
		
		_deselectItems: function(name) {
			this._selectItems("");
		},		
		_selectItem: function(name) 
		{
			/*
			 * Highlight the selected option
			 */
			var itemLinks = this.elementListDiv.childNodes;
			for (var i=0; i < itemLinks.length; i++) {
				var anchorText = itemLinks[i].textContent;
				if(anchorText == name) {
					itemLinks[i].className = "element-div-selected";
					this.currentQueryDiv.className = "element-div-unselected";
				} else {
					itemLinks[i].className = "element-div-unselected";
				}
			}			
		},
		
		/**
		 * Helper function to sort a list of report descriptors
		 * Note: Assume that report names are unique
		 */
		_sort: function() {
			var names = [];
			for (var i = 0; i < this._elementList.length; i++) {
				names[i] = this._elementList[i].element.name;
			}
			names = names.sort();
			var result = [];
			for (var i = 0; i < names.length; i++) {
				result[i] = this._find(names[i]); 
			}
			return result;
		},
		
		_find: function(name) {
			for (var i = 0; i < this._items[i].length; i++) {
				if (this._items[i].name == name) {
					return this._items[i];
				}
			}
			return null;
		},
		
		_exists: function(item) {
			for (var i = 0; i < this._items.length; i++) {
				if (this._items[i].name == item.name) {
					return true;
				}
			}
			return false;
		}		
	}
);

})();
