/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.StatementEditor");

/*
 * This widget extends the Common RichText Editor widget of Quality Manager
 * Usage in Manual Test Editor.
 * author: vasrivas
 */

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor");
dojo.require("dijit._editor.range");
dojo.require("dijit._editor.selection");
dojo.require("dijit._editor.RichText");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.CommentsPlugin");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.AttachmentsPlugin");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.ComparePlugin");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.DataPoolPlugin");

(function(){

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.StatementEditor", [com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor], {
	//@Overridden postCreate of dijit.Editor
	postCreate: function(){
			/*
			 * Here we are directly calling dijitEditor super class since
			 * StatementEditor needs direct access to dijit.RichText
			 */
			if(this.customUndo){
				dojo['require']("dijit._editor.range");
				this._steps=this._steps.slice(0);
				this._undoedSteps=this._undoedSteps.slice(0);
			}
			if(dojo.isArray(this.extraPlugins)){
				this.plugins=this.plugins.concat(this.extraPlugins);
			}
			dijit.Editor.superclass.postCreate.apply(this, arguments);
			this.commands = dojo.i18n.getLocalization("dijit._editor", "commands", this.lang);
			if(!this.toolbar){
				// if we haven't been assigned a toolbar, create one
				var toolbarNode = dojo.doc.createElement("div");
				dojo.place(toolbarNode, this.editingArea, "before");
				this.toolbar = new dijit.Toolbar({}, toolbarNode);
			}
			dojo.forEach(this.plugins, this.addPlugin, this);
			this.onNormalizedDisplayChanged(); //update toolbar button status
		},
	
	//@Overridden events of dijit.RichText
	queryCommandAvailable: function(/*String*/command){
		// summary:
		//		Tests whether a command is supported by the host. Clients SHOULD check
		//		whether a command is supported before attempting to use it, behaviour
		//		for unsupported commands is undefined.
		// command: The command to test for
		//This will be the default list of supported commands 
		var ie = 1;
		var mozilla = 1 << 1;
		var safari = 1 << 2;
		var opera = 1 << 3;
		var safari420 = 1 << 4;
		var gt420 = dojo.isSafari;
		function isSupportedBy(browsers){
			return {
				ie: Boolean(browsers & ie),
				mozilla: Boolean(browsers & mozilla),
				safari: Boolean(browsers & safari),
				safari420: Boolean(browsers & safari420),
				opera: Boolean(browsers & opera)
			}
		}
		var supportedBy = null;
		switch(command.toLowerCase()){
			case "bold": case "italic": case "underline":
			case "subscript": case "superscript":
			case "fontname": case "fontsize":
			case "forecolor": case "hilitecolor":
			case "justifycenter": case "justifyfull": case "justifyleft":
			case "justifyright": case "delete": case "selectall":
				supportedBy = isSupportedBy(mozilla | ie | safari | opera);
				break;
			case "createlink": case "unlink": case "removeformat":
			case "inserthorizontalrule": case "insertimage":
			case "insertorderedlist": case "insertunorderedlist":
			case "indent": case "outdent": case "formatblock":
			case "inserthtml": case "undo": case "redo": case "strikethrough":
				supportedBy = isSupportedBy(mozilla | ie | opera | safari420);
				break;
			case "blockdirltr": case "blockdirrtl":
			case "dirltr": case "dirrtl":
			case "inlinedirltr": case "inlinedirrtl":
				supportedBy = isSupportedBy(ie);
				break;
			case "cut": case "copy": case "paste":
				supportedBy = isSupportedBy( ie | mozilla | safari420);
				break;
			case "inserttable":
				supportedBy = isSupportedBy(mozilla | ie);
				break;
			case "insertcell": case "insertcol": case "insertrow":
			case "deletecells": case "deletecols": case "deleterows":
			case "mergecells": case "splitcell":
				supportedBy = isSupportedBy(ie | mozilla);
				break;
			case "attachments": case "comments": case "pastecompare" :
				supportedBy = isSupportedBy(mozilla | ie);
				break;
			default: return false;
		}

		return (dojo.isIE && supportedBy.ie) ||
			(dojo.isMoz && supportedBy.mozilla) ||
			(dojo.isSafari && supportedBy.safari) ||
			(gt420 && supportedBy.safari420) ||
			(dojo.isOpera && supportedBy.opera);  // Boolean return true if the command is supported, false otherwise
	},
	//@Overridden shortcuts of dijit.RichText
	setupDefaultShortcuts: function(){
		var ctrl = this.KEY_CTRL;
		var exec = function(cmd, arg){
			if(cmd == "backspaceKeyHandler")
				return function(){ return this.execCommand(cmd,arguments); } ;
			return arguments.length == 1 ? function(){ return this.execCommand(cmd); } :
				function(){ return this.execCommand(cmd, arg); }
		}
		this.addKeyHandler("b", ctrl, exec("bold"));
		this.addKeyHandler("i", ctrl, exec("italic"));
		this.addKeyHandler("u", ctrl, exec("underline"));
		this.addKeyHandler("a", ctrl, exec("selectall"));
		this.addKeyHandler("1", ctrl, exec("formatblock", "h1"));
		this.addKeyHandler("2", ctrl, exec("formatblock", "h2"));
		this.addKeyHandler("3", ctrl, exec("formatblock", "h3"));
		this.addKeyHandler("4", ctrl, exec("formatblock", "h4"));
		this.addKeyHandler("13", null, exec("enterKeyHandler"));
		this.addKeyHandler("13", ctrl, exec("ctrlEnterKeyHandler"));
		this.addKeyHandler("8", null, exec("backspaceKeyHandler"));
		this.addKeyHandler("\\", ctrl, exec("insertunorderedlist"));
		if(!dojo.isIE){
			this.addKeyHandler("Z", ctrl, exec("redo"));
		}
	},
	//@Overridden events of dijit.RichText
	execCommand: function(/*String*/command, argument){
		var returnValue;
		this.focus();

		command = this._normalizeCommand(command);
		if(argument != undefined){
			if(command == "heading"){
				throw new Error("unimplemented");
			}else if((command == "formatblock") && dojo.isIE){
				argument = '<'+argument+'>';
			}
		}
		if(command == "backspacekeyhandler"){
			returnValue = false;
			if(argument[0].keyCode != dojo.keys.BACKSPACE)
				return true;
			else{
				var startingPoint = false;
				try{
					var selection = dijit.range.getSelection(this.window);
					var parent = dojo.withGlobal(this.window, "getParentElement", dijit._editor.selection);
					var header = dijit.range.getAncestor(parent,/^(?:LI)$/);
					if (selection.getRangeAt(0).endOffset == 0 && !header) {
						startingPoint = true;
						returnValue = true;
					}
				}catch(e){
					console.debug("StatementEditor backspace key error: - > "+e);
				}
				if(startingPoint){
					this.endEditing(true);
					setTimeout(dojo.hitch(this, function(){
						this.parentWidget.handleRowDelete(this.id, dojo.byId(this.parentNodeId));
					}),0);
				}else //let browser handle it
					return true;
			}
		}else if(command == "enterkeyhandler"){
			// See if you are currently within an ordered/unordered list
			try{
				var parent = dojo.withGlobal(this.window, "getParentElement", dijit._editor.selection);
				var header = dijit.range.getAncestor(parent,/^(?:LI)$/);
			}catch(e){
				console.debug("StatementEditor enter key error: - > "+e);
			}	
			if(header)//let browser handle it
				return true;
			else{
				// Creates or moves to the next step
				this.endEditing(true);
				setTimeout(dojo.hitch(this, function(){
					this.parentWidget._handleAddClick(null, dojo.byId(this.parentNodeId));
				}),0);
				returnValue=true;
			}	
		}else if(command == "ctrlenterkeyhandler"){
			this.endEditing(true);
			setTimeout(dojo.hitch(this, function(){
				try{
					this.parentWidget._handleRowInsert(dojo.byId(this.parentNodeId));
				}catch(e){
					console.debug("StatementEditor ctrl+enter key error: - > "+e);
				}
			}),0);
			returnValue=true;	
		}
		if(!returnValue)	
			this.inherited('execCommand', arguments);
	},
	
	//@Overridden events of dijit.RichText
	open: function(/*DomNode?*/element){
		//BUG in RTE,doesnt disconnect events on close but re-registers on open
		dojo.forEach(this._connects, function(array){
			dojo.forEach(array, dojo.disconnect);
		});
		this._connects=[];
		this.inherited('open', arguments);
		//Bug in RTE the editor does not expand properly _getIframeDocTxt() has height set to 100%
		dojo.style(this.document.body,"height","auto");
		//FIX defect 7603, iframe is defaultly non-transparent in IE
		if(dojo.isIE){
			this.iframe.allowTransparency = true;
			if(this.iframe)
				this.iframe.style.height = this.minHeight; 
		}
	},
	//On IE the focus does not reach the editor. 
	//Bug in RTE current version (IE takes time to switch off unselectable).
	//Forcing focus on the editor by calling execCommand.
	_IEFocus: function(){
		dijit._editor.RichText.prototype.execCommand.call(this, 'inserthtml', '');
		this.focus();
	}, 

	_error: function(errorObj){
		console.error("Error: (message: '" + errorObj.message + "') from Statement Editor#_error");
	},
	//Get the toolbar state for the editor
	getState: function(){
		var plugins = this._plugins;
		var valueMap = {};
		for(var i=0;i<plugins.length;i++){
			var plugin = plugins[i];
			var command = plugin.command;
			var value = "";
			switch(command){
				case "bold": case "italic": case "underline": case "strikethrough":
					//Toggle button only
					value = plugin.button.checked;
					break;
				case "fontSize": case "fontName":
					//Filtering Select only	
					value = plugin.button.valueNode.value;
					break;
				case "foreColor": case "hiliteColor":
					//DropDown only	
					value = plugin.dropDown.value;
					break;		
				default:
					//Rest any property will not be passed to the next editor inline
					command =""; 
					break;
			}
			if(command != "" && value && value != ""){
				valueMap[command] = value;
			}	
		}
		return valueMap;
	},
	
	//Set the toolbar state for the editor
	setState: function(valueMap){
		var plugins = this._plugins;
		for(var i=0;i<plugins.length;i++){
			var plugin = plugins[i];
			var command = plugin.command;
			var value = valueMap[command];
			if(value != null && value != undefined){
				switch(command){
					case "bold": case "italic": case "underline": case "strikethrough":
						//Directly on the document only
						if(value)
							this.execCommand(command);
						break
					case "fontSize": case "fontName":
						//Filtering Select Buttons only	
						plugin.button.setValue(value);
						break
					case "foreColor": case "hiliteColor":
						//DropDown Buttons only	
						plugin.dropDown.value = value;
						plugin.dropDown.onChange(value);
						break;	
					default:
						//Should not reach here, map to above options
						break;
				}
			}		
		}			
	}
});
})();
