/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2006. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.properties._workItem.QMQueryResults");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
(function() { // module container

var Constants 				= com.ibm.rqm.planning.web.client.internal.Constants;
var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;

var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;
var ImageCache              = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

var ROW_WORK_ITEM_ID_PROP	= "_jazzRowWorkItemId";
var ROW_QUERY_PROP			= "_jazzRowQuery";
var ROW_LINKS_PROP			= "_jazzRowLinks";

// constant for stripping hyperlinks
var STRIP_HYPERLINK_REGEX	= /(<a.*?>)(.*?)(<\/a>)/gi;
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
dojo.declare("QMQueryResults.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    // Summary:
    //
    getNumColumns: function(content)
    {
        return 7;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        var headings = [
            Constants.getMessage("labelId"),
            Constants.getMessage("labelSummary"),
            Constants.getMessage("labelOwnedBy"),
            Constants.getMessage("labelstatus"),
            Constants.getMessage("labelPriority"),
            Constants.getMessage("labelSeverity"),
            Constants.getMessage("labelModifiedDate")
        ];
        return headings[index];
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        return null;
    },
     
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },   
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return (atColumnIndex == 3);     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        switch(atColumnIndex) {
            case 0:
                return true;
        }
        return false;   
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        switch(atColumnIndex) {
            case 0: 
                return 10;
            case 1: 
                return 40;
            case 2: 
                return 10;
            case 3: 
                return 10;
            case 4: 
                return 10;
            case 5: 
                return 10;                  
            case 6: 
                return 10;                                   
        }
        return 0;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_execution_result_" + rowData.id;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = TableConstants.RENDER_TYPE_TEXT;

        switch(atColumnIndex) {
            case 5: 
                type = TableConstants.RENDER_TYPE_IMAGE;
                break;
            case 6: 
                type = TableConstants.RENDER_TYPE_DATE;
                break;               
                               
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "";
        switch(atColumnIndex) {
            case 0: 
                val = rowData.labels[1];
                break;
            case 1: 
                val = rowData.labels[2];
                break;
            case 2: 
                val = rowData.labels[3];
                break;
            case 3: 
                val = rowData.labels[4];
                break;          
            case 4: 
                val = rowData.labels[5];
                break;  
            case 5: 
                val = rowData.labels[6];
                break;  
            case 6: 
                val = Utility.formatDateFromLong(rowData.labels[7]);
                break; 
        }
        return val;
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
		switch(atColumnIndex) {
            case 5: 
                val = {
					tooltip: rowData.labels[6],
					image: ImageCache["SEVERITY_" + rowData.labels[6].toUpperCase()]
				};
                break;
        }
        return val;
    }
    
    
});
/**
 * Provides a UI to iterate through the work items in a query result.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.construction.properties._workItem.QMQueryResults",
	[dijit._Widget, dijit._Templated],
	{
        templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view">' +
                          '      <div dojoAttachPoint="_queryName" class="query-name"></div>' +
                          '</div>',

        widgetsInTemplate: true,
		
		// object initializer
		constructor: function() {
			this._rowCache= [];
			this.callback = null;
		},
        postCreate: function() 
        {
            this.inherited("postCreate", arguments);
        },
		/*
		 *  Return the results of the last query
		 */ 
		getResults: function() {
			return this._rowCache;
		}, 
		/*
		 * Methods for table population and clearing.
		 */
		
		setQueryResultsTable: function(query, queryResultsSummaryPage, orderByAttribute, direction) {
			checkNotNull(query, "query");
			checkNotNull(queryResultsSummaryPage, "queryResultsSummaryPage");
            this._queryName.innerHTML = query.name;
            if(!this.tableViewer)
			{
				this.tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();        
            	this._tableViewerNode.appendChild(this.tableViewer.domNode);
	            this.tableViewer.setContentProvider(new QMQueryResults.TableProvider());
	            this.tableViewer.setTableListener(new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener());
				this.tableViewer.setContentPageSize(20);
				this.tableViewer.setTableRowsCheckable(true, /* isSingleSelect */false);
			}
            var defectRows = [];                      
            var i=0;
            //show only results which are defects.
            if(queryResultsSummaryPage.rows){
				var row;
                while((row=queryResultsSummaryPage.rows[i++]))
                {
                    if(row.labels[0] == "Defect")
                    {
                        defectRows.push(row);
                    }
                }
            }
            this.tableViewer.setContent(defectRows);
            this.tableViewer.show();
			if(this.callback)
				this.callback();
		},

		clearSelection: function() {
			if(this.tableViewer){
				this.tableViewer.rowCheckedHandler.clearRowStates();
	            this.tableViewer.rowRenderer.clearRowStates();
			}
			if(this.callback)
				this.callback();
        },
		
		setCallback: function(callback){
			this.callback = callback;
		}
	}
);

})();