/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.planning.PlanTeam");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanTeam");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DateTextBox");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ImageClient = com.ibm.team.workitem.web.client.internal.ImageClient;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
	    
    var UNASSIGNED_TEAM = Constants.getMessage("labelUnassigned");
    var UNASSIGNED_TEAM_MESSAGE = Constants.getMessage("labelNoTeamAssigned");
    var EMPTY_TEAM_MESSAGE = Constants.getMessage("labelNoMembers");
	
	var EDIT_PROJECT_AREA_ACTION_URI = "#action=com.ibm.team.process.editProjectArea&itemId=";
	var EDIT_TEAM_AREA_ACTION_URI = "#action=com.ibm.team.process.editTeamArea&itemId=";
	var wrapText  = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText;
    
    /**
     * Widget to display list of link elements in the LeftNavigation.
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanTeam", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/planning/PlanTeam.html"),
        
		labelLoading: Constants.getMessage("labelLoading"),
		labelManageTeams: Constants.getMessage("labelManageTeams"),
		
        constructor: function(){
            this._teamAreas = null; /* ProcessAreaDTO[] */
            this._teamArea = null; /* TeamAreaDTO */
        },
        
        
        postCreate: function(args){
            this.inherited('postCreate', arguments);
            //set up workitem support for this section
            this.addWorkItemSupport();
            this._plan = this.getArtifact();
            this._teamArea = this._plan && this._plan.teamArea ? this._plan.teamArea : null;
			this._testTeamLabel.innerHTML = wrapText(
                        Constants.getMessage("labeltestteam"), 
                        "", 
                        85, // hard code the max width since the dom node has not been added to the tree. 
                        true,
                        "<BR/>");
			this._teamMemberslabel.innerHTML = wrapText(
                        Constants.getMessage("labelteammembers"), 
                        "", 
                        85, // hard code the max width since the dom node has not been added to the tree. 
                        true,
                        "<BR/>");
        },
        
		//@Overriden from AbstractArtifactEditorSection - invoked on first expand
        initialize: function(isEditable){
            this._initTeamAreas();   
			this.initialized = true;
        },
				
        //@Overriden from AbstractArtifactEditorSection
        getActions: function(){
            return null;
        },
        
        //@Overriden from AbstractArtifactEditorSection
        getAttributes: function(){
			if (this.initialized && this._teamAreaSelect.value != null) {
	            return {
	                teamAreaItemId: this.initialized ? this._teamAreaSelect.value : null
	            }				
			}
			return null;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.initialize();
            this.addWorkItemSupport();
        },		
        
        _onTeamAreaAdminClick: function(){
			var teamOption = this._teamAreaSelect.options[this._teamAreaSelect.selectedIndex];
			//if team area is selected, show 'edit team area' page for that team area
			if (teamOption.text != UNASSIGNED_TEAM) {
				window.location.href = EDIT_TEAM_AREA_ACTION_URI + teamOption.value;
			} else {
				//otherwise, show edit project area for the current project area
				window.location.href = EDIT_PROJECT_AREA_ACTION_URI + EditorUtil.getCurrentProjectArea().itemId;
			}            
        },
        
        _onTeamAreaChange: function(){
            this._showTeamMembers(this._teamAreaSelect.value);
			this.setDirty();
        },
        
        _initTeamAreas: function(){
            //get the list of process areas (top level is project area; team areas are children)
            var self = this;
            var projectArea = EditorUtil.getCurrentProjectArea();
            var serviceObj = {
                success: function(processAreas){
                    //find current project area						
                    for (var i = 0; i < processAreas.length; i++) {
                        if (processAreas[i].name == projectArea.name) {
                            self._teamAreas = processAreas[i].children;
                            break;
                        }
                    }
                    self._showTeamAreas();
                },
                failure: function(e){
                    self._error(e);
                }
            };
            var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
            ProcessClient.getProcessAreaHierarchies(responseHandler);
        },
        
        _showTeamAreas: function(){
            //function for adding team areas to list (to be used recursively for nested teams)
            var addToSelect = function(select, teams, currentTeam, level){
                for (var i = 0; i < teams.length; i++) {
                    var option = new Option(teams[i].name, teams[i].itemId);
                    if (currentTeam != null && currentTeam.name == teams[i].name) {
                        option.selected = true;
                    }
					SelectUtility.add( select, option );
                    if (teams[i].children) {
                        addToSelect(select, teams[i].children, currentTeam, level + 1);
                    }
                }
            }
            
            //add the team areas to the select
            if (this._teamAreas != null && this._teamAreas.length > 0) {
                dojox.data.dom.removeChildren(this._teamAreaSelect);
				SelectUtility.add( this._teamAreaSelect, new Option(UNASSIGNED_TEAM, "") );
                addToSelect(this._teamAreaSelect, this._teamAreas, this._teamArea, 0);
            }
            
            //show the admin link
            this._adminButton.disabled = !EditorUtil.isAdmin();
			if (!this.isEditable() || this.isPrintView()) {
				this._teamAreaSelect.disabled = true;
				this._adminButton.style.display = "none";
			}
            
            //update the team members
            this._showTeamMembers(this._teamAreaSelect.value);
        },
        
        _showTeamMembers: function(teamAreaItemId){
            //show loading
            dojox.data.dom.removeChildren(this._teamMemberList);
            if (teamAreaItemId == null || dojo.trim(teamAreaItemId).length == 0 || teamAreaItemId == UNASSIGNED_TEAM) {
                this._showEmptyTeam();
                return;
            }
            this._teamMemberList.appendChild(document.createTextNode("Loading..."));
            
            //get the list of team members
            var self = this;
            var serviceObj = {
                success: function(/* TeamAreaDTO */teamArea){
                    //show the team members  
                    self._teamArea = teamArea;
                    dojox.data.dom.removeChildren(self._teamMemberList);
                    if (teamArea && teamArea.members && teamArea.members.length > 0) {
                        for (var i = 0; i < teamArea.members.length; i++) {
                            var member = teamArea.members[i]; /* ContributorDTO */
                            var li = document.createElement("li");
                            //el.style = "";
                            li.innerHTML = member.name + " (" + member.emailAddress + ")";
                            self._teamMemberList.appendChild(li);
                        }
                    }
                    else {
                        self._showEmptyTeam();
                    }
                },
                failure: function(e){
                    self._error(e);
                }
            };
            var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
            ProcessClient.getTeamAreaByUUID({
                uuid: teamAreaItemId
            }, responseHandler);
            
            //show the admin link
            this._adminButton.disabled = !EditorUtil.isAdmin();
        },
        
        _showEmptyTeam: function(){
			this._teamArea = "";
            dojox.data.dom.removeChildren(this._teamMemberList);
            var li = document.createElement("li");
            li.innerHTML = EMPTY_TEAM_MESSAGE;
            this._teamMemberList.appendChild(li);
        },
                
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ PlanTeam#_error");
        }
    });
})();
