/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.planning.environment.MultiSelectCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {
var DELIMITER = ", ";
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.planning.environment.MultiSelectCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //  
	  
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
		var selectLabel = document.createElement("label");

        var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        
        if (editable) {
            node = document.createElement("select");
			node.multiple = "multiple";
			node.size = "3";
            node.className = "combo-cell";
            node.cellModifier = cellModifier;
            node.rowObject = rowObject;
            node.columnIndex = columnIndex;
            node.lastSelected = [];
		    
            if(typeof cellWidthHint != "undefined") {
                node.style.width = cellWidthHint + "px";
            }	        	
            node.onchange = function() {
                var val = null;
				var oldValues = this.lastSelected;
                var newValues = [];
            
                for(var i = 0; i < this.options.length; i++) {
                    if(this.options[i].selected) {
                        val = this.options[i].value;
						newValues.push(val);
                    }
                }
                if(this.cellModifier)
                    this.cellModifier.cellValueChanged(oldValues, newValues, this.rowObject, this.columnIndex);
				 this.lastSelected = newValues;
            }
        } else {
            var node = document.createElement("span");
        }
        
        var tooltip = null;
        if (extendedDisplayInfo && extendedDisplayInfo['choices']) {
            var selected = null;//(typeof extendedDisplayInfo['select'] != 'undefined') ? extendedDisplayInfo['select'] : cellData;
            tooltip = extendedDisplayInfo['tooltip'];
            
            if (editable) {
                // select combo box node when editable
                this.setEnumChoices(node, extendedDisplayInfo['choices']);
            } else {
            // span text node when not editable
                var choices = extendedDisplayInfo['choices'];
                // something is selected so get the corresponding text in the choices
				var text = "";
				var selVals = []
                for (var i = 0; i < choices.length; i++) {
                    if (choices[i].selected) {
					  	selVals.push(choices[i].text);
					}
                }
				for( var i=0;i<selVals.length;i++){
					if(i < (selVals.length - 1))
						text += selVals[i] + DELIMITER;
					else
						text += selVals[i]; 
				}
                if(viewer.isCompactMode && typeof cellWidthHint != "undefined") {
                    text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, node, cellWidthHint);
                }
                node.appendChild(document.createTextNode(text));
            }
        }
        
        // the parent class's attach tooltip will not create a tooltip
        // if the string is null, or undefined. Don't have to check it here.
        this._attachTooltipToCell(node, tooltip);    
        
		selectLabel.appendChild(node);
        return selectLabel;
    },
    
    setEnumChoices: function(select, choices)
    {
        var hasSelect = false;
        
        if(choices && choices.length > 0) {
            for (var i = 0; i < choices.length; i++) {
                select.options[i] = new Option(choices[i].text, choices[i].value);
                
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(choices[i].selected) {
                    select.options[i].selected = true;
//                    hasSelect = true;
                }    
            }
            // select the first item by default.
//            if(!hasSelect)
//                select.options[0].selected = true;
        }
    }    
});

})();
