/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView");
 
dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.mapper.TestCaseMapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.mapper.CategoryMapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.CategoryUtil");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.ExecutionPopUp");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView");


(function() {

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;
var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var TestCaseMapper = com.ibm.rqm.planning.web.client.internal.mapper.TestCaseMapper;
var CategoryMapper = com.ibm.rqm.planning.web.client.internal.mapper.CategoryMapper;
var CategoryClient = com.ibm.rqm.planning.web.client.internal.CategoryClient;
var CategoryUtil   = com.ibm.rqm.planning.web.ui.internal.util.CategoryUtil;
var GlobalEvents   = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
var ExecutionPopUp = com.ibm.rqm.planning.web.ui.internal.view.construction.ExecutionPopUp;
var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var AbstractFilteredTableView = com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView;
var PreferenceClient = com.ibm.rqm.planning.web.client.internal.PreferenceClient;

dojo.declare("TestCaseTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    constructor: function(args) {
		dojo.mixin(this, args);
        //this.parentView = parentView;
    },
    
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        var selection = tableViewer.getCheckedRows();
        var selectionCount = selection.length;
        var archivedCount = 0;
        var nonArchivedCount = 0;
        
        // determine how many selected test cases are archived
        for (var i = 0; i < selectionCount; i++) {
            if (selection[i].archived) {
                archivedCount++;
            } else {
                nonArchivedCount++;
            }
        }
        
        if (this.parentView.deleteActionNode)
            // can delete archived or non-archived test cases
            this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
		if (this.parentView.changeSuspectActionNode)
            // can not invoke if there is any archived test case selected
            this.parentView.changeSuspectActionNode.setDisabled(archivedCount > 0 || nonArchivedCount == 0 || !tableViewer.isEditable());
		if (this.parentView.executeActionNode)
            // can not invoke if there is any archived test case selected
            this.parentView.executeActionNode.setDisabled(archivedCount > 0 || nonArchivedCount != 1 || !tableViewer.isEditable());	
        if (this.parentView.genEWIActionNode)
            // can not generate ewi if there is any archived test case selected
            this.parentView.genEWIActionNode.setDisabled(archivedCount > 0 || nonArchivedCount == 0 || !tableViewer.isEditable());	
			
		//if table view has a rowCheckListener, invoke it
		var p = this.parentView;
        if (p && p.rowCheckedListener && dojo.isFunction(p.rowCheckedListener)) {
			p.rowCheckedListener(rowObject, state);
		}
    },
    
    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        this.rowCheckStateChanged(tableViewer, rowObjects, state);
    },
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    },

    tableContentFilterActionInvoked: function(tableViewer, actionId, filters, callback, callbackData)
    {
        if(callback) {
            callback(actionId, filters);
        }                     
    },
	
	// Summary:
    //    If the client have specified a header image and specified that it's an action (see TableContentProvider 
    //    for details), when the column's image is clicked, this message is invoked.
    //
    tableHeaderColumnActionInvoked: function(tableViewer, columnIndex, /*optional*/actionId)
    {
		this.parentView.tableHeaderColumnActionInvoked(columnIndex, actionId);  
    }  
});

dojo.declare("TestCaseTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
	// Summary:
	//    Method to be called when a cell renderer changes the cell value. This notifies
	//    the cell's value change event. 
	//    
	//    
	cellValueChanged: function(oldValue, newValue, rowObject, columnIndex){
		// store the new values in our internal data model        
		rowObject.revision = newValue ? newValue : "";
	},
	
	cellActionInvoked: function(rowObject, columnIndex){
	}
});

dojo.declare("TestCaseTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    constructor: function(args) {
        dojo.mixin(this, args);
		this.categoryTypeMap = {};
		this.columnProfile = null;
		if (!this.categoryTypes || !dojo.isArray(this.categoryTypes)) {
			throw new Error("Parameter 'categoryTypes' is REQUIRED!");
		}
		
        this.COLUMNS = {
            TYPE: this.toColumn("", 2, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_IMAGE, 'type'),
			ID: this.toColumn(Constants.getMessage("labelId"), 3, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_LINK, 'id'),
			SUSPECT: this.toColumn(Constants.getMessage("labelSuspect"), 2, true, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_IMAGE, null),
            NAME: this.toColumn(Constants.getMessage("labelName"), 30, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_LINK, 'name'),
            STATE: this.toColumn(Constants.getMessage("labelState"), 5, true, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT, 'state'),
            WEIGHT: this.toColumn(Constants.getMessage("labelWeight"), 5, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT, 'weight'),
            MODIFIED: this.toColumn(Constants.getMessage("labelModified"), 20, true, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_DATE, 'modified'),
			
			SNAPSHOTS: this.toColumn(Constants.getMessage("labelSnapshots"), 15, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_ENUM, 'snapshot')
        }      
		var numCategories = this.categoryTypes.length;
		this.COLUMNS.CATEGORIES = [];
		for (var i = 0; i < numCategories; i++) {
			var t = this.categoryTypes[i];
			if (!t.archived) {
				this.categoryTypeMap[t.itemId] = t;
				var width = Math.floor(35 / numCategories);
				var name = Constants.getMessage(t.name); // out of the box category name has translated value.
				this.COLUMNS[t.itemId] = this.toColumn(name, width, true, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT);
				this.COLUMNS[t.itemId].typeItemId = t.itemId;
				this.COLUMNS.CATEGORIES.push(this.COLUMNS[t.itemId]);
			}
		}
        this.PROFILES = {};
        this.PROFILES.ALL = [this.COLUMNS.TYPE, this.COLUMNS.ID, this.COLUMNS.SUSPECT, this.COLUMNS.NAME, this.COLUMNS.STATE];		
		this.PROFILES.ALL = this.PROFILES.ALL.concat(this.COLUMNS.CATEGORIES);
        this.PROFILES.ALL = this.PROFILES.ALL.concat([this.COLUMNS.WEIGHT, this.COLUMNS.MODIFIED]);  
		
		this.PROFILES.MINIMAL = [this.COLUMNS.TYPE, this.COLUMNS.ID, this.COLUMNS.NAME, this.COLUMNS.STATE];
		this.PROFILES.MINIMAL = this.PROFILES.MINIMAL.concat(this.COLUMNS.CATEGORIES);
		
        this.PROFILES.SNAPSHOT = [this.COLUMNS.TYPE, this.COLUMNS.ID, this.COLUMNS.NAME, this.COLUMNS.SNAPSHOTS, this.COLUMNS.STATE];
        this.PROFILES.SNAPSHOT = this.PROFILES.SNAPSHOT.concat(this.COLUMNS.CATEGORIES);		
		
        this.columnProfile = null;
        this.columnProfileName = null;
        this.setProfile(this.profile ? this.profile : "ALL"); //default to 'ALL' (full) column profile        
    },
    
    toColumn: function(name, width, groupable, alignment, type, rowName) {
        return {
            name: name,
            width: width,
            groupable: groupable,
            alignment: alignment,
            type: type,
            rowName: rowName
        }
    },  
	
	getProfile: function() {
		return this.columnProfile;
	},

    getProfileName: function() {
        return this.columnProfileName;
    },
			
	setProfile: function(/* string */ profile) {
		if (this.PROFILES[profile]) {
			this.columnProfileName = profile;
			this.columnProfile = this.PROFILES[profile];
		}
	},

    getColumnIndex: function(name) {
        for (var i = 0; i < this.columnProfile.length; i++) {
            if (this.columnProfile[i].name == name) {
                return i;
            }
        }
        return -1;
    },
	
	setCategoryTypes: function(newTypes) {
        this.categoryTypes = newTypes;
    },
		
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.columnProfile.length;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this.columnProfile[index].name;
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
		var column = this.columnProfile[atColumnIndex];
		if (!this.enableRowEditable || (column.rowName && column.rowName != "weight") || (column.name == this.COLUMNS.SUSPECT.name)) 
			return null;
		else {
			var img = {
				actionId: column.name ? column.name : Constants.getMessage("labelWeight"),
				src: ImageCache.BULK_CHANGE,
				disabled: true
			};
			img.tooltip = Constants.getMessage("labelChangeProperties", img.actionId);
			return img;
		}
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },   
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].groupable;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].name != this.COLUMNS.TYPE.name;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {			
		var column = this.columnProfile[atColumnIndex];
		if (this.columnProfileName == "ALL") {
			return column.width;
		}
		//TODO - refactor into base class or create profile class
		//if using a smaller column profile, adjust widths
		var total = 0;
		for (var colName in this.columnProfile) {
			total += this.columnProfile[colName].width;
		}
		//split the difference between all columns proportionately
        var rowBuffer = 100 - total;   //percentage of width unused
		var columnBuffer = rowBuffer * (column.width / total);  //proportionately redistrubute
		return (column.width + Math.round(columnBuffer));        
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_testCase_" + (rowData.id ? "id_"+rowData.id : "participant_"+rowData.participantId);
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = this.columnProfile[atColumnIndex].type;
        // link is disabled as text label if the test case is archived
        if (rowData.archived && (type == TableConstants.RENDER_TYPE_LINK)) {
            return TableConstants.RENDER_TYPE_TEXT;
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    	
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "";
		var column = this.columnProfile[atColumnIndex];
        switch(column.name) {
            case this.COLUMNS.TYPE.name: 
                val = "";
                break;
            case this.COLUMNS.ID.name: 
                val = rowData.id;
                break;
            case this.COLUMNS.NAME.name: 
                val = rowData.name;
				if (rowData.snapshotInfo) {
					var snap = rowData.snapshotInfo;
					val += Constants.getMessage("labelRevisionInfo", snap.revision, snap.name);
				}
                break;
            case this.COLUMNS.STATE.name: 
                //val = rowData.state;
				val = rowData.archived ? Constants.getMessage("labelDeleted") : EditorUtil.getStateNameFromID(rowData.state);             
                break;
            case this.COLUMNS.WEIGHT.name: 
                val = rowData.weight;
                break;
            case this.COLUMNS.MODIFIED.name: 
                val = new Date();
                val.setTime(rowData.modified);
                break;   
			case this.COLUMNS.SUSPECT.name: 
                val = (rowData.suspect)? Constants.getMessage("labelSuspect") : Constants.getMessage("labelNonSuspect");
                break; 	
            case this.COLUMNS.SNAPSHOTS.name: 
                val = "";
                break;  				
			default:
			    val = this.getCategoryByType(rowData, column.typeItemId) || "";
				break;            
        }
        return val;
    },	

    getCellRawData: function(rowData, atColumnIndex)
    {
        var val = "";
        var rowName = this.columnProfile[atColumnIndex].rowName;
        
        if(rowName) {
            return (rowData[rowName] || val);
        }
        
        var column = this.columnProfile[atColumnIndex];
		if(column.name == this.COLUMNS.SUSPECT.name) {
			val = (rowData.suspect)?"Suspect":"Non-Suspect";
		}
		else {
			val = this.getCategoryByType(rowData, column.typeItemId) || "";
		}
        return val;
    },
	
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = {};
		var colName = this.columnProfile[atColumnIndex].name;
        switch(colName) {
            case this.COLUMNS.TYPE.name: 
                val.image = ImageCache.TEST_CASE;
                val.tooltip = rowData.archived ? Constants.getMessage("labelTCDeleted") : Constants.getMessage("labelClickToOpenTestCase");
                if (!rowData.archived) {
                    if (rowData.newItem && this.newItemLinkCallback) {
                        val.onclick = dojo.hitch(this, function(){
                            this.newItemLinkCallback(rowData.participantId);
                        });
                    } else if (rowData.snapshotInfo) {
                        val.href = Utility.getTestCaseURI(rowData.itemId, rowData.snapshotInfo.revision);
                    } else {
                        val.href = Utility.getTestCaseURI(rowData.id);
                    }
                }
                break;
			case this.COLUMNS.ID.name:
                if (!rowData.archived) {
                    val.linkType = Constants.getMessage("TestCaseID");
                }
            case this.COLUMNS.NAME.name: 
                if (!rowData.archived) {
                    if (rowData.newItem && this.newItemLinkCallback) {
                        val.onclick = dojo.hitch(this, function(){
                            this.newItemLinkCallback(rowData.participantId);
                        });
                    } else if (rowData.snapshotInfo) {
                        val.href = Utility.getTestCaseURI(rowData.itemId, rowData.snapshotInfo.revision);
                    } else {
                        val.href = Utility.getTestCaseURI(rowData.id);
                    }
                }
                break;
            case this.COLUMNS.STATE.name: 
            case this.COLUMNS.WEIGHT.name:
			case this.COLUMNS.MODIFIED.name:
                break;       
			case this.COLUMNS.SUSPECT.name: 
                if (rowData.suspect) {
					val.image = ImageCache.TESE_SUSPECT;
				} else if (!rowData.suspect) {
					val.image = ImageCache.TESE_NONE_SUSPECT;
				} else {
					val.image = "";
				}
				break;
			case this.COLUMNS.SNAPSHOTS.name:
                val.select = "";
                val.choices = [];
                val.choices.push({
                    text: Constants.getMessage("labelCurrent"),
                    value: ""
                });				
				if (rowData.snapshots) {
					for (var i = 0; i < rowData.snapshots.length; i++) {
						val.choices.push({
							text: (rowData.snapshots[i].revision + ": " + rowData.snapshots[i].name),
							value: rowData.snapshots[i].revision
						})
					}
				}
				break;
        }
        return val;		
    },
    
    getCategoryByType: function(testCase, typeItemId) {
        if (testCase && testCase.categories) {
			var type = this.categoryTypeMap[typeItemId];
			for (var i = 0; i < (type.categories ? type.categories.length : 0); i++) {
				for (var j = 0; j < testCase.categories.length; j++) {
					if (type.categories[i].itemId == testCase.categories[j]) {
                		return type.categories[i].name;
            		}
				}
			}
        }
		return Constants.getMessage("labelUnassigned");
    }
	
});

dojo.declare("TestCaseTableView.TableFilterProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider,
{
    constructor: function(tableView, openSection, addSnapshotsFilter) {
        this.tableView = tableView;
        this.initOpenFilter = openSection;
        this.addSnapshotsFilter = addSnapshotsFilter;

        // define the static filters
        this.filters = [];
        this.filters.push({		
            name: "myItemsOnly",  // must match key in the arg list of the API
            title: Constants.getMessage("labelOnlyMyTestCases"),
            promptTitle: Constants.getMessage("labelOnlyMyTestCases"),
            type: TableContentFilterProvider.FIELD_CHECKBOX,
            defaultValue: true, params: null
        });
        if (this.addSnapshotsFilter == true) {
            this.filters.push({
                name: "includeSnapshots",  // must match key in the arg list of the API
                title: Constants.getMessage("labelIncludeSnapshots"),
                promptTitle: Constants.getMessage("labelIncludeSnapshots"),
                type: TableContentFilterProvider.FIELD_CHECKBOX,
                defaultValue: false, params: null               
            });
        }		
		this.filters.push({
            name: "name",  // must match key in the arg list of the API
            title: Constants.getMessage("labelname"),
            promptTitle: Constants.getMessage("labelName"),
            type: TableContentFilterProvider.FIELD_TEXT,
            defaultValue: null, params: null
        });

    },

    getTitle: function() {
        return Constants.getMessage("labelFilterTitle");    
    },
    
    getSummaryText: function() {
        return Constants.getMessage("labelFilterDescription");
    },
    
    getIsSectionOpenInitially: function() {
       return this.initOpenFilter; 
    },
    
    getInitialVisibleFilterCount: function() {
        return 4;    
    },
    
    getFilterCount: function() {
        return this.filters.length;
    },
    
    getFilterName: function(index) {
        return this.filters[index].name;
    },
    
    getFilterTitle: function(index) {
        return this.filters[index].title;
    },
    
    getFilterPromptTitle: function(index) {
        return this.filters[index].promptTitle;        
    },
    
    getFilterType: function(index) {
        return this.filters[index].type;
    },
    
    getFilterDefaultValue: function(index) {
        return this.filters[index].defaultValue;
    },
    
    getFilterParams: function(index) {        
        return this.filters[index].params;
    },
    
    getCustomField: function(index) {
        return null;
    },
    
    getActionsInfo: function() {
        var actions = [
            {
                id: "cancel",
                title: Constants.getMessage("labelReset"),
                callback: dojo.hitch(this, "filterResetActionInvoked"),
                callbackData: null,
                isDefault: false
            }, {
                id: "run",
                title: Constants.getMessage("labelRun"),
                callback: dojo.hitch(this, "filterRunActionInvoked"),
                callbackData: null,
                isDefault: true
            }
        ];
        return actions;
    },
    
    filterResetActionInvoked: function(actionName, filters) {
        this.tableView.tableViewer.setContentFilterAttributeProvider(this.tableView.filterProvider);
    },

    filterRunActionInvoked: function(actionName, filters){
        if (this.tableView) {
            // must have the view context to perform the query and update the table display
            var allCategories = [];
            var args = {};
            
            for (var nameKey in filters) {
                // iterate through the filters
                if (filters[nameKey] != null) {
                    // skip those that are not specified
                    if (nameKey.indexOf("categories") == 0) {
                        // filter name has "categories" prefix, flatten all selected categories 
                        // into an array of comma separated strings
                        allCategories.push(filters[nameKey].join(","));
                    } 
                    else {
                        if (nameKey == "weight") {
                            args['weightComparator'] = filters[nameKey].operator;
                            args['weightValue'] = filters[nameKey].value;
                        }
                        else {
                            args[nameKey] = filters[nameKey] || "";
                            if(args[nameKey] == 'Any@@')
                                args[nameKey] = '';
                            }
                    }
                }
            }
            // add the flattened categories array to the args if there is any defined
            if (allCategories.length > 0) {
                args["categories"] = allCategories;
            }
            this.tableView.runQuery(args);
        }
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView", 
    [dijit._Widget, dijit._Templated, AbstractFilteredTableView],
{    
    templateString: '<div class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" "noline"=true dojoAttachPoint="viewHeaderNode">' +
                    '    <div dojoAttachPoint="_queryTableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
					'  <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" ></div>' +
				    '</div>',

    editable: true,
	printable: false,
    widgetsInTemplate: true,

    constructor: function() {
		this.categoryTypes = null;
	 	this.testCases = null;
		this.newItemLinkCallback = null;
		this.rowCheckedListener = null;
        this.showInitLoadProgress = false;
        this.showQueryFilters = false;
		this.allowQuerySnapshots = false;
        this.initOpenFilter = false;
        this.filterArgs = null;
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);

		EditorUtil.loadStateTransitionConstraintSettings();  //Load the prefs in case we attempt actions that require checking (like execute)

        // the view is in a query page if there is a query name otherwise it is within
        // some container that don't need the header
        this.isQueryPage = this.queryName;

        // apply the user preference whether to initially open the filter area
        if (this.isQueryPage && this.showQueryFilters) {
            var context = {
                self: this,
                prefCallback: function(userPref) {
                    this.self.initOpenFilter = (userPref && typeof userPref['value'] != "undefined") ? userPref.value : false;
                    this.self._initCategory();
                }
            }
            PreferenceClient.getCurrentUserPreferencesWithNullPref(context, context.prefCallback, Constants.USER_UI_CUSTOMISE_QUERYLISTS_FILTER_INITOPEN_CATEGORY.key, false, true);         
        } else {
            this._initCategory();
        }
		
		this.setSearchFunction(dojo.hitch(TestCaseClient, "searchTestCases"));	
		this.setSearchCallback(dojo.hitch(this, "show"));
		this.setResponseHandlerType(TestCaseMapper);
		this.setTypeLabel(Constants.getMessage("labelTestCase"));
        if (this.postQueryFilterCallback) {
            this.setSearchFilterCallback(this.postQueryFilterCallback);
        }			
    },

    _initCategory: function() {
        // we're showing query filters, we'll be managing our own
        // contents. Initialize the query filters first.
        var callback = (this.showQueryFilters) ? "_initializeFilters" : "_initialize";
        
        //get the category types (used instead of resolving them for each test case)
        var responseHandler = new CategoryMapper(this, callback, "_error");
        CategoryClient.getCategoryTypeSummaries(responseHandler, Constants.MODEL_TYPE_CASE);
	},
	
    // Constructs the query filter definitions and the content filter provider for the
    // table. The table will construct the filters section based on the definitions.
    // 
    _initializeFilters: function(categoryTypes) {
        // create the filter provider for the table
        this.filterProvider = new TestCaseTableView.TableFilterProvider(this, this.initOpenFilter, this.allowQuerySnapshots);

        this.moreFilters = [];

        this.moreFilters.push({
            name: "stateIds", // must match key in the arg list of the API
            title: Constants.getMessage("labelStateLabel"),
            promptTitle: Constants.getMessage("labelState"),
            type: TableContentFilterProvider.FIELD_TOGGLELIST,
            defaultValue: [],
            params: {options: [{
                    name: Constants.getMessage("com.ibm.rqm.planning.common.new"),
                    value: Constants.STATETRANSITION_STATE_DRAFT,
                    checked: false
                }, {
                    name: Constants.getMessage("com.ibm.rqm.planning.common.underreview"),
                    value: Constants.STATETRANSITION_STATE_UNDERREVIEW,
                    checked: false
                }, {
                    name: Constants.getMessage("com.ibm.rqm.planning.common.approved"),
                    value: Constants.STATETRANSITION_STATE_APPROVED,
                    checked: false
                }, {
                    name: Constants.getMessage("com.ibm.rqm.planning.common.retired"),
                    value: Constants.STATETRANSITION_STATE_RETIRED,
                    checked: false
                }
            ]}
        });
        
        this.moreFilters.push({
            name: "suspect",
            title: Constants.getMessage("labelSuspect") + Constants.getMessage("labelColen"),
            promptTitle: Constants.getMessage("labelSuspect"),
            type: TableContentFilterProvider.FIELD_ENUM, 
            defaultValue: "Any@@", 
            params: [
                {
                    name: Constants.getMessage("labelAnyCap"),
                    value: 'Any@@'
                },
                {
                    name: Constants.getMessage("labelSuspect"),
                    value: "true"
                }, 
                {
                    name: Constants.getMessage("labelNonSuspect"),
                    value: "false"
                }
            ]
        });
        this.moreFilters.push({
            name: "weight",
            title: Constants.getMessage("labelWeight") + Constants.getMessage("labelColen"),
            promptTitle: Constants.getMessage("labelWeight"),
            type: TableContentFilterProvider.FIELD_NUMERIC,
            defaultValue: null,
            params: {'showNumericOperators':true}            
        });

        // initialize filters for all the categoryType
        this.currentCategoryFilters = CategoryUtil.getCategoryTypesFilters(categoryTypes);
        this.moreFilters = this.moreFilters.concat(this.currentCategoryFilters);
            
        //callback: initialize the orginator filter with the list of contributors
        var callback = dojo.hitch(this, function() {
            // add the category filters to the end if there is any
            for (var j = 0; j < this.moreFilters.length; j++) {
                this.filterProvider.filters.push(this.moreFilters[j]);
            }
            this._initialize(categoryTypes);
        });
        
        var contributorFilter = {           
            //filterName, filterLabel, filterPrompt, filterType, callback
            name: "originatorUUIDs",
            label: Constants.getMessage("labeloriginatorLabel"),
            prompt: Constants.getMessage("labelOriginator"),
            type:TableContentFilterProvider.FIELD_TOGGLELIST
        }
        this.addContributorFilter(contributorFilter, this.filterProvider.filters, callback);

    },
    	
    _initialize: function(categoryTypes) {
		this.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
		
		//create the table provider
        this.provider = new TestCaseTableView.TableProvider({
            profile: this.profile ? this.profile : Constants.TABLE_PROFILE_ALL,
            categoryTypes: this.categoryTypes,
            newItemLinkCallback: this.newItemLinkCallback,
			enableRowEditable: this.enableRowEditable
        });		
		
		//create table viewer		
		var isMinimal = this.profile && this.profile != Constants.TABLE_PROFILE_ALL;
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            //turn off all controls if in print mode             
			showPagingControl: !this.printable,
			showPagingRangeControl: !this.printable,
            showGroupFilterControls: !this.printable,
            showPagingActionsControls: (!isMinimal && !this.printable),
            allowAdjustSettings: !isMinimal,
            autoSaveSettings: !isMinimal
        });	               

        if (this.showQueryFilters) {
            tableViewer.setContentFilterAttributeProvider(this.filterProvider);
            
            // we're not showing any content yet, disable the no item message
            tableViewer.noItemMessage = " ";
        }
        
        // setup the table's id for saving table settings
        if(!isMinimal) {
            tableViewer.tableId = (this.isQueryPage) ? "RQM-TC-QUERY-TABLEVIWER" : "RQM-TC-TABLEVIWER";
        }
        
        if (this.isQueryPage) {
            // it is a query page so set header title and add table viewer as the header content
            this.viewHeaderNode.setTitle(this.queryName);
            this.viewHeaderNode.setCSHelpId("rqm_view_test_cases.html");        
			this._tableViewerNode.style.display = "none";
            dojox.data.dom.removeChildren(this._queryTableViewerNode);
			this._queryTableViewerNode.appendChild(tableViewer.domNode);
        } else {
            // it is not a query page i.e. within a section without header
            this.viewHeaderNode.domNode.style.display = "none";
            dojox.data.dom.removeChildren(this._tableViewerNode);
			this._tableViewerNode.appendChild(tableViewer.domNode);
        }
        
        var content = this.testCases ? this.testCases : [];
        tableViewer.setContent(content);
        tableViewer.setTableRowsCheckable(this.editable && !this.printable);        
        tableViewer.setContentProvider(this.provider);
		tableViewer.setTableListener(new TestCaseTableView.TableListener({
			parentView: this
		}));
        tableViewer.setCellModifier(new TestCaseTableView.CellModifier());
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelTestCases")));
        tableViewer.setUnspecifiedGroupTitle(Constants.getMessage("labelUnassigned"));
        if (this.enableRowSelection) {
            tableViewer.setTableRowsCheckable(this.editable && !this.printable, /* isSingleSelect */ false);
        }
		if(this.isSingleSelect){
			tableViewer.setTableRowsCheckable(this.editable && !this.printable, /* isSingleSelect */ true);
		}
        var permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_CASE);

		//add refresh action if applicable
        if (this.refreshCallback && dojo.isFunction(this.refreshCallback)) {
            this.refreshAction = tableViewer.addAction("Refresh", Constants.getMessage("labelRefresh"), this, "refresh-icon-image", this.refreshCallback);
        } 
		
		//add any actions provided by the consumer
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            var action = this.actions[i];
            if(!this.isQueryPage && action.label == Constants.getMessage("labelRemoveTestCases")){//If not querypage remove action will be sent
				this.deleteActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
				this.deleteActionNode.setDisabled(true);
			} else if (action.label == Constants.getMessage("labelGenEWIs")) {
                this.genEWIActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
                this.genEWIActionNode.setDisabled(true);
            }
			else if(action.label == Constants.getMessage("labelChangeTestCaseProperties")) {
				this.batchUpdateNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
                this.batchUpdateNode.setDisabled(true);
			} else
				tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);	
        }		

		//Check Test Case Execute Permission
		var tc_exec_permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_CASE_EXE);
        // default to allow if fail to fetch permission
		var hasExePermission = !tc_exec_permission ? true : tc_exec_permission 
																&& tc_exec_permission[PermissionsClient.ACTION_EXECUTE];
        if (this.isQueryPage) {
            if (hasExePermission) {
                this.executeActionNode = tableViewer.addAction(
                        "execute",                                  // action id
                        Constants.getMessage("labelExeTest"),       // label
                        null,                                       // callback data
                        "execute-icon-image",                       // icon class
                        dojo.hitch(this, "_onExecuteActionClick")); // callback
                this.executeActionNode.setDisabled(true);
            }
        }
        
        var cat_permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_CATEGORY);
        if (!cat_permission || cat_permission[PermissionsClient.ACTION_EDIT]) {
            //add action for managing categories
            this.configureActionNode = tableViewer.addAction("configure", // action id
                Constants.getMessage("labelManageCategories", Constants.TYPE_CASE), // label
                null, // callback data
                "categories-icon-image", // icon class
                dojo.hitch(this, "_handleCategoriesClick") // callback
            );
        }
        
        if (!permission || permission[PermissionsClient.ACTION_EDIT]) {
            //add action for changing suspect attribute	
            this.changeSuspectActionNode = tableViewer.addAction(
                "ChangeSuspect", 
                Constants.getMessage("labelChangeSuspect"), 
                this, 
                "req-suspect-icon-image", 
                dojo.hitch(this, "_onChangeSuspectActionClick")
            );
            this.changeSuspectActionNode.setDisabled(true);
        }
				
        // for query page there will be no passed in actions and we need a delete action
        if (this.isQueryPage) {
            if (!permission || permission[PermissionsClient.ACTION_ARCHIVE]) {
                this.deleteActionNode = tableViewer.addAction(
                    "delete",                                  // action id
                    Constants.getMessage("labelDelete"),       // label
                    null,                                      // callback data
                    "delete-icon-image",                       // icon class
                    dojo.hitch(this, "_onDeleteActionClick")); // callback
                this.deleteActionNode.setDisabled(true);
            }
        }
        if(this.showInitLoadProgress) {
            tableViewer.showLoadingProgress();
        }
		if (this.printable) {
			tableViewer.setContentPageSize(-1);
		}
        tableViewer.show();
        tableViewer.setEditable(this.editable);
        this.tableViewer = tableViewer;	
		
        // apply the user preference whether to run my query immediately.
        if (this.isQueryPage && this.showQueryFilters) {
            PreferenceClient.getCurrentUserPreferences(this, this.initRunQuery, Constants.USER_UI_CUSTOMISE_QUERYLISTS_AUTORUN_MYQUERY_CATEGORY.key);
        }	
    },

    //API
    getAction: function(id) {
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            if (this.actions[i].label == id) {
                return this.actions[i];
            }
        }
    },
    
    getSelectedItems: function() {
        return this.tableViewer.getCheckedRows();
    },
    
    refresh: function(content, skipClearStatus) {       
        this.testCases = content;
        var cancelProgress = this.showInitLoadProgress;
        
        // we don't create the table until we fetched the categories.
        // The table may not have existed at this point. The real data
        // has arrived. Always reset the init load progress flag. 
        if(this.showInitLoadProgress) {
            this.showInitLoadProgress = false;
        }
		if (this.tableViewer) {
            if(cancelProgress) {
                this.tableViewer.endLoadingProgress();
            }
			this.tableViewer.setContent(content);
			this.tableViewer.refresh();
		}
        if (this.viewHeaderNode && !skipClearStatus) {
            this.viewHeaderNode.clearStatus();
        }  		
    },  
	
	// Summary:
    //    Get categoryTypes first, then displays the table.
    //    The content provider is changed, so invoke _reContructTable to reset the whole table
    //
	show: function(content, skipClearStatus) {
		this.testCases = content;
        
        //reset CategoryTypes
		var self = this;
		var handler = {
            success: function(categoryTypes) {
                //reset TableViewer first
                self.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
                var cancelProgress = self.showInitLoadProgress;
                // we don't create the table until we fetched the categories.
                // The table may not have existed at this point. The real data
                // has arrived. Always reset the init load progress flag. 
                if (self.showInitLoadProgress) {
                    self.showInitLoadProgress = false;
                }
                if (self.tableViewer) {
                    if (cancelProgress) {
                        self.tableViewer.endLoadingProgress();
                    }
					self._reContructTable(categoryTypes, content);
                }
                if (self.viewHeaderNode && !skipClearStatus) {
                    self.viewHeaderNode.clearStatus();
                }
                
                //reset viewBuilder?
                if (self.showQueryFilters) {
                    var newCategoryFilters = CategoryUtil.getCategoryTypesFilters(self.categoryTypes);
                    if (!CategoryUtil.isCategoryFiltersSame(newCategoryFilters, self.currentCategoryFilters)) {
                        //category filters are changed, user have the choice to reset viewbuilder or not
                        self._confirmResetViewBuilder(newCategoryFilters);
                    }

                    if (self.queryFiltersCallback) {
                        self.queryFiltersCallback();
                    }
                }
            },
            error: function(e) {
                self._error(e);
            }
        }
		var responseHandler = new CategoryMapper(handler, "success", "error");
        CategoryClient.getCategoryTypeSummaries(responseHandler, Constants.MODEL_TYPE_CASE);		
	},
	
	
	//Internal
	_onExecuteActionClick: function(){
		var selectedItems = this.tableViewer.getCheckedRows();
		var selectedCase = selectedItems[0];
		
		if (EditorUtil.applyTransactionConstraint(Constants.STATETRANSITION_PREFERENCE_ONLYEXECUTEAPPROVED)) 
		{
			var state = selectedCase.state;
			if (state != Constants.STATETRANSITION_STATE_APPROVED) 
			{
				var errorStr = Constants.getMessage("msgCannotExecuteTestCaseNotApproved", selectedCase.name);
                if (this.isQueryPage)
					this.viewHeaderNode.setStatus(Constants.getMessage("cantExecuteTestCase"), ViewHeaderLayout.WARNING, errorStr);
				else
					alert(errorStr);
				return;
			}
		}
	    var execPopUp = new ExecutionPopUp();
		var args = {
			testCase: selectedCase
		}
		execPopUp.initialize(args);
	},
	
    _handleCategoriesClick: function() {
		//refresh tableview by public event
        this._categoriesEditor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor({
			artifactType: Constants.MODEL_TYPE_CASE
		});
    },
	
    _handleCategoriesChanged: function(categoryTypes) {
		//reset View Builder
		this.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
		var newCategoryFilters = CategoryUtil.getCategoryTypesFilters(this.categoryTypes);
		this._resetViewBuilder(newCategoryFilters);
		
		//reset test cases' table
        this._reContructTable(categoryTypes);
    },	
 
    _onDeleteActionClick: function() {
        // check if any selected test cases are current displayed in another tabs
        var openedArtifacts = [];
        this.selectedCases = [];
        if (this.artifactsDisplayContainer) {
            var userSelectedCases = this.tableViewer.getCheckedRows();
            
            if (userSelectedCases.length > 0) {
                for (var i = 0; i < userSelectedCases.length; i++) {
                    var caseURI = Utility.getTestCaseURI(userSelectedCases[i].id);
                    if (this.artifactsDisplayContainer.hasPage(caseURI)) {
                        openedArtifacts.push(userSelectedCases[i]);
                    } else {
                        this.selectedCases.push(userSelectedCases[i]);
                    }
                }
            }
        } else {
            this.selectedCases = this.tableViewer.getCheckedRows();
        }
        
        var prompt = Constants.getMessage("msgDelTestCaseConfirm");
        if (openedArtifacts.length > 0) {
            if (this.selectedCases.length > 0) {
                // some selected artifacts are not opened in tabs
                prompt = Constants.getMessage("msgDelSomeOpenTestCaseConfirm");
            } else {
                // all selected artifacts are opened in tabs
                prompt = Constants.getMessage("msgDelAllOpenTestCaseConfirm");
            }
        }

        var args = {
            'message': prompt,
            'options': openedArtifacts,
            'okCallback': 
                function(view) {
                    view._deleteAction();
                },
            'okCallbackData': this,
            'cancelCallback': null,
            'cancelCallbackData': null
        };
        this.confirmDialog = new com.ibm.asq.common.web.ui.internal.widgets.OptionsDialog(args);
        this.confirmDialog.setTitle(Constants.getMessage("labelConfirmation"));
        this.confirmDialog.open();
    },
	
	_onChangeSuspectActionClick: function() {
		var args = {
            'message': Constants.getMessage("msgChangeSuspectConfirm"),
            'yesCallback': 
                function(view) {
                    view._changeSuspectAction();
                },
            'yesCallbackData': this,
            'noCallback': null,
            'noCallbackData': null
        };
        var confirmDialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        confirmDialog.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
        confirmDialog.open();
	},
	
	_changeSuspectAction: function() {
		var selectedCases = this.tableViewer.getCheckedRows();
        this.selectedCaseCount = selectedCases.length;
        if (this.selectedCaseCount > 0) {
			var itemsArray = new Array();
            for (var i = 0; i < this.selectedCaseCount; i++) {
				//if (selectedCases[i].suspect) {
					itemsArray.push({
						versionableItemId: selectedCases[i].versionableItemId,
						stateId: selectedCases[i].versionableItemId,           
						suspect: !selectedCases[i].suspect
					});
				//}
            }
			var responseHandler = new ServiceResponseHandler(this, "_changeSuspectSuccess", "_error");
            TestCaseClient.changeSuspectTestCase(responseHandler,
                    {
                        itemsJsonString: dojo.toJson(itemsArray)
                    });
            }
	},
	
	_changeSuspectSuccess: function(response) {
		if (response) {
			for (var i = 0; i < this.testCases.length; i++) {
				for (var j = 0; j < response.length; j++) {
					if (this.testCases[i].itemId == response[j].itemId) {
						this.testCases[i].suspect = response[j].suspect;
						break;
					}
				}
			}
		}
		this.refresh(this.testCases);
		this.changeSuspectActionNode.setDisabled(true);
	},
    
    _deleteAction: function() {         
        this.selectedCases = this.selectedCases.concat(this.confirmDialog.getSelection());
        this.selectedCaseCount = this.selectedCases.length;

        if (this.selectedCaseCount > 0) {
            this.deleteActionNode.setDisabled(true);
            this.errorCaseCount = 0;
            this.deletedCaseCount = 0;

            for (var i = 0; i < this.selectedCaseCount; i++) {
                var responseHandler = new ServiceResponseHandler(this, "_deleteSuccess", "_deleteFailure");
                TestCaseClient.archiveTestCase(responseHandler,
                        {
                            uuid_versionable: this.selectedCases[i].versionableItemId,
                            archive: true
                        });
                if (this.isQueryPage)
                    this.viewHeaderNode.setProgressStart(Constants.getMessage("msgDeletingTestCase", this.selectedCases[i].name));
            }
        }
    },
	
	_deleteSuccess: function(response) {
        if (this.isQueryPage)
            this.viewHeaderNode.setProgressEnd();
        this.deletedCaseCount++;
        this._setDeleteStatus();

    },
    
    _deleteFailure: function(err) {
        if (this.isQueryPage)
            this.viewHeaderNode.setProgressEnd();
        this.errorCaseCount++;
        this._setDeleteStatus();
    },
    
    _setDeleteStatus: function() {
        if ((this.deletedCaseCount + this.errorCaseCount) == this.selectedCaseCount) {
            if (this.isQueryPage)
                this.viewHeaderNode.setStatus(
                    Constants.getMessage("msgDeletedTestCases", this.deletedCaseCount, this.selectedCaseCount),
                    this.errorCaseCount == 0 ? ViewHeaderLayout.SUCCESS : this.selectedCaseCount == 1 ? ViewHeaderLayout.ERROR : ViewHeaderLayout.INFO
                );
            if (this.deletedCaseCount > 0) {
                // notify all pages (including itself) test cases are deleted
                dojo.publish(GlobalEvents.ArtifactsDeleted, [Constants.TYPE_CASE, this.selectedCases]);
            }
        }
    },	

    // handler on artifacts changed
    onArtifactChanged: function(changeType, artifactType, artifactArray) {
        // only care about test cases
        if (artifactType == Constants.TYPE_CASE) {
            if (changeType == GlobalEvents.ArtifactsAdded) {
                // add test cases to the table
                if (!this.testCases) {
                    this.testCases = artifactArray;
                } else {
                    this.testCases = this.testCases.concat(artifactArray);
                }
                this.refresh(this.testCases, true);
            } else {
                // iterate through the array of artifacts
                for (var i = 0; i < artifactArray.length; i++) {
                    // iterate through the array of test cases in the table
                    for (var j = 0; j < this.testCases.length; j++) {
                        // handle the matching test case
                        if (artifactArray[i].id == this.testCases[j].id) {
                            if (changeType == GlobalEvents.ArtifactsUpdated) {
                                // swap test case on update
                                this.testCases.splice(j, 1, artifactArray[i]);
                                this.refresh(this.testCases, true);
                            } else if (changeType == GlobalEvents.ArtifactsDeleted) {
                                // remove test case from table on delete
                                this.testCases.splice(j, 1);
                                this.refresh(this.testCases, true);
                            }
                            break;
                        }
                    }
                }
            }
        }
        else if(artifactType == Constants.TYPE_TC_CATEGORY) {
            this._handleCategoriesChanged(artifactArray);
        }
    },
	
	tableHeaderColumnActionInvoked: function(columnIndex, actionId) {
		if(this.parent && this.parent._handleBatchUpdateClick) {
			this.parent._handleBatchUpdateClick(columnIndex, actionId, this.categoryTypes);
		}
	},
	
	_mergeCategory: function(categories, CategoryTypeItemId, CategoryValueItemId) {
		var newCategories = [];
 
		for(var i = 0; i < this.categoryTypes.length; i++){
			var type = this.categoryTypes[i];
			var newValueItemId = null;
			
			if (type.itemId == CategoryTypeItemId) {
				newValueItemId = CategoryValueItemId;
			}
			else {
				newValueItemId = this._getCategoryByType(type, categories);
			}
	        if(newValueItemId)
	            newCategories.push(newValueItemId);
		}
  
        return newCategories;
    },
  
    _getCategoryByType: function(type, categories) {
        for(var i = 0; i < (type.categories ? type.categories.length:0); i++) {
            for(var j = 0; j < (categories?categories.length:0); j++) {
                if(type.categories[i].itemId == categories[j])
                    return categories[j];
            }
        }
        return null;
    },
    
	_reContructTable: function(categoryTypes, testCases) {
		if(testCases)
			this.tableViewer.setContent(testCases);
			
		var currentPageCtr = this.tableViewer.getCurrentPageControl();	
		//reset test cases' table
        this.provider = new TestCaseTableView.TableProvider({
            profile: this.profile ? this.profile : Constants.TABLE_PROFILE_ALL,
            categoryTypes: categoryTypes,
            newItemLinkCallback: this.newItemLinkCallback,
            enableRowEditable: this.enableRowEditable
        });
		this.tableViewer.setContentProvider(this.provider);
//        this.tableViewer.show();
		this.tableViewer.reconstruct(currentPageCtr);
	},

	_confirmResetViewBuilder: function(newCategoryFilters) {
        //reset ViewBuilder
		var self = this;
        var args = {
            title: Constants.getMessage("labelConfirmation"),
            message: Constants.getMessage("msgChangeViewBuildConfirm"),
            yesCallback: function() {
				self._resetViewBuilder(newCategoryFilters);
			},
            noCallback: function(){
                return false;
            }
        }
        var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        dialog.setSecondaryTitle(args.title);
        dialog.open();
	},
	
	_resetViewBuilder: function(newCategoryFilters) {
		if (this.filterProvider) {
			var newFilters = [];
			var oldFilters = this.filterProvider.filters;
			for(var i = 0; i < oldFilters.length; i++) {
				if(oldFilters[i].name.indexOf("categories") == -1)
					newFilters.push(oldFilters[i]);
			}
			this.currentCategoryFilters = newCategoryFilters;
			newFilters = newFilters.concat(this.currentCategoryFilters);
			delete this.filterProvider.filters;
			this.filterProvider.filters = newFilters;
			
			this.tableViewer.setContentFilterAttributeProvider(this.filterProvider);
		}
	},
	
    _error: function(e) {
        console.error(this.declaredClass + ": Error: " + e.message);
    }
    
});

})();
