/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.table.cellrenderers.ScriptComboCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.table.cellrenderers.ScriptComboCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
		var selectLabel = document.createElement("label");
       
		var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        

        node = document.createElement("select");
    
        node.className = "combo-cell";
        node.cellModifier = cellModifier;
        node.rowObject = rowObject;
        node.columnIndex = columnIndex;
        node.lastSelected = null;
        
        if(typeof cellWidthHint != "undefined") {
            node.style.width = cellWidthHint + "px";
        }
        node.onchange = function() {
            var val = null;
            var lastSelected = this.lastSelected;
        
            for(var i = 0; i < this.options.length; i++) {
                if(this.options[i].selected) {
                    val = this.options[i].value;
                }
            }
            this.lastSelected = val;
            if(this.cellModifier)
                this.cellModifier.cellValueChanged(lastSelected, val, this.rowObject, this.columnIndex);
        };
        
        var tooltip = null;
        if (extendedDisplayInfo && extendedDisplayInfo['choices']) {
			var selected = (typeof extendedDisplayInfo['select'] != 'undefined') ? extendedDisplayInfo['select'] : cellData;
			tooltip = extendedDisplayInfo['tooltip'];
			
			
			// select combo box node whenever it is editable or not
			this.setEnumChoices(node, extendedDisplayInfo['choices'], selected);
		}
        
        // the parent class's attach tooltip will not create a tooltip
        // if the string is null, or undefined. Don't have to check it here.
        this._attachTooltipToCell(node, tooltip);    
        
		selectLabel.appendChild(node);
        return selectLabel;
    },
    
    setEnumChoices: function(select, choices, value)
    {
        var hasSelect = false;
        
        if(choices && choices.length > 0) {
            for (var i = 0; i < choices.length; i++) {
                select.options[i] = new Option(choices[i].text, choices[i].value);
                
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(value && choices[i].value == value) {
                    select.options[i].selected = true;
                    hasSelect = true;
                }    
            }
            // select the first item by default.
            if(!hasSelect)
                select.options[0].selected = true;
        }
    }    
});

})();
