/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.viewlets.ReportViewlet");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.viewlets.BaseReportViewlet");

(function() {

var Viewlet 					= com.ibm.team.dashboard.web.ui.Viewlet;
var BaseReportViewlet			= com.ibm.rqm.planning.web.ui.internal.view.viewlets.BaseReportViewlet;

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;	
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.ReportViewlet",	
	BaseReportViewlet,
{
	/*
	 * Static Variable Container 
	 */
	_static: function() {},
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {
		// Call super
		this.inherited("init", arguments);

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		// Cascading parameters map
		this._groupParametersExecution = null;
		this._singleParametersTestCase = null;
		this._singleParametersMachine = null;
		this._singleParametersOwner = null;
		this._singleParametersState = null;
		
		// Parameter URI names
		this._parameterURINames = new Array();
		this._setParameterURINames();
		
		// Get the current report preference value
		this._reportUUID = this.getPreference(this._staticCommon._PREFERENCE_ID_REPORT);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {
		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}
		
		// Preferences
		this._static._PREFERENCE_ID_PLAN = "plan";
		this._static._PREFERENCE_ID_ITERATION = "iteration";
		this._static._PREFERENCE_ID_TESTCASE = "testcase";
		this._static._PREFERENCE_ID_MACHINE = "machine";
		this._static._PREFERENCE_ID_OWNER = "owner";
		this._static._PREFERENCE_ID_STATE = "state";
		
		// Cascading parameter groups
		this._static._PARAMETER_GROUP_NAME = "PlanIterationTestcase";
		
		// Single parameters
		this._static._PARAMETER_SINGLE_TESTCASE = "testcase";
		this._static._PARAMETER_SINGLE_MACHINE = "machine";
		this._static._PARAMETER_SINGLE_OWNER = "owner";
		this._static._PARAMETER_SINGLE_STATE = "state";
		
		// Work Item Micro Reports
		this._staticCommon._EXECUTION_TREND = "execution.OverallStatus";
		this._staticCommon._microReportIDMap[this._staticCommon._EXECUTION_TREND] = "execution.OverallStatus";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_setParameterURINames: function() {
		// Test Plan
		this._parameterURINames.push("plan");
		// Iteration
		this._parameterURINames.push("iteration");
		// Test case
		this._parameterURINames.push("testcase");
		// Machine
		this._parameterURINames.push("machine");
		// Owner
		this._parameterURINames.push("owner");
		// State
		this._parameterURINames.push("state");
		// Zoom
		this._parameterURINames.push(this._staticCommon._PARAMETER_NAME_ZOOM);
		// Width
		this._parameterURINames.push("width");
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_resetCascadingParameters: function() {
		this._groupParametersExecution = null;
		this._singleParametersTestCase = null;
		this._singleParametersMachine = null;
		this._singleParametersOwner = null;
		this._singleParametersState = null;
	},

	/*
 	 * Public Method
 	 * 
 	 * Called to update viewlet content (e.g. Settings changed)
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
	    this._updateLoadingMessage(Constants.getMessage("labelRefreshingReport"));
		
		this._getParameters();
		
		this._getReport(
			this._parameterURINames,
			this.getParameterValues());
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */
	_getReport: function(parameterNames, parameterValues) {
		// debug parameters
		// console.debug("Original parameter and values:");
		// this._debugParameters(parameterNames, parameterValues);
				
		// if the parameterValues contains null value, remove all the parameters except for the 'zoom'
		var _report_parameterNames = new Array();
		var _report_parameterValues = new Array();
		
		//if (this._hasNullValues(parameterValues)) {			
		//	_report_parameterNames.push(this._staticCommon._PARAMETER_NAME_ZOOM);
		//	_report_parameterValues.push(this._quote(this.isZoomed()));
		//} else {
		//	_report_parameterNames = parameterNames;
		//	_report_parameterValues = parameterValues;
		//}
		_report_parameterNames = this._deleteNullNames(parameterValues,parameterNames);
		_report_parameterValues = this._deleteNullValues(parameterValues);
		
		// debug parameters
		// console.debug("New parameter and values:");
		// this._debugParameters(_report_parameterNames, _report_parameterValues);
		
		// Generate the hash key for the report request
		var hashKey = this._hashArray(parameterValues);
		// Check to see if the report is cached
		// var cachedHtml = this._getReportFromCache(hashKey);
		// var isReportCached = (cachedHtml != null);
		// Display whether the report is cached or live
		// this._displayCacheStatus(isReportCached);
		// Display cached report
		// if (isReportCached) {
			// Add the report to the display area
			// this._addReportToViewlet(cachedHtml);
			// Service invocation unnecessary
			// this.getSite().doneRefresh();
			// return;
		// }
		// Generate the click-through URL
		this._generateClickThroughURL(
			this._reportUUID, 
			_report_parameterNames, 
			_report_parameterValues
		);
		
		// Update loading message
		this._updateLoadingMessage(Constants.getMessage("labelLoading"));
		// Show the loading message
		this._showLoadingArea();
		
		// Remember the zoom state at the time of this request
		var previousZoomState = this.isZoomed();
		
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(html) {
				self._processReportHTML(html, hashKey, previousZoomState);
				self.getSite().doneRefresh();
			},
			// Service failure
			failure: function(errorObject) {
				self._logError(errorObject);
				self._displayErrorMessage(Constants.getMessage("labelReportLoadingFailed"), errorObject);
				self.getSite().doneRefresh();
			}
		};
	
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		this._callRESTGetRenderedReport(
			handler, 
			this._reportUUID,
			_report_parameterNames, 
			_report_parameterValues);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @param Object currentSettings
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceOptions: function(preferenceId, currentSettings) {
		// Note:  Incomplete preference not handled because it is not dynamic
		// Get preference values
		if (preferenceId == this._static._PREFERENCE_ID_PLAN) {
			// Test Plan
			if (this._groupParametersExecution == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._groupParametersExecution == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionTestPlan();
			
		} else if (preferenceId == this._static._PREFERENCE_ID_ITERATION) {
			// Iteration
			if (this._groupParametersExecution == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._groupParametersExecution == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionIteration(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_TESTCASE) {
			// Test Case
			if (this._singleParametersTestCase == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersTestCase == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionTestCase(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_MACHINE) {
			// Machine
			if (this._singleParametersMachine == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersMachine == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionMachine(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_OWNER) {
			// Owner
			if (this._singleParametersOwner == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersOwner == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionOwner(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_STATE) {
			// State
			if (this._singleParametersState == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersState == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionState(currentSettings);
			
		} else {
			var errorObject = new Error("Unrecognized preference: " + preferenceId);
			this._logError(errorObject);
			this._displayErrorMessage(Constants.getMessage("labelPreferenceLoadingFailed"), errorObject);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Array
 	 * @overrides super
 	 */		
	settingChanging: function(preferenceId) {
		var depedencies = new Array();
		if (preferenceId == this._static._PREFERENCE_ID_PLAN) {
			// Test Plan -> Iteration
			depedencies.push(this._static._PREFERENCE_ID_ITERATION);
		}
		return depedencies;
	},

    /*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_getParameters: function() {
		// Update loading message
		this._updateLoadingMessage(Constants.getMessage("labelLoading"));
		// Show the loading message
		this._showLoadingArea();
		// Asynchronous call
		this._getGroupParameters(
			this._static._PARAMETER_GROUP_NAME,
			this._callBackSuccessGetGroupParameters,
			this._callBackErrorGetGroupParameters);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array
 	 */	
	getParameterValues: function() {		
		var parameterValues = new Array();
		
		// Test Plan
		var testPlan = this.getPreference(this._static._PREFERENCE_ID_PLAN);		
		testPlan = this._normalizeParameterValue(testPlan);
		parameterValues.push(testPlan);
		
		// Iteration
		var iteration = this.getPreference(this._static._PREFERENCE_ID_ITERATION);		
		iteration = this._normalizeParameterValue(iteration);
		parameterValues.push(iteration);
		
		// Test Case
		var testCase = this.getPreference(this._static._PREFERENCE_ID_TESTCASE);		
		testCase = this._normalizeParameterValue(testCase);
		parameterValues.push(testCase);
		
		// Machine
		var machine = this.getPreference(this._static._PREFERENCE_ID_MACHINE);		
		machine = this._normalizeParameterValue(machine);
		parameterValues.push(machine);
		
		// Owner
		var owner = this.getPreference(this._static._PREFERENCE_ID_OWNER);		
		owner = this._normalizeParameterValue(owner);
		parameterValues.push(owner);
		
		// State
		var state = this.getPreference(this._static._PREFERENCE_ID_STATE);		
		state = this._normalizeParameterValue(state);
		parameterValues.push(state);
		
		// Zoom
		parameterValues.push(this._quote(this.isZoomed()));
		
		// width
		parameterValues.push(this.container.domNode.clientWidth);
		return parameterValues;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return boolean
 	 * @overrides super
 	 */		
	_isSettingsEqual: function(oldSettings, newSettings) {

		if (oldSettings.preferences.plan != newSettings.preferences.plan) {
			return false;
		} else if (oldSettings.preferences.iteration != newSettings.preferences.iteration) {
			return false;
		} else if (oldSettings.preferences.testcase != newSettings.preferences.testcase) {
			return false;
		} else if (oldSettings.preferences.machine != newSettings.preferences.machine) {
			return false;
		} else if (oldSettings.preferences.owner != newSettings.preferences.owner) {
			return false;
		} else if (oldSettings.preferences.state != newSettings.preferences.state) {
			return false;
		}
		
		return true;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 */		
	_getPreferenceOptionTestPlan: function() {
		// Get parameters		
		var executionResult = this._groupParametersExecution;		
		var testPlan = null;
		try {
			// Get test plan from the dto
			testPlan = executionResult.parameters[0];
		} catch(errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(testPlan, null);
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}	
		return result;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionIteration: function(currentSettings) {
		// Get parameters
		var executionResult = this._groupParametersExecution;
		var iteration;
		try {
			// Get the current preference value for the team area
			// Cascades: Test Plan -> Iteration
			var testPlanValue = currentSettings.preferences.plan;
			var index = executionResult.parameters[0].values.indexOf(testPlanValue);
			var allIterations = executionResult.parameters[0].children;			
			iteration = allIterations[index];
		} catch(errorObject) {
			return this._getPreferenceFailureList();
		}

		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(iteration, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		return result;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionTestCase: function(currentSettings) {
		// Get parameters
		var testCase = this._singleParametersTestCase;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(testCase, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionMachine: function(currentSettings) {
		// Get parameters
		var machine = this._singleParametersMachine;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(machine, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionOwner: function(currentSettings) {
		// Get parameters
		var owner = this._singleParametersOwner;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(owner, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionState: function(currentSettings) {
		// Get parameters
		var state = this._singleParametersState;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(state, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @param com.ibm.team.reports.rest:ReportParameterGroupDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetGroupParameters: function(self, groupName, dto) {
		
		// Debug
		// console.debug(dto);
		
		try {
			// Create a map out of the DTO
			var map = self._processDTOGroupParameters(dto);
			// Store global
			self._groupParametersExecution = dto;
		} catch (e) {
			self._groupParametersExecution = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
		
		
		self._getSingleParameters(
			self._static._PARAMETER_SINGLE_TESTCASE,
			self._callBackSuccessGetSingleParameterTestCase,
			self._callBackErrorGetSingleParameters);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameterTestCase: function(self, parameterName, dto) {
				
		// Debug
		//console.debug(dto);
		
		try {
			// Create a custom object out of the DTO
			var result = self._processDTOSingleParameters(dto);
			// Store global
			self._singleParametersTestCase = result;
		} catch (e) {
		 	self._singleParametersTestCase = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
	
		self._getSingleParameters(
			self._static._PARAMETER_SINGLE_MACHINE,
			self._callBackSuccessGetSingleParameterMachine,
			self._callBackErrorGetSingleParameters);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameterMachine: function(self, parameterName, dto) {
		
		try {
			// Create a custom object out of the DTO
			var result = self._processDTOSingleParameters(dto);
			// Store global
			self._singleParametersMachine = result;
		} catch (e) {
			self._singleParametersMachine = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
		
		
		self._getSingleParameters(
			self._static._PARAMETER_SINGLE_OWNER,
			self._callBackSuccessGetSingleParameterOwner,
			self._callBackErrorGetSingleParameters);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameterOwner: function(self, parameterName, dto) {
		try {
			// Create a custom object out of the DTO
			var result = self._processDTOSingleParameters(dto);
			// Store global
			self._singleParametersOwner = result;
		} catch (e) {
		 	self._singleParametersOwner = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
		self._getSingleParameters(
			self._static._PARAMETER_SINGLE_STATE,
			self._callBackSuccessGetSingleParameterState,
			self._callBackErrorGetSingleParameters);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameterState: function(self, parameterName, dto) {
		try {
			// Create a custom object out of the DTO
			var result = self._processDTOSingleParameters(dto);
			// Store global
			self._singleParametersState = result;
		} catch (e) {
		 	self._singleParametersState = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}	
			// Show the display area
		self._showDisplayArea();
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @return void
 	 */
	_callBackErrorGetSingleParameters: function(self, parameterName) {
		// Set error
		self.parameterName = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeParameterValue: function(value) {
		if (value == this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
			return this._quote("");
		}
		return this._quote(value);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @return void
 	 */
	_callBackErrorGetGroupParameters: function(self, groupName) {
		// Set error
		self._groupParametersExecution = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},
	
	/*
 	 * Private Method
 	 * Check whether the array contains the null value
 	 * 
 	 * @param Array parameterValues
 	 * @return boolean
 	 */	
	_hasNullValues: function(parameterValues) {
		for(var j in parameterValues ) {
			if( parameterValues[j] == "'null'" && parameterValues[j] != "''" && parameterValues[j] != "'{Null}'" ) {
				return true;
			}
		}	
		return false;
	},
	/*
 	 * Private Method
 	 * Delete the parameter value array contains the null value
 	 * 
 	 * @param Array parameterValues
 	 * @return newParameterValus
 	 */	
	_deleteNullValues: function(parameterValues) {
	
		var newParaValues = new Array();
		for( var j in parameterValues ) {
			if( parameterValues[j] != "'null'" && parameterValues[j] != "''" && parameterValues[j] != "'{Null}'") {
				newParaValues.push(parameterValues[j]);
			} 
		}
		return newParaValues;
	},
	
	/*
 	 * Private Method
 	 * Delete the parameter name array that parameter value is null
 	 * 
 	 * @param Array parameterValues
 	 * @param Array parameterNames
 	 * @return newParameterNames
 	 */	
	_deleteNullNames: function(parameterValues, parameterNames) {
	
		var newParaNames = new Array();
		for( var j in parameterValues ) {
			if( parameterValues[j] != "'null'" && parameterValues[j] != "''" && parameterValues[j] != "'{Null}'" ) {
				newParaNames.push(parameterNames[j]);
			} 
		}
		return newParaNames;
	},
	/*
 	 * Private Method
 	 * 
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */
	_debugParameters: function(parameterNames, parameterValues) {
		for( var i in parameterNames ) {
			console.debug("Parameter Name" + i + ":" + parameterNames[i]);
		}
		
		for( var j in parameterValues ) {
			console.debug("Parameter Value" + j + ":" + parameterValues[j]);
		}
	}
});

})();