/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.viewlets.ViewletPage");

dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletPage");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.viewlets.ViewletContainer");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

(function() {
	
var ViewletContainer = com.ibm.rqm.planning.web.ui.internal.view.viewlets.ViewletContainer;
var ViewletRegistry = com.ibm.team.dashboard.web.client.internal.ViewletRegistry;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.ViewletPage", 
				  com.ibm.team.dashboard.web.ui.internal.ViewletPage, 
{
	/*
	 * Adds a viewlet widget to the dashboard either at the top or bottom
	 * of a specific column (does not update server state). A viewlet index
	 * of -1 or undefined will place the viewlet at the bottom of the
	 * column.
	 */
	_addViewlet: function(/* ViewletHelper */ viewletHelper, /* number */ columnIndex, /* number */ viewletIndex) {
		var serviceObj = {
			_columnIndex: columnIndex,
			_viewletHelper: viewletHelper,
			_viewletIndex: viewletIndex,
			_viewletPage: this,
			success: function(/* ViewletDefinitionDTO[] */ viewletDefinitionDTOs) {
				// set up params
				var args = {
					viewletHelper: this._viewletHelper,
					viewletDefinition: viewletDefinitionDTOs[0],
					viewletPage: this._viewletPage,
					columns: this._viewletPage.columns,
					columnIndex: this._columnIndex
				};
				// add viewlet to destination column
				var column = this._viewletPage.columns[this._columnIndex];
				var viewletContainer = new ViewletContainer(args);
				
				if (this._viewletIndex != -1 && column.widgets.length >= this._viewletIndex) {
					// insert at given index
					column.add(viewletContainer, {index: this._viewletIndex, handle: viewletContainer._header, type: "viewlet", animate: true});
				} else {
					// append to the end of the column
					column.add(viewletContainer, {handle: viewletContainer._header, type: "viewlet", animate: false});
				}
				this._viewletPage._setEmptyTabMessage(false);
			},
			failure: function(/* Error */ error) {
				error.message = "The viewlet definition could not be fetched: " + error.message;
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		ViewletRegistry.getDefinitions([viewletHelper.getDefinitionId()], srh, this.dashboardContainer.getServiceTracker());
	}
});

})();

