/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineSearchTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	
	var column_types = {
		'Machine': [TableConstants.RENDER_TYPE_IMAGE, TableConstants.RENDER_TYPE_LINK, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_IMAGE ],
		'VirtualImage': [TableConstants.RENDER_TYPE_IMAGE, TableConstants.RENDER_TYPE_LINK, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT ]
	};
    var SEARCH_ATTRIBUTE_HEADINGS = {
        'Machine' : [
		  {title: "", native_column: "machine\\.name", isSortable: false, includeInFilter: false, width: 1}, 
		  {title: RtlmUiUtil.getMessage("pl_name"), native_column: "machine\\.name", isSortable: true, includeInFilter: true, width: 23}, 
		  {title: RtlmUiUtil.getMessage("pl_group"), native_column: "machine\\.group", isSortable: true, includeInFilter: true, width: 25  },
		  {title: RtlmUiUtil.getMessage("pl_operatingsystem"), native_column: "machine\\.operatingsystem", isSortable: true, includeInFilter: true, width: 25  },
		  {title: RtlmUiUtil.getMessage("pl_cpu"), native_column: "machine\\.cpu", isSortable: true, includeInFilter: true, width: 25  },
		  {title: RtlmUiUtil.getMessage("Reserved"), native_column: "Reserved", isSortable: false, includeInFilter: false, width: 1  }
		],
		'VirtualImage' : [
          {title: "", native_column: "VirtualImage\\.name", isSortable: false, includeInFilter: false, width: 5}, 
          {title: RtlmUiUtil.getMessage("VirtualImage.name"), native_column: "VirtualImage\\.name", isSortable: true, includeInFilter: true, width: 25}, 
          {title: RtlmUiUtil.getMessage("pl_group"), native_column: "VirtualImage\\.group", isSortable: true, includeInFilter: true, width: 20  },
          {title: RtlmUiUtil.getMessage("pl_ImageType"), native_column: "VirtualImage\\.ImageType", isSortable: true, includeInFilter: true, width: 20  },
          {title: RtlmUiUtil.getMessage("pl_operatingsystem"), native_column: "VirtualImage\\.operatingsystem", isSortable: true, includeInFilter: true, width: 25  },
          {title: RtlmUiUtil.getMessage("pl_VMFile"), native_column: "VirtualImage\\.VMFile", isSortable: true, includeInFilter: true, width: 25  }
		]
    };
    
    var ATTRIBUTE_ID_FIELDS = {
        'Machine' : "machine\\.id",
        'VirtualImage' : "VirtualImage\\.id"
    };
    
    var SEARCH_ATTRIBUTE_CRITERIA = {
        'Machine' : "machine.id, machine.name, machine.group, machine.ipAddress, machine.operatingsystem, machine.cpu",
        'VirtualImage' : "VirtualImage.id, VirtualImage.name, VirtualImage.group, VirtualImage.ImageType, VirtualImage.operatingsystem, VirtualImage.VMFile"
    };
    

com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineSearchInfo = {
	getSearchAttributeHeadings : function()
	{
		return( SEARCH_ATTRIBUTE_HEADINGS );
	},
	
	getIdFields: function()
	{
		return( ATTRIBUTE_ID_FIELDS );
	},
	
	getSearchAttributeCriteria: function()
	{
		return( SEARCH_ATTRIBUTE_CRITERIA );
	}
};

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineSearchTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider,
{
	constructor: function( assetType )
	{
		this.setParameters( { "column_headings" : SEARCH_ATTRIBUTE_HEADINGS,
					  "id_fields" : ATTRIBUTE_ID_FIELDS,
					  "column_types" : column_types,
					  "column_section" : assetType } );
		this.assetType = assetType;
	},
    
	    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
    	var type = this.inherited("getCellDisplayType", arguments);
		var headings = this.column_headings[this.columns_section];
    
		//If this is the "reserved column and the machine is not reserved then we'll use an empty string as the data
		//so indicate that the cell display type is text	
    	if (headings[atColumnIndex]["native_column"] == "Reserved") 
    	{
			if (!this.receiver._isMachineReserved(rowData["machine.id"]))
	   			return( TableConstants.RENDER_TYPE_TEXT );
    	}

		return( type );
    },
	getCellDisplayData: function(rowData, atColumnIndex)
    {
		var data = this.inherited( "getCellDisplayData", arguments );
		var headings = this.column_headings[this.columns_section];
    	
    	if (headings[atColumnIndex]["native_column"] == "Reserved") 
		{
			if (this.receiver)
			{
				if (this.receiver._isMachineReserved(rowData["machine.id"]))
					data = dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/images/icons/elcl16/reserve.gif");
				else
					data = ""
			}
			else
				data = "Reserve";
		}
    	
    	return( ( data ) ? data : "" );
    },
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
        if(atColumnIndex == 0) {
            val = {};
 			var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
            val['href'] = AssetUriUtil.getViewAssetDetailURI( this.columns_section, this.getRowDataId( rowData ) );
			val.image = this.assetType == "Machine" ? ImageCache.MACHINE : ImageCache.VIRTUALIMAGE;
        }
        if(atColumnIndex == 1) {
            val = {};
 			var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
            val['href'] = AssetUriUtil.getViewAssetDetailURI( this.columns_section, this.getRowDataId( rowData ) );
        }

        return val;
    },
           
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return( atColumnIndex > 1 &&  atColumnIndex < 5 );
    }     
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineSearchListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	constructor: function( params )
	{
		for( var param in params )
		{
			this[ param ] = params[ param ];
		}
	},
	
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
    	if( this.receiver )
    	{
    		this.receiver._checkButtonStatus();
    	}
    }
});


})();
