/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.ConfigSummary");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
//xojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea");

(function() 
{
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	
	dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.ConfigSummary", [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="config-summary" dojoAttachPoint="domNode" tabindex="-1">' +
                    '    <table dojoAttachPoint="SummaryTable" class="config-table">' +
					'     <tr align = "left">' +
					'       <td>' +
					'         <div dojoAttachPoint="nameArea" align="left" display="none"></div>' +
					'         <div dojoAttachPoint="defaultTextArea" align="left" display="none"></div>' +
					'       </td>' +
					'     </tr>' +
                    '     <tr align="left">' +
                    '       <td width="100%" align="left">' +
                    '         <div dojoAttachPoint="configArea"></div>' +
                    '       </td>' +
                    '     </tr>' +
                    '   </table>' +
					'</div>',

    widgetsInTemplate: true,


	//The name of the config
	configName: null,
	
	asset: null,
	
	//The description of the config
	configDescription: null,
	
	// the asset as a string
	//Either a configString or a configPart should be specified but not both 
	configString: null,	
	//A part to display
	configPart:null,
	//the id (if any) of the config
	configID:null,
	
	// the type of the assset
	configType: null,
	
	// interal catalog object
	theCatalog: null,
	
	configTable: null,
	
	maxColumns:4,
	
	ATTNAME:"Name",
	ATTVALUE:"Value",

	postCreate: function() 
	{
        this.theCatalog = new com.ibm.asq.common.web.client.internal.Catalog();

		//Load create the asset from the string
		if (this.configString != null)
			this.asset = this.theCatalog.createConfigurationFromJson(this.configString);
		else
			this.asset = null;
		
		//Construct the widget
		this._constructWidget();
	},
	
	_constructWidget :function()
	{
		//Hide the default text area
		if (this.defaultTextArea != null)
			this.defaultTextArea.style.display = "none";
		//Show the config area
		this.configArea.style.display = "inline";
		
		//Create the HTML table control that we will fill
		if (this.configTable == null) 
		{
			this.configTable = document.createElement("table");
			this.configTable.cellPadding = "0";
			this.configTable.cellSpacing = "0";
			this.configTable.border = "0";
		}
		else
		{
			//reloading an existing config so remove all the table rows
			while(this.configTable.rows.length > 0)  
		        this.configTable.deleteRow(this.configTable.rows.length-1); 
		}
		//If there is a description, show it first
		if (this.configDescription != null)
		{
			var descriptionRow = this.configTable.insertRow(0);
			var td = descriptionRow.insertCell(0);
			var txt = document.createTextNode(this.configDescription);
			td.appendChild(txt);
		}
		
		//Now build up an array of name-value pairs of attribute names and values
		//The array should only hold pairs where the value is not null/empty
		var arrayOfAtts = new Array();

		if (this.asset != null) {
			var partCount = this.asset.getRootParts().length;
			var createSection = (partCount > 1);
			for (var i = 0; i < partCount; i++) {
				var part = this.asset.getRootPart(i);
				arrayOfAtts = new Array();
				this._processPart(part, arrayOfAtts);
				if (createSection) {
					var sectionTable = this._createPartSection(i);
					var parentTable = sectionTable ? sectionTable : this.configTable;
					this._buildTableBody(arrayOfAtts, parentTable);
				}
				else 
					this._buildTableBody(arrayOfAtts, this.configTable);
			}
		}
		else 
			if (this.configPart != null) {
				this._processPart(this.configPart, arrayOfAtts);
				this._buildTableBody(arrayOfAtts, this.configTable);
			}
		
		this.configArea.appendChild(this.configTable);
	},
	

	_processPart: function(part, arrayOfAtts)
	{
		if (part == null)
			return;
		
		//Process each attribute of the part	
		var attributeNames = part.getAttributeNames();

		for (var i = 0; i < attributeNames.length; i++) 
		{
			//Get the attribute
			var attribute = part.getAttribute(attributeNames[i]);
			//Get the type of the attribute
			var type = attribute.getType();
			var name = attribute.getName();
			var attValue = null;
			
			//First handle the case where the attribute is an array.
			if (attribute.isArray()) 
			{
				var arrayValues = attribute.getArrayValues();
				for (var j = 0; j < arrayValues.length; j++)
				{
					var subAttribute = arrayValues[j];
					if (subAttribute == null)
						continue;
					if (attribute.getType() == 'ConfigurationPartType') 
					{
						var value = attribute.getArrayValue(j).getName();
						if (value != null)
							arrayOfAtts.push({ATTNAME: name, ATTVALUE: value});
						this._processPart(subAttribute, arrayOfAtts);
					}
					else 
						this._processAttribute(subAttribute, arrayOfAtts);
				}
				//if an array type then we're done with this attibute
				continue;
			}
			this._processAttribute(attribute, arrayOfAtts);
		}
	},
	

	_processAttribute :function(attribute, arrayOfAtts)
	{
		//If we're here then the attribute cannot be an array type.
		
		var type = attribute.getType();
		var name = attribute.getName();

		//Let's handle the part type first
		if (type == 'ConfigurationPartType')
		{
            var part = attribute.getValue();
            if (part != null) {
                var value = part.getName();
                arrayOfAtts.push({
                    ATTNAME: name,
                    ATTVALUE: value
                });
                this._processPart(part, arrayOfAtts);
            }
            else {
                arrayOfAtts.push({
                    ATTNAME: name,
                    ATTVALUE: null
                });
            }
		}
		if (type == 'StringType') 
		{
			var stringValue = attribute.getValue();
			if (stringValue != null)
				arrayOfAtts.push({
					ATTNAME:name, 
					ATTVALUE:stringValue
				});
		}
		if (type == 'NumericType') 
		{
			var numValue = attribute.getValue();
			arrayOfAtts.push({
					ATTNAME:name, 
					ATTVALUE:numValue
					});
		}
	},

	/**
	 * When there is multiple parts in this config, this function is called to create a section for each part to shown in. Subclass should override.
	 * @param {Object} sectionTitle
	 */
	_createPartSection: function(partIndex) {
		return null;
	},

	_buildTableBody :function(arrayOfAtts, parentTable)
	{
		if (arrayOfAtts == null || arrayOfAtts.length == 0)
			return;
		
		//Loop over all the attribute and create table rows.
		//Each row will contain at most this.maxColumns columns
		var colCount = 0;
		var currentRow = null;
		for (var i = 0; i < arrayOfAtts.length; i++)
		{
			var name = arrayOfAtts[i].ATTNAME;
			var value = arrayOfAtts[i].ATTVALUE;
			
			if (name == null || name.length == 0 || value == null || value.length == 0)
				continue;
			if (colCount == 0) 
				currentRow = parentTable.insertRow(parentTable.rows.length);

			var td = currentRow.insertCell(0);
			
			//Get the display name for the attribute from our string catalog
			//If there is no corresponding name in the catalog use the attribute name directly
			var displayName = RtlmUiUtil.getMessage(name);
			if (displayName == null || displayName.length == 0)
				displayName = name;
			var labelSpan = document.createElement("span");
			labelSpan.className="attlabel";
			var lbltxt = document.createTextNode(displayName + "  ");
			labelSpan.appendChild(lbltxt);
			td.appendChild(labelSpan);
			
			var valueSpan = document.createElement("span");
			valueSpan.className="attvalue";
			
			var valtxt = document.createTextNode(value);
			valueSpan.appendChild(valtxt);
			td.appendChild(valueSpan);

			colCount++;
			
			if (currentRow != null && colCount == this.maxColumns) 
			{
				currentRow = null;
				colCount = 0;
			}
		}
		//Spit out the last row if necessary
//		if (currentRow != null) 
//			tBody.appendChild(currentRow);
	},
	
	setPart :function(part)
	{
		this.configPart = part;
	},
	
	setConfigString: function(configString)
	{
		this.configString = configString;
		this.asset = this.theCatalog.createConfigurationFromJson(this.configString);
	},
	
	refresh: function()
	{
		this._constructWidget();
	},
	
	setMaxColumns :function(columns)
	{
		this.maxColumns = columns;
	},
	
	setDefaultText: function(str)
	{
		//Hide the config area
		this.configArea.style.display="none";
		this.defaultTextArea.style.display="inline";
		this.defaultTextArea.innerHTML = str;
	},
	
	setDefaultClass: function(theClass)
	{
		dojo.addClass(this.defaultTextArea, theClass);
	},
	
	setName: function(str)
	{
		this.nameArea.style.display="inline";
		this.nameArea.innerHTML = str;
	},
	
	clearName: function()
	{
		this.nameArea.innerHTML = "";
		this.nameArea.style.display="none";
	},
	
	setNameClass: function(theClass)
	{
		dojo.addClass(this.defaultTextArea, theClass);
	}
});

})();
