/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //   The Directory pane displays a list of client specified entries that are the
 //   "bookmarks" to different locations on the current page. When an entry in the pane is
 //   clicked, the user is scrolled to the location on the page. It's subclassed from the
 //   StationaryPane which can track the current scroll location of the page and always keeps
 //   the pane in view (by auto sliding up or down). The user can disable the tracking. 
 //   
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.DirectoryPane");

dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.StationaryPane");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");

( function() {

var Constants = com.ibm.asq.common.web.client.internal.Constants;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.DirectoryPane",
             com.ibm.asq.common.web.ui.internal.widgets.StationaryPane,
{   
    // initializes the parent class's attributes 
    title: Constants.getMessage("titleTOC"),
    trackMessage: Constants.getMessage("msgUnpinView"),
    untrackMessage: Constants.getMessage("msgPinView"),
        
    constructor: function() {
        this.entries = [];
        this.showAll = false;
    },
    
    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        
        if(this.entries) {
            this.setEntries(this.entries);
        }        
    },
    
    // Summary:
    //    loads the table of contents entries in the pane. This method expects
    //    an array of entries info to be passed. The valid data is as follows:
    //
    //    var entries = [
    //       {title: "name of the entry",
    //        image: "image src ref", (Optional to create an image for the entry)
    //
    //     provide one or the other (either a dom node for the entry, or an anchor's tag)
    //
    //        node: domNodRef, (Optional) the dom node of the entry to scroll to
    //        tag: "anchorTag" (Optional) the anchor's name to scroll to   
    //    ];
    //    
    //
    setEntries: function(entries)
    {
        this.entries = entries;           
        dojox.data.dom.removeChildren(this.containerNode);
        this.entriesNode = document.createElement("ul");
        this.entriesNode.className = "entries";
        
        if(this.entries && this.entries.length > 0) {            
            for(var i = 0; i < this.entries.length; i++) {               
                var aLink = document.createElement("a");   
				aLink.setAttribute("href", "javascript:{}");             
                if(this.entries[i].image) {
                    var img = document.createElement("img");
                    img.src = this.entries[i].image;
                    aLink.appendChild(img);
                }
                if(this.entries[i].title) {
					var node = document.createElement("span");
					node.style.left = "-3000px";
					node.style.position = "absolute";
					node.style.display = "inline";
					node.innerHTML = Constants.getMessage("skipto");
					aLink.appendChild(node);
                    aLink.appendChild(document.createTextNode(this.getWordWrappedString( this.containerNode, this.rulerNode, this.entries[i].title )));
                }
                if(this.entries[i].tag) {
                    //aLink.href = window.location + "#" + this.entries[i].tag;
                    aLink.onclick = dojo.hitch(this, "selectEntry", i);
                }
                /*
                if(this.entries[i].node) {
                    aLink.refNode = this.entries[i].node;
                    aLink.onclick = function() {
                        dijit.scrollIntoView(this.refNode);
                    }
                }
                */
                var li = document.createElement("li");
                li.appendChild(aLink);
                this.entriesNode.appendChild(li);
            }                        
        } else {
            var li = document.createElement("li");
            li.innerHTML = Constants.getMessage("Empty");
            this.entriesNode.appendChild(li);
        }
        this.containerNode.appendChild(this.entriesNode);
        
        //add the show all option if its enabled
        if (this.showAllCallback != null) {
            this.addShowAllOption();    
        }        
    },
    
    selectEntry: function(index) {  
		if( window.event )
		{
			//This code will only run under IE.  It prevents the link from propagating to perform
			//href action for the link.  In this way it will prevent the dialog from coming up telling
			//the user that they are about to leave the site.  
	    	window.event.cancelBubble = true;
			window.event.returnValue = false;
		} 
		
        this.selectedIndex = index;  
        this.highlightSelected(index);
        if (this.entries && this.entries[index] && this.entries[index].callback) {
            this.entries[index].callback();
        }
    },
    
    highlightSelected: function(selectedIndex) {
        this.selectedIndex = selectedIndex;
        if (!this.entriesNode || !this.entriesNode.childNodes) {
            return;
        }
        for (var i = 0; i < this.entriesNode.childNodes.length; i++) {
            if (i == selectedIndex) {
                dojo.addClass(this.entriesNode.childNodes[i], "selected");
            } else {
                dojo.removeClass(this.entriesNode.childNodes[i], "selected");
            }
        }
    },
    
    addShowAllOption: function() {
		this._showAllOption = document.createElement("input");
        this._showAllOption.type = "checkbox";
		this._showAllOption.setAttribute("tabindex", "0");
		this._showAllOption.id = "showAllCheckbox";
        this._showAllOption.onclick = dojo.hitch(this, "toggleShowAll");
		
        var ul = document.createElement("ul");
        dojo.addClass(ul, "entries");        
        var li = document.createElement("li");
		var inputLabel = document.createElement("label");
		inputLabel.innerHTML = Constants.getMessage("labelShowAllSections");
		inputLabel.setAttribute("for", "showAllCheckbox");
        li.appendChild(this._showAllOption);
        li.appendChild(inputLabel);
        dojo.addClass(li, "show-all-option");
        ul.appendChild(li);
        
        this.containerNode.appendChild(ul);
		
		// fixing defect 9893
		this._showAllOption.checked = this.showAll;
    },
    
    toggleShowAll: function(event) {
        this.showAll = !this.showAll;
        this.showAllCallback(this.showAll, this.entries[this.selectedIndex].tag);
        dojo.publish("com-ibm-asq.common-CollapsibleSection-sectionSizeChanged", null);
    }
});

})();
