/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Summary:
//    InsertAttachmentPlugin extends dojo's dijit._editor._Plugin to provide attachment enhancements 
//     for rich text editing across the QM application.  
//       

InsertAttachPlugin = dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachPlugin");
InsertAttachPlugin.PLUGIN_ID = 'insertattach';

// require the editor plugins to be used...
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._editor._Plugin");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachDialog");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");

(function() {
	
	var Constants = com.ibm.asq.common.web.client.internal.Constants;

	var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    var DOWNLOAD_SERVICE_URI = "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/";
	
    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachPlugin", [dijit._editor._Plugin], {
        buttonClass: dijit.form.DropDownButton,
        useDefaultCommand: false,
        
        //@Overridden from dijit._editor._Plugin
        constructor: function(){
            com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachPlugin.superclass.constructor.apply(this, arguments);
        },
        
        //@Overridden from dijit._editor._Plugin
        postCreate: function(args){
            com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachPlugin.superclass.postCreate.apply(this, arguments);
        },
        
        _initButton: function(){
            var _this = this;
            this.tag = this.command == InsertAttachPlugin.PLUGIN_ID ? 'inserthtml' : 'a';
            this.command = 'inserthtml';
            var toolTipDialog = (this.toolTipDialog = new dijit.TooltipDialog({
                title: Constants.getMessage("InsertAttachment"),
                onOpen: function(){
                    _this._onOpenDialog();
                    dijit.TooltipDialog.prototype.onOpen.apply(this, arguments);
					//remove the 'V'
					dojo.removeClass(this.domNode,"dijitTooltipConnector");
					dojo.removeClass(this.domNode,"dijitTooltipBelow");
					dojo.removeClass(this.domNode,"dijitTooltipAbove");
					dojo.removeClass(this.containerNode,"dijitTooltipFocusNode");
                },
                onCancel: function(){
                    setTimeout(dojo.hitch(_this, "_onCloseDialog"), 0);
                },
                onClose: dojo.hitch(this, "_onCloseDialog")
            }));
            
            this.button = new this.buttonClass({
                label: Constants.getMessage("InsertAttachment"),
                showLabel: false,
                dropDown: this.toolTipDialog,
                iconClass: "AttachmentIcon AttachmentIconPos",
                tabIndex: "-1"
            });
            
            this.attachWidget = new com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachDialog();
            this.attachWidget.initialize({
				parent: this,
				isPreviewMode: false,
                isNonEditableMode: false
            });
            toolTipDialog.setContent(this.attachWidget.domNode);
			toolTipDialog.domNode.style.overflow="auto";
			toolTipDialog.startup();
            console.debug("InsertAttachment init");
        },
        
        setToolbar: function(){
            this.inherited("setToolbar", arguments);
        },
        
        setValue: function(args){
            //TODO: prevent closing popup if the text is empty
            this._onCloseDialog();
            if (dojo.isIE) {
                var a = dojo.withGlobal(this.editor.window, "getAncestorElement", dijit._editor.selection, [this.tag]);
                if (a) {
                    dojo.withGlobal(this.editor.window, "selectElement", dijit._editor.selection, [a]);
                }
            }
            var template = '';
				if(args.attaches){
					var src = TeamServerClient._getServicePrefix() + DOWNLOAD_SERVICE_URI;
					for(var i=0; i<args.attaches.length; i++){
						template = '<a href="' + src +args.attaches[i].uuid + '">'+args.attaches[i].fileName+'</a>';
					}
				}
            this.editor.execCommand('inserthtml', template);
        },
        
		getAttachments: function(){
			return this.attachments;	
		},
		
        _onCloseDialog: function(){
            // FIXME: IE is really messed up here!!
            if (dojo.isIE) {
                if ("_savedSelection" in this) {
                    var b = this._savedSelection;
                    delete this._savedSelection;
                    this.editor.focus();
                    var range = this.editor.document.selection.createRange();
                    range.moveToBookmark(b);
                    range.select();
                }
            }
            else {
                this.editor.focus();
            }
        },
        
        _onOpenDialog: function(){
            var a = dojo.withGlobal(this.editor.window, "getAncestorElement", dijit._editor.selection, [this.tag]);
            var url, text;
            // FIXME: IE is *really* b0rken
            if (dojo.isIE) {
                this._savedSelection = this.editor.document.selection.createRange().getBookmark();
            }
        }
    });
    
})();

dojo.subscribe("dijit.Editor.getPlugin", null, function(o){
    if (o.plugin) {
        return;
    }
    switch (o.args.name) {
        case InsertAttachPlugin.PLUGIN_ID:
            o.plugin = new com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachPlugin({
                command: o.args.name
            });
    }
});
