/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The default TableViewer's sorter object. Client can override the standard
 //    behavior of the sorter, which can only act on the content provider provided
 //    information to perform sorting.
 // 
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter", null,
{
    sortBy: function(viewer, objects, column, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects);
        
        this.viewer = viewer;
        this.groupCol = -1;
        this.sortCol = column;
        this.sortDir = sortDir;
        sortContainer.sort(dojo.hitch(this, "_sortComparator"));
        
        var collate = [];
        collate.push(sortContainer);
        return collate;
    },
    
    groupSortBy: function(viewer, objects, /*primary key (group by)*/column1, /*secondary key*/column2, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects); 
        
        this.viewer = viewer;
        this.groupCol = column1;
        this.sortCol = column2;
        this.sortDir = sortDir;
        sortContainer.sort(dojo.hitch(this, "_groupComparator"));
        
        var collate = [];
        var groupVal = null;
        var aGroup = null;
        
        // collate the objects into groups bins
        for(var i = 0; i < sortContainer.length; i++) {
            var val = this.viewer.getContentProvider().getCellRawData(sortContainer[i], this.groupCol) || this.viewer.getContentProvider().getCellDisplayData(sortContainer[i], this.groupCol);
            if(val != groupVal) {
                if(aGroup != null) {
                    collate.push(aGroup);
                    aGroup = null;
                }
                if(aGroup == null)
                    aGroup = [];
                groupVal = val;
            }
            aGroup.push(sortContainer[i]);
        }
        if(aGroup)
            collate.push(aGroup);
        return collate;
    },
        
    _groupComparator: function(obj1, obj2)
    {
        var val1 = this.viewer.getContentProvider().getCellRawData(obj1, this.groupCol);   
        var val2 = this.viewer.getContentProvider().getCellRawData(obj2, this.groupCol);

        if (!val1 || !val2) {
            val1 = this.viewer.getContentProvider().getCellDisplayData(obj1, this.groupCol);   
            val2 = this.viewer.getContentProvider().getCellDisplayData(obj2, this.groupCol);
            
            if (val1 && val2) {
                // this is a time stamp type. Convert the timestamp value to a Date
                // object for the comparison.
                //
                if (this.viewer.getContentProvider().getCellDisplayType(obj1, this.groupCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                    var date1 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val1);
                    val1 = date1.getTime();
                }
                if (this.viewer.getContentProvider().getCellDisplayType(obj2, this.groupCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                    var date2 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val2);
                    val2 = date2.getTime();
                }
            }
        }
        if (val1 && dojo.isString(val1)) {
            val1 = val1.toLowerCase();
        }
        if (val2 && dojo.isString(val1)) {
            val2 = val2.toLowerCase();
        }

        // same group, try secondary sorting.
        if(val1 == val2) {
            // there's a secondary sort and it's not the same as the group
            // sort column, sort by it. 
            if(this.sortCol > -1 && this.groupCol != this.sortCol)
                return this._sortComparator(obj1, obj2);
                
            return 0;
        }
        return (val1 > val2) ? 1 : -1;
    },
    
    _sortComparator: function(obj1, obj2)
    {
        var val1 = this.viewer.getContentProvider().getCellRawData(obj1, this.sortCol);   
        var val2 = this.viewer.getContentProvider().getCellRawData(obj2, this.sortCol);

        if (!val1 || !val2) {
            val1 = this.viewer.getContentProvider().getCellDisplayData(obj1, this.sortCol);   
            val2 = this.viewer.getContentProvider().getCellDisplayData(obj2, this.sortCol);
            
            if (val1 && val2) {
                // this is a time stamp type. Convert the timestamp value to a Date
                // object for the comparison.
                //
                if (this.viewer.getContentProvider().getCellDisplayType(obj1, this.sortCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                    var date1 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val1);
                    val1 = date1.getTime();
                }
                if (this.viewer.getContentProvider().getCellDisplayType(obj2, this.sortCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                    var date2 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val2);
                    val2 = date2.getTime();
                }
            }
        }
        if (val1 && dojo.isString(val1)) {
            val1 = val1.toLowerCase();
        }
        if (val2 && dojo.isString(val1)) {
            val2 = val2.toLowerCase();
        }
        
        if(val1 == val2)
            return 0;
        
        if(this.sortDir == TableConstants.SORT_DIR_ASCEND) {
            return (val1 > val2) ? 1 : -1;
        }
        else {
            return (val1 > val2) ? -1 : 1;            
        }
    }
});

})();
